@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: mapdrawgdal.c 7418 2008-02-29 00:02:49Z nsavard $
+ * $Id: mapdrawgdal.c 9496 2009-10-20 03:26:07Z warmerdam $
  *
  * Project:  MapServer
  * Purpose:  Code for drawing GDAL raster layers.  Called from 
@@ -33,7 +33,7 @@
 #include "mapresample.h"
 #include "mapthread.h"
 
-MS_CVSID("$Id: mapdrawgdal.c 7418 2008-02-29 00:02:49Z nsavard $")
+MS_CVSID("$Id: mapdrawgdal.c 9496 2009-10-20 03:26:07Z warmerdam $")
 
 extern int InvGeoTransform( double *gt_in, double *gt_out );
 
@@ -54,10 +54,11 @@ extern int InvGeoTransform( double *gt_in, double *gt_out );
 #endif
 
 static int
-LoadGDALImage( GDALRasterBandH hBand, int iColorIndex, 
-               layerObj *layer, 
-               int src_xoff, int src_yoff, int src_xsize, int src_ysize, 
-               GByte *pabyBuffer, int dst_xsize, int dst_ysize );
+LoadGDALImages( GDALDatasetH hDS, int band_numbers[4], int band_count,
+		layerObj *layer, 
+		int src_xoff, int src_yoff, int src_xsize, int src_ysize, 
+		GByte *pabyBuffer,
+		int dst_xsize, int dst_ysize );
 static int 
 msDrawRasterLayerGDAL_RawMode(
     mapObj *map, layerObj *layer, imageObj *image, GDALDatasetH hDS, 
@@ -118,11 +119,11 @@ int msDrawRasterLayerGDAL(mapObj *map, layerObj *layer, imageObj *image,
                 *pabyRawAlpha = NULL;
   int truecolor = FALSE, classified = FALSE;
   int red_band=0, green_band=0, blue_band=0, alpha_band=0, cmt=0;
+  int band_count, band_numbers[4];
   gdImagePtr gdImg = NULL;
   GDALDatasetH hDS = hDSVoid;
   GDALColorTableH hColorMap;
-  GDALRasterBandH hBand1, hBand2, hBand3, hBandAlpha;
-
+  GDALRasterBandH hBand1=NULL, hBand2=NULL, hBand3=NULL, hBandAlpha=NULL;
   memset( cmap, 0xff, MAXCOLORS * sizeof(int) );
 
 /* -------------------------------------------------------------------- */
@@ -176,9 +177,10 @@ int msDrawRasterLayerGDAL(mapObj *map, layerObj *layer, imageObj *image,
               CSLFetchNameValue( layer->processing, "RAW_WINDOW" ) );
       
       if( layer->debug )
-          msDebug( "msDrawGDAL(%s): using RAW_WINDOW=%s\n",
+          msDebug( "msDrawGDAL(%s): using RAW_WINDOW=%s, dst=0,0,%d,%d\n",
                    layer->name,
-                   CSLFetchNameValue( layer->processing, "RAW_WINDOW" ) );
+                   CSLFetchNameValue( layer->processing, "RAW_WINDOW" ),
+                   image->width, image->height );
 
       if( CSLCount(papszTokens) != 4 )
       {
@@ -242,8 +244,8 @@ int msDrawRasterLayerGDAL(mapObj *map, layerObj *layer, imageObj *image,
       urx = GEO_TRANS(adfInvGeoTransform+0,copyRect.maxx,copyRect.maxy);
       ury = GEO_TRANS(adfInvGeoTransform+3,copyRect.maxx,copyRect.maxy);
       
-      src_xoff = MAX(0,(int) llx);
-      src_yoff = MAX(0,(int) ury);
+      src_xoff = MAX(0,(int) floor(llx+0.5));
+      src_yoff = MAX(0,(int) floor(ury+0.5));
       src_xsize = MIN(MAX(0,(int) (urx - llx + 0.5)),
                       GDALGetRasterXSize(hDS) - src_xoff);
       src_ysize = MIN(MAX(0,(int) (lly - ury + 0.5)),
@@ -274,11 +276,25 @@ int msDrawRasterLayerGDAL(mapObj *map, layerObj *layer, imageObj *image,
           return 0;
       }
 
-#ifndef notdef
       if( layer->debug )
           msDebug( "msDrawGDAL(): src=%d,%d,%d,%d, dst=%d,%d,%d,%d\n", 
                    src_xoff, src_yoff, src_xsize, src_ysize, 
                    dst_xoff, dst_yoff, dst_xsize, dst_ysize );
+#ifndef notdef
+      if( layer->debug )
+      {
+          double d_src_xoff, d_src_yoff, geo_x, geo_y;
+
+          geo_x = mapRect.minx + dst_xoff * map->cellsize;
+          geo_y = mapRect.maxy - dst_yoff * map->cellsize;
+
+          d_src_xoff = (geo_x - adfGeoTransform[0]) / adfGeoTransform[1];
+          d_src_yoff = (geo_y - adfGeoTransform[3]) / adfGeoTransform[5];
+          
+          msDebug( "source raster PL (%.3f,%.3f) for dst PL (%d,%d).\n",
+                   d_src_xoff, d_src_yoff,
+                   dst_xoff, dst_yoff );
+      }
 #endif
   }
 
@@ -372,7 +388,7 @@ int msDrawRasterLayerGDAL(mapObj *map, layerObj *layer, imageObj *image,
   }
   else
   {
-      int band_count, *band_list;
+      int *band_list;
 
       band_list = msGetGDALBandList( layer, hDS, 4, &band_count );
       if( band_list == NULL )
@@ -401,6 +417,26 @@ int msDrawRasterLayerGDAL(mapObj *map, layerObj *layer, imageObj *image,
       free( band_list );
   }
 
+  band_numbers[0] = red_band;
+  band_numbers[1] = green_band;
+  band_numbers[2] = blue_band;
+  band_numbers[3] = 0;
+  
+  if( blue_band != 0 && alpha_band != 0 )
+  {
+      band_numbers[3] = alpha_band;
+      band_count = 4;
+  }
+  else if( blue_band != 0 && alpha_band == 0 )
+      band_count = 3;
+  else if( alpha_band != 0 )
+  {
+      band_numbers[1] = alpha_band;
+      band_count = 2;
+  }
+  else
+      band_count = 1;
+  
   if( layer->debug > 1 || (layer->debug > 0 && green_band != 0) )
   {
       msDebug( "msDrawGDAL(): red,green,blue,alpha bands = %d,%d,%d,%d\n", 
@@ -662,90 +698,40 @@ int msDrawRasterLayerGDAL(mapObj *map, layerObj *layer, imageObj *image,
   }
 
   /*
-   * Read the entire region of interest in one gulp.  GDAL will take
-   * care of downsampling and window logic.  
+   * Allocate imagery buffers.
    */
-  pabyRaw1 = (unsigned char *) malloc(dst_xsize * dst_ysize);
+  pabyRaw1 = (unsigned char *) malloc(dst_xsize * dst_ysize * band_count);
   if( pabyRaw1 == NULL )
   {
-      msSetError(MS_MEMERR, "Allocating work image of size %dx%d failed.",
-                 "msDrawRasterLayerGDAL()", dst_xsize, dst_ysize );
-      return -1;
-  }
-
-  if( LoadGDALImage( hBand1, 1, layer, 
-                     src_xoff, src_yoff, src_xsize, src_ysize, 
-                     pabyRaw1, dst_xsize, dst_ysize ) == -1 )
-  {
-      free( pabyRaw1 );
+      msSetError(MS_MEMERR, "Allocating work image of size %dx%dx%d failed.",
+                 "msDrawRasterLayerGDAL()", dst_xsize, dst_ysize, band_count );
       return -1;
   }
 
   if( hBand2 != NULL && hBand3 != NULL )
   {
-      pabyRaw2 = (unsigned char *) malloc(dst_xsize * dst_ysize);
-      if( pabyRaw2 == NULL )
-      {
-          msSetError(MS_MEMERR, "Allocating work image of size %dx%d failed.",
-                     "msDrawRasterLayerGDAL()", dst_xsize, dst_ysize );
-          return -1;
-      }
-      
-      if( LoadGDALImage( hBand2, 2, layer, 
-                         src_xoff, src_yoff, src_xsize, src_ysize, 
-                         pabyRaw2, dst_xsize, dst_ysize ) == -1 )
-      {
-          free( pabyRaw1 );
-          free( pabyRaw2 );
-          return -1;
-      }
-
-      pabyRaw3 = (unsigned char *) malloc(dst_xsize * dst_ysize);
-      if( pabyRaw3 == NULL )
-      {
-          msSetError(MS_MEMERR, "Allocating work image of size %dx%d failed.",
-                     "msDrawRasterLayerGDAL()", dst_xsize, dst_ysize );
-          return -1;
-      }
-      
-      if( LoadGDALImage( hBand3, 3, layer, 
-                         src_xoff, src_yoff, src_xsize, src_ysize, 
-                         pabyRaw3, dst_xsize, dst_ysize ) == -1 )
-      {
-          free( pabyRaw1 );
-          free( pabyRaw2 );
-          free( pabyRaw3 );
-          return -1;
-      }
+      pabyRaw2 = pabyRaw1 + dst_xsize * dst_ysize * 1;
+      pabyRaw3 = pabyRaw1 + dst_xsize * dst_ysize * 2;
   }
 
   if( hBandAlpha != NULL )
   {
-      pabyRawAlpha = (unsigned char *) malloc(dst_xsize * dst_ysize);
-      if( pabyRawAlpha == NULL )
-      {
-          msSetError(MS_MEMERR, "Allocating work image of size %dx%d failed.",
-                     "msDrawRasterLayerGDAL()", dst_xsize, dst_ysize );
-          return -1;
-      }
-      
-      if( LoadGDALImage( hBandAlpha, 4, layer, 
-                         src_xoff, src_yoff, src_xsize, src_ysize, 
-                         pabyRawAlpha, dst_xsize, dst_ysize ) == -1 )
-      {
-          free( pabyRaw1 );
-          if( pabyRaw3 != NULL )
-          {
-              free( pabyRaw2 );
-              free( pabyRaw3 );
-          }
+      if( hBand2 != NULL )
+          pabyRawAlpha = pabyRaw1 + dst_xsize * dst_ysize * 3;
+      else
+          pabyRawAlpha = pabyRaw1 + dst_xsize * dst_ysize * 1;
+  }
 
-          free( pabyRawAlpha );
-          return -1;
-      }
+  /*
+   * Load image data into buffers with scaling, etc.
+   */
+  if( LoadGDALImages( hDS, band_numbers, band_count, layer, 
+		      src_xoff, src_yoff, src_xsize, src_ysize, 
+		      pabyRaw1, dst_xsize, dst_ysize ) == -1 )
+  {
+      free( pabyRaw1 );
+      return -1;
   }
-  else
-      pabyRawAlpha = NULL;
 
 /* -------------------------------------------------------------------- */
 /*      Single band plus colormap with alpha blending to 8bit.          */
@@ -972,15 +958,6 @@ int msDrawRasterLayerGDAL(mapObj *map, layerObj *layer, imageObj *image,
   ** Cleanup
   */
 
-  if( pabyRaw3 != NULL )
-  {
-      free( pabyRaw2 );
-      free( pabyRaw3 );
-  }
-
-  if( pabyRawAlpha != NULL )
-      free( pabyRawAlpha );
-
   free( pabyRaw1 );
 
   if( hColorMap != NULL )
@@ -1249,172 +1226,220 @@ static int ApplyLUT( int iColorIndex, layerObj *layer,
 }
 
 /************************************************************************/
-/*                           LoadGDALImage()                            */
+/*                           LoadGDALImages()                           */
 /*                                                                      */
-/*      This call will load and process one band of input for the       */
+/*      This call will load and process 1-4 bands of input for the      */
 /*      selected rectangle, loading the result into the passed 8bit     */
 /*      buffer.  The processing options include scaling.                */
 /************************************************************************/
 
 static int
-LoadGDALImage( GDALRasterBandH hBand, int iColorIndex,  layerObj *layer, 
-               int src_xoff, int src_yoff, int src_xsize, int src_ysize, 
-               GByte *pabyBuffer, int dst_xsize, int dst_ysize )
+LoadGDALImages( GDALDatasetH hDS, int band_numbers[4], int band_count,
+		layerObj *layer, 
+		int src_xoff, int src_yoff, int src_xsize, int src_ysize, 
+		GByte *pabyWholeBuffer,
+		int dst_xsize, int dst_ysize )
     
 {
+    int    iColorIndex, result_code=0;
     CPLErr eErr;
-    double dfScaleMin=0.0, dfScaleMax=255.0, dfScaleRatio, dfNoDataValue;
-    const char *pszScaleInfo;
-    float *pafRawData;
-    int   nPixelCount = dst_xsize * dst_ysize, i, bGotNoData = FALSE;
+    float *pafWholeRawData;
+
+/* -------------------------------------------------------------------- */
+/*      Are we doing a simple, non-scaling case?  If so, read directly  */
+/*      and return.                                                     */
+/* -------------------------------------------------------------------- */
+    if( CSLFetchNameValue( layer->processing, "SCALE" ) == NULL
+	&& CSLFetchNameValue( layer->processing, "SCALE_1" ) == NULL
+	&& CSLFetchNameValue( layer->processing, "SCALE_2" ) == NULL
+	&& CSLFetchNameValue( layer->processing, "SCALE_3" ) == NULL
+	&& CSLFetchNameValue( layer->processing, "SCALE_4" ) == NULL )
+    {
+        eErr = GDALDatasetRasterIO( hDS, GF_Read, 
+				    src_xoff, src_yoff, src_xsize, src_ysize, 
+				    pabyWholeBuffer, 
+				    dst_xsize, dst_ysize, GDT_Byte,
+				    band_count, band_numbers,
+				    0,0,0);
+
+        if( eErr != CE_None )
+	{
+            msSetError( MS_IOERR, "GDALDatasetRasterIO() failed: %s", 
+                        CPLGetLastErrorMsg(), "drawGDAL()" );
+	    return -1;
+	}
+
+	for( iColorIndex = 0; 
+	     iColorIndex < band_count && result_code == 0; iColorIndex++ )
+	{
+	    result_code = ApplyLUT( iColorIndex+1, layer, 
+				    pabyWholeBuffer 
+				    + dst_xsize*dst_ysize*iColorIndex, 
+				    dst_xsize, dst_ysize );
+	}
+	
+	return result_code;
+    }
+
+/* -------------------------------------------------------------------- */
+/*      We need to do some scaling.  Will load into either a 16bit      */
+/*      unsigned or a floating point buffer depending on the source     */
+/*      data.  We offer a special case for 16U data because it is       */
+/*      common and it is a substantial win to avoid alot of floating    */
+/*      point operations on it.                                         */
+/* -------------------------------------------------------------------- */
+    /* TODO */
+
+/* -------------------------------------------------------------------- */
+/*      Allocate the raw imagery buffer, and load into it (band         */
+/*      interleaved).                                                   */
+/* -------------------------------------------------------------------- */
+    pafWholeRawData = 
+        (float *) malloc(sizeof(float) * dst_xsize * dst_ysize * band_count );
+
+    if( pafWholeRawData == NULL )
+    {
+        msSetError(MS_MEMERR, 
+                   "Allocating work float image of size %dx%dx%d failed.",
+                   "msDrawRasterLayerGDAL()", 
+                   dst_xsize, dst_ysize, band_count );
+        return -1;
+    }
+	
+    eErr = GDALDatasetRasterIO( 
+        hDS, GF_Read, 
+        src_xoff, src_yoff, src_xsize, src_ysize, 
+        pafWholeRawData, dst_xsize, dst_ysize, GDT_Float32, 
+        band_count, band_numbers,
+        0, 0, 0 );
+    
+    if( eErr != CE_None )
+    {
+        msSetError( MS_IOERR, "GDALDatasetRasterIO() failed: %s", 
+                    CPLGetLastErrorMsg(), "drawGDAL()" );
+        
+        free( pafWholeRawData );
+        return -1;
+    }
 
 /* -------------------------------------------------------------------- */
 /*      Fetch the scale processing option.                              */
 /* -------------------------------------------------------------------- */
-    pszScaleInfo = CSLFetchNameValue( layer->processing, "SCALE" );
-    if( pszScaleInfo == NULL )
+    for( iColorIndex = 0; iColorIndex < band_count; iColorIndex++ )
     {
-        char szBandScalingName[20];
-
-        sprintf( szBandScalingName, "SCALE_%d", iColorIndex );
-        pszScaleInfo = CSLFetchNameValue( layer->processing, 
-                                          szBandScalingName);
-    }
-
-    if( pszScaleInfo != NULL )
-    {
-        char **papszTokens;
-
-        papszTokens = CSLTokenizeStringComplex( pszScaleInfo, " ,", 
-                                                FALSE, FALSE );
-        if( CSLCount(papszTokens) == 1
-            && EQUAL(papszTokens[0],"AUTO") )
+	unsigned char *pabyBuffer;
+	double dfScaleMin=0.0, dfScaleMax=255.0, dfScaleRatio, dfNoDataValue;
+	const char *pszScaleInfo;
+	float *pafRawData;
+	int   nPixelCount = dst_xsize * dst_ysize, i, bGotNoData = FALSE;
+	GDALRasterBandH hBand =GDALGetRasterBand(hDS,band_numbers[iColorIndex]);
+	pszScaleInfo = CSLFetchNameValue( layer->processing, "SCALE" );
+	if( pszScaleInfo == NULL )
         {
-            dfScaleMin = dfScaleMax = 0.0;
+	    char szBandScalingName[20];
+	    
+	    sprintf( szBandScalingName, "SCALE_%d", iColorIndex+1 );
+	    pszScaleInfo = CSLFetchNameValue( layer->processing, 
+					      szBandScalingName);
         }
-        else if( CSLCount(papszTokens) != 2 )
+	
+	if( pszScaleInfo != NULL )
         {
-            msSetError( MS_MISCERR, 
-                        "SCALE PROCESSING option unparsable for layer %s.",
-                        layer->name, 
-                        "msDrawGDAL()" );
-            return -1;
+	    char **papszTokens;
+	    
+	    papszTokens = CSLTokenizeStringComplex( pszScaleInfo, " ,", 
+						    FALSE, FALSE );
+	    if( CSLCount(papszTokens) == 1
+		&& EQUAL(papszTokens[0],"AUTO") )
+            {
+		dfScaleMin = dfScaleMax = 0.0;
+            }
+	    else if( CSLCount(papszTokens) != 2 )
+            {
+                free( pafWholeRawData );
+		msSetError( MS_MISCERR, 
+			    "SCALE PROCESSING option unparsable for layer %s.",
+			    layer->name, 
+			    "msDrawGDAL()" );
+		return -1;
+            }
+	    else
+            {
+		dfScaleMin = atof(papszTokens[0]);
+		dfScaleMax = atof(papszTokens[1]);
+            }
+	    CSLDestroy( papszTokens );
         }
-        else
-        {
-            dfScaleMin = atof(papszTokens[0]);
-            dfScaleMax = atof(papszTokens[1]);
-        }
-        CSLDestroy( papszTokens );
-    }
     
-/* -------------------------------------------------------------------- */
-/*      If we have no scaling parameters, just truncate to 8bit.        */
-/* -------------------------------------------------------------------- */
-    if( pszScaleInfo == NULL )
-    {
-        eErr = GDALRasterIO( hBand, GF_Read, 
-                             src_xoff, src_yoff, src_xsize, src_ysize, 
-                             pabyBuffer, dst_xsize, dst_ysize, GDT_Byte, 0,0);
-
-        if( eErr != CE_None )
-            msSetError( MS_IOERR, "GDALRasterIO() failed: %s", 
-                        CPLGetLastErrorMsg(), "drawGDAL()" );
-
-        if( eErr == CE_None )
-        {
-            return ApplyLUT( iColorIndex, layer, 
-                             pabyBuffer, dst_xsize, dst_ysize );;
-        }
-        else
-            return -1;
-    }
-
-/* -------------------------------------------------------------------- */
-/*      Otherwise, allocate a temporary floating point buffer, and      */
-/*      load into that.                                                 */
-/* -------------------------------------------------------------------- */
-    pafRawData = (float *) malloc(sizeof(float) * dst_xsize * dst_ysize );
-
-    if( pafRawData == NULL )
-    {
-        msSetError(MS_MEMERR, 
-                   "Allocating work float image of size %dx%d failed.",
-                   "msDrawRasterLayerGDAL()", dst_xsize, dst_ysize );
-        return -1;
-    }
-
-    eErr = GDALRasterIO( hBand, GF_Read, 
-                         src_xoff, src_yoff, src_xsize, src_ysize, 
-                         pafRawData, dst_xsize, dst_ysize, GDT_Float32, 0, 0 );
-    
-    if( eErr != CE_None )
-    {
-        msSetError( MS_IOERR, "GDALRasterIO() failed: %s", 
-                    CPLGetLastErrorMsg(), "drawGDAL()" );
-
-        free( pafRawData );
-        return -1;
-    }
-
 /* -------------------------------------------------------------------- */
 /*      If we are using autoscaling, then compute the max and min       */
 /*      now.  Perhaps we should eventually honour the offsite value     */
 /*      as a nodata value, or get it from GDAL.                         */
 /* -------------------------------------------------------------------- */
-    if( dfScaleMin == dfScaleMax )
-    {
-        dfScaleMin = dfScaleMax = pafRawData[0];
+	pafRawData = pafWholeRawData + iColorIndex * dst_xsize * dst_ysize;
 
-        for( i = 1; i < nPixelCount; i++ )
+	if( dfScaleMin == dfScaleMax )
         {
-            dfScaleMin = MIN(dfScaleMin,pafRawData[i]);
-            dfScaleMax = MAX(dfScaleMax,pafRawData[i]);
+	    dfScaleMin = dfScaleMax = pafRawData[0];
+	    
+	    for( i = 1; i < nPixelCount; i++ )
+            {
+		dfScaleMin = MIN(dfScaleMin,pafRawData[i]);
+		dfScaleMax = MAX(dfScaleMax,pafRawData[i]);
+            }
+	    if( dfScaleMin == dfScaleMax )
+                dfScaleMax = dfScaleMin + 1.0;
         }
-        if( dfScaleMin == dfScaleMax )
-            dfScaleMax = dfScaleMin + 1.0;
-    }
-
-    if( layer->debug > 0 )
-        msDebug( "msDrawGDAL(%s): scaling to 8bit, src range=%g,%g\n",
-                 layer->name, dfScaleMin, dfScaleMax );
-
+	
+	if( layer->debug > 0 )
+            msDebug( "msDrawGDAL(%s): scaling to 8bit, src range=%g,%g\n",
+                     layer->name, dfScaleMin, dfScaleMax );
+	
 /* -------------------------------------------------------------------- */
 /*      Now process the data.                                           */
 /* -------------------------------------------------------------------- */
-    dfScaleRatio = 256.0 / (dfScaleMax - dfScaleMin);
-
-    for( i = 0; i < nPixelCount; i++ )
-    {
-        float fScaledValue = (float) ((pafRawData[i]-dfScaleMin)*dfScaleRatio);
-
-        if( fScaledValue < 0.0 )
-            pabyBuffer[i] = 0;
-        else if( fScaledValue > 255.0 )
-            pabyBuffer[i] = 255;
-        else
-            pabyBuffer[i] = (int) fScaledValue;
-    }
-
-    free( pafRawData );
+	dfScaleRatio = 256.0 / (dfScaleMax - dfScaleMin);
+	pabyBuffer = pabyWholeBuffer + iColorIndex * nPixelCount;
 
+	for( i = 0; i < nPixelCount; i++ )
+        {
+	    float fScaledValue = (float) ((pafRawData[i]-dfScaleMin)*dfScaleRatio);
+	    
+	    if( fScaledValue < 0.0 )
+                pabyBuffer[i] = 0;
+	    else if( fScaledValue > 255.0 )
+                pabyBuffer[i] = 255;
+	    else
+                pabyBuffer[i] = (int) fScaledValue;
+        }
+	
 /* -------------------------------------------------------------------- */
 /*      Report a warning if NODATA keyword was applied.  We are         */
 /*      unable to utilize it since we can't return any pixels marked    */
 /*      as nodata from this function.  Need to fix someday.             */
 /* -------------------------------------------------------------------- */
-    dfNoDataValue = msGetGDALNoDataValue( layer, hBand, &bGotNoData );
-    if( bGotNoData )
-        msDebug( "LoadGDALImage(%s): NODATA value %g in GDAL\n"
-                 "file or PROCESSING directive ignored.  Not yet supported for\n"
-                 "unclassified scaled data.\n",
-                 layer->name, dfNoDataValue );
-
+	dfNoDataValue = msGetGDALNoDataValue( layer, hBand, &bGotNoData );
+	if( bGotNoData )
+            msDebug( "LoadGDALImage(%s): NODATA value %g in GDAL\n"
+                     "file or PROCESSING directive ignored.  Not yet supported for\n"
+                     "unclassified scaled data.\n",
+                     layer->name, dfNoDataValue );
+	
 /* -------------------------------------------------------------------- */
 /*      Apply LUT if there is one.                                      */
 /* -------------------------------------------------------------------- */
-    return ApplyLUT( iColorIndex, layer, 
-                             pabyBuffer, dst_xsize, dst_ysize );;
+	result_code = ApplyLUT( iColorIndex+1, layer, 
+				pabyBuffer, dst_xsize, dst_ysize );;
+	if( result_code == -1 )
+        {
+            free( pafWholeRawData );
+            return result_code;
+        }
+    }
+
+    free( pafWholeRawData );
+
+    return result_code;
 }
 
 /************************************************************************/
@@ -1860,6 +1885,7 @@ msDrawRasterLayerGDAL_16BitClassification(
     GDALDataType eDataType;
     float fDataMin=0.0, fDataMax=255.0, fNoDataValue;
     const char *pszScaleInfo;
+    const char *pszBuckets;
     int  bUseIntegers = FALSE;
     int  *cmap, c, j, k, bGotNoData = FALSE, bGotFirstValue;
     gdImagePtr gdImg = image->img.gd;
@@ -1924,6 +1950,7 @@ msDrawRasterLayerGDAL_16BitClassification(
 /* -------------------------------------------------------------------- */
 /*      Fetch the scale processing option.                              */
 /* -------------------------------------------------------------------- */
+    pszBuckets = CSLFetchNameValue( layer->processing, "SCALE_BUCKETS" );
     pszScaleInfo = CSLFetchNameValue( layer->processing, "SCALE" );
 
     if( pszScaleInfo != NULL )
@@ -1963,10 +1990,17 @@ msDrawRasterLayerGDAL_16BitClassification(
     if( eDataType == GDT_Byte || eDataType == GDT_Int16 
         || eDataType == GDT_UInt16 )
     {
-        dfScaleMin = fDataMin - 0.5;
-        dfScaleMax = fDataMax + 0.5;
-        nBucketCount = (int) floor(fDataMax - fDataMin + 1.1);
-        bUseIntegers = TRUE;
+        if( pszScaleInfo == NULL )
+        {
+            dfScaleMin = fDataMin - 0.5;
+            dfScaleMax = fDataMax + 0.5;
+        }
+
+        if( pszBuckets == NULL )
+        {
+            nBucketCount = (int) floor(fDataMax - fDataMin + 1.1);
+            bUseIntegers = TRUE;
+        }
     }
 
 /* -------------------------------------------------------------------- */
@@ -1982,25 +2016,22 @@ msDrawRasterLayerGDAL_16BitClassification(
 /* -------------------------------------------------------------------- */
 /*      How many buckets?  Only set if we don't already have a value.   */
 /* -------------------------------------------------------------------- */
-    if( nBucketCount == 0 )
+    if( pszBuckets == NULL )
     {
-        const char *pszBuckets;
-
-        pszBuckets = CSLFetchNameValue( layer->processing, "SCALE_BUCKETS" );
-        if( pszBuckets == NULL )
+        if( nBucketCount == 0 )
             nBucketCount = 65536;
-        else
+    }
+    else
+    {
+        nBucketCount = atoi(pszBuckets);
+        if( nBucketCount < 2 )
         {
-            nBucketCount = atoi(pszBuckets);
-            if( nBucketCount < 2 )
-            {
-                free( pafRawData );
-                msSetError( MS_MISCERR, 
-                            "SCALE_BUCKETS PROCESSING option is not a value of 2 or more: %s.",
-                            pszBuckets, 
-                            "msDrawRasterLayerGDAL_16BitClassification()" );
-                return -1;
-            }
+            free( pafRawData );
+            msSetError( MS_MISCERR, 
+                        "SCALE_BUCKETS PROCESSING option is not a value of 2 or more: %s.",
+                        pszBuckets, 
+                        "msDrawRasterLayerGDAL_16BitClassification()" );
+            return -1;
         }
     }
 
