@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: mapfile.h 8540 2009-02-12 20:20:12Z tamas $
+ * $Id: mapfile.h 9194 2009-07-21 15:49:45Z aboudreault $
  *
  * Project:  MapServer
  * Purpose:  Keyword definitions for the mapfiles and symbolfiles.
@@ -282,4 +282,10 @@ enum MS_TOKEN_SOURCES {MS_FILE_TOKENS=0, MS_STRING_TOKENS, MS_URL_TOKENS};
 /* rfc48 geometry transforms */
 #define GEOMTRANSFORM 1220
 
+/* rfc55 output resolution */
+#define DEFRESOLUTION 1221
+
+/* label repeat enhancement */
+#define REPEATDISTANCE 1222
+
 #endif /* MAPFILE_H */
