@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: mapgml.c 7736 2008-06-24 03:54:41Z tomkralidis $
+ * $Id: mapgml.c 9455 2009-10-14 04:42:49Z dmorissette $
  *
  * Project:  MapServer
  * Purpose:  shapeObj to GML output via MapServer queries.
@@ -31,7 +31,7 @@
 #include "maperror.h"
 #include "mapgml.h"
 
-MS_CVSID("$Id: mapgml.c 7736 2008-06-24 03:54:41Z tomkralidis $")
+MS_CVSID("$Id: mapgml.c 9455 2009-10-14 04:42:49Z dmorissette $")
 
 /* Use only mapgml.c if WMS or WFS is available */
 
@@ -1205,6 +1205,7 @@ static void msGMLWriteGroup(FILE *stream, gmlGroupObj *group, shapeObj *shape, g
 }
 #endif
 
+/* Dump GML query results for WMS GetFeatureInfo */
 int msGMLWriteQuery(mapObj *map, char *filename, const char *namespaces)
 {
 #if defined(USE_WMS_SVR)
@@ -1215,6 +1216,7 @@ int msGMLWriteQuery(mapObj *map, char *filename, const char *namespaces)
   FILE *stream=stdout; /* defaults to stdout */
   char szPath[MS_MAXPATHLEN];
   char *value;
+  const char *pszMapSRS = NULL;
 
   gmlGroupListObj *groupList=NULL;
   gmlItemListObj *itemList=NULL;
@@ -1248,12 +1250,27 @@ int msGMLWriteQuery(mapObj *map, char *filename, const char *namespaces)
   /* a schema *should* be required */
   msOWSPrintEncodeMetadata(stream, &(map->web.metadata), namespaces, "description", OWS_NOERR, "\t<gml:description>%s</gml:description>\n", NULL);
 
+  /* Look up map SRS. We need an EPSG code for GML, if not then we get null and we'll fall back on the layer's SRS */
+  pszMapSRS = msOWSGetEPSGProj(&(map->projection), NULL, namespaces, MS_TRUE);
+
   /* step through the layers looking for query results */
   for(i=0; i<map->numlayers; i++) {
+    const char *pszOutputSRS = NULL;
     lp = (GET_LAYER(map, map->layerorder[i]));
 
     if(lp->dump == MS_TRUE && lp->resultcache && lp->resultcache->numresults > 0) { /* found results */
 
+#ifdef USE_PROJ
+      /* Determine output SRS, if map has none, then try using layer's native SRS */
+      if ((pszOutputSRS = pszMapSRS) == NULL) {
+          pszOutputSRS = msOWSGetEPSGProj(&(lp->projection), NULL, namespaces, MS_TRUE);
+          if (pszOutputSRS == NULL) {
+              msSetError(MS_WMSERR, "No valid EPSG code in map or layer projection for GML output", "msGMLWriteQuery()");
+              continue;  /* No EPSG code, cannot output this layer */
+           }
+      }
+#endif
+
       /* start this collection (layer) */
       /* if no layer name provided fall back on the layer name + "_layer" */
       value = (char*) malloc(strlen(lp->name)+7);
@@ -1261,27 +1278,19 @@ int msGMLWriteQuery(mapObj *map, char *filename, const char *namespaces)
       msOWSPrintValidateMetadata(stream, &(lp->metadata), namespaces, "layername", OWS_NOERR, "\t<%s>\n", value);
       msFree(value);
 
-      /* actually open the layer */
-      status = msLayerOpen(lp);
-      if(status != MS_SUCCESS) return(status);
-
-      /* retrieve all the item names */
-      status = msLayerGetItems(lp);
-      if(status != MS_SUCCESS) return(status);
-
       /* populate item and group metadata structures (TODO: test for NULLs here, shouldn't happen) */
-      itemList = msGMLGetItems(lp, "G");
-      constantList = msGMLGetConstants(lp, "G");
-      groupList = msGMLGetGroups(lp, "G");      
-      geometryList = msGMLGetGeometries(lp, "G");
+      itemList = msGMLGetItems(lp, namespaces);
+      constantList = msGMLGetConstants(lp, namespaces);
+      groupList = msGMLGetGroups(lp, namespaces);
+      geometryList = msGMLGetGeometries(lp, namespaces);
 
       for(j=0; j<lp->resultcache->numresults; j++) {
-        status = msLayerGetShape(lp, &shape, lp->resultcache->results[j].tileindex, lp->resultcache->results[j].shapeindex);
+        status = msLayerResultsGetShape(lp, &shape, lp->resultcache->results[j].tileindex, lp->resultcache->results[j].shapeindex);
         if(status != MS_SUCCESS) return(status);
 
 #ifdef USE_PROJ
         /* project the shape into the map projection (if necessary), note that this projects the bounds as well */
-        if(msProjectionsDiffer(&(lp->projection), &(map->projection)))
+        if(pszOutputSRS == pszMapSRS && msProjectionsDiffer(&(lp->projection), &(map->projection)))
           msProjectShape(&lp->projection, &map->projection, &shape);
 #endif
 
@@ -1292,20 +1301,18 @@ int msGMLWriteQuery(mapObj *map, char *filename, const char *namespaces)
         msOWSPrintValidateMetadata(stream, &(lp->metadata), namespaces, "featurename", OWS_NOERR, "\t\t<%s>\n", value);
         msFree(value);
 
-        /* write the feature geometry and bounding box */
-        if(!(geometryList && geometryList->numgeometries == 1 && strcasecmp(geometryList->geometries[0].name, "none") == 0)) {
+        /* Write the feature geometry and bounding box unless 'none' was requested. */
+        /* Default to bbox only if nothing specified and output full geometry only if explicitly requested */
+        if(!(geometryList && geometryList->numgeometries == 1 && strcasecmp(geometryList->geometries[0].name, "none") == 0)) { 
 
 #ifdef USE_PROJ
-          if(msOWSGetEPSGProj(&(map->projection), &(map->web.metadata), namespaces, MS_TRUE)) { /* use the map projection first */
-            gmlWriteBounds(stream, OWS_GML2, &(shape.bounds), msOWSGetEPSGProj(&(map->projection), NULL, namespaces, MS_TRUE), "\t\t\t");
-            gmlWriteGeometry(stream, NULL, OWS_GML2, &(shape), msOWSGetEPSGProj(&(map->projection), &(map->web.metadata), namespaces, MS_TRUE), NULL, "\t\t\t");
-          } else { /* then use the layer projection and/or metadata */
-            gmlWriteBounds(stream, OWS_GML2, &(shape.bounds), msOWSGetEPSGProj(&(map->projection), &(map->web.metadata), namespaces, MS_TRUE), "\t\t\t");
-            gmlWriteGeometry(stream, NULL, OWS_GML2, &(shape), msOWSGetEPSGProj(&(lp->projection), &(lp->metadata), namespaces, MS_TRUE), NULL, "\t\t\t");
-          }
+          gmlWriteBounds(stream, OWS_GML2, &(shape.bounds), pszOutputSRS, "\t\t\t");
+          if (geometryList && geometryList->numgeometries > 0 )
+              gmlWriteGeometry(stream, geometryList, OWS_GML2, &(shape), pszOutputSRS, NULL, "\t\t\t");
 #else
           gmlWriteBounds(stream, OWS_GML2, &(shape.bounds), NULL, "\t\t\t"); /* no projection information */
-          gmlWriteGeometry(stream, NULL, OWS_GML2, &(shape), NULL, NULL, "\t\t\t");
+          if (geometryList && geometryList->numgeometries > 0 )
+            gmlWriteGeometry(stream, geometryList, OWS_GML2, &(shape), NULL, NULL, "\t\t\t");
 #endif
 
         }
@@ -1351,7 +1358,7 @@ int msGMLWriteQuery(mapObj *map, char *filename, const char *namespaces)
       msGMLFreeItems(itemList);
       msGMLFreeGeometries(geometryList);
 
-      msLayerClose(lp);
+      /* msLayerClose(lp); */
     }
   } /* next layer */
 
@@ -1368,6 +1375,40 @@ int msGMLWriteQuery(mapObj *map, char *filename, const char *namespaces)
 #endif
 }
 
+
+/************************************************************************/
+/*                             msAxisSwapShape                          */
+/*                                                                      */
+/*      Utility function to swap x and y coordiatesn Use for now for    */
+/*      WFS 1.1.x                                                       */
+/************************************************************************/
+void msAxisSwapShape(shapeObj *shape)
+{
+    double tmp;
+    int i,j;
+
+    if (shape)
+    {
+        for(i=0; i<shape->numlines; i++) 
+        {
+            for( j=0; j<shape->line[i].numpoints; j++ ) 
+            {
+                tmp = shape->line[i].point[j].x;
+                shape->line[i].point[j].x = shape->line[i].point[j].y;
+                shape->line[i].point[j].y = tmp;
+            }
+        }
+
+        /*swap bounds*/
+        tmp = shape->bounds.minx;
+        shape->bounds.minx = shape->bounds.miny;
+        shape->bounds.miny = tmp;
+
+        tmp = shape->bounds.maxx;
+        shape->bounds.maxx = shape->bounds.maxy;
+        shape->bounds.maxy = tmp;
+    }
+}
 /*
 ** msGMLWriteWFSQuery()
 **
@@ -1391,13 +1432,42 @@ int msGMLWriteWFSQuery(mapObj *map, FILE *stream, int maxfeatures, char *default
   gmlConstantObj *constant=NULL;
 
   char *namespace_prefix=NULL;
+  const char *axis = NULL;
+  int bSwapAxis = 0;
+  double tmp;
 
   msInitShape(&shape);
 
+  /*add a check to see if the map projection is set to be north-east*/
+  for( i = 0; i < map->projection.numargs; i++ )
+  {
+      if( strstr(map->projection.args[i],"epsgaxis=") != NULL )
+      {
+          axis = strstr(map->projection.args[i],"=") + 1;
+          break;
+      }
+  }
+
+  if (axis && strcasecmp(axis,"ne") == 0 )
+    bSwapAxis = 1;
+
+  
   /* Need to start with BBOX of the whole resultset */
   if (msGetQueryResultBounds(map, &resultBounds) > 0)
-    gmlWriteBounds(stream, outputformat, &resultBounds, msOWSGetEPSGProj(&(map->projection), &(map->web.metadata), "FGO", MS_TRUE), "      ");
+  {
+      if (bSwapAxis)
+      {
+          tmp = resultBounds.minx;
+          resultBounds.minx =  resultBounds.miny;
+          resultBounds.miny = tmp;
 
+          tmp = resultBounds.maxx;
+          resultBounds.maxx =  resultBounds.maxy;
+          resultBounds.maxy = tmp;
+
+      }
+      gmlWriteBounds(stream, outputformat, &resultBounds, msOWSGetEPSGProj(&(map->projection), &(map->web.metadata), "FGO", MS_TRUE), "      ");
+  }
   /* step through the layers looking for query results */
   for(i=0; i<map->numlayers; i++) {
 
@@ -1408,14 +1478,6 @@ int msGMLWriteWFSQuery(mapObj *map, FILE *stream, int maxfeatures, char *default
       const char *value;
       int featureIdIndex=-1; /* no feature id */
 
-      /* actually open the layer */
-      status = msLayerOpen(lp);
-      if(status != MS_SUCCESS) return(status);
-
-      /* retrieve all the item names. (Note : there might be no attributes) */
-      status = msLayerGetItems(lp);
-      /* if(status != MS_SUCCESS) return(status); */
-
       /* setup namespace, a layer can override the default */
       namespace_prefix = (char*) msOWSLookupMetadata(&(lp->metadata), "OFG", "namespace_prefix");
       if(!namespace_prefix) namespace_prefix = default_namespace_prefix;
@@ -1448,7 +1510,7 @@ int msGMLWriteWFSQuery(mapObj *map, FILE *stream, int maxfeatures, char *default
       }
 
       for(j=0; j<lp->resultcache->numresults; j++) {
-        status = msLayerGetShape(lp, &shape, lp->resultcache->results[j].tileindex, lp->resultcache->results[j].shapeindex);
+        status = msLayerResultsGetShape(lp, &shape, lp->resultcache->results[j].tileindex, lp->resultcache->results[j].shapeindex);
         if(status != MS_SUCCESS) 
             return(status);
 
@@ -1471,7 +1533,10 @@ int msGMLWriteWFSQuery(mapObj *map, FILE *stream, int maxfeatures, char *default
             msIO_fprintf(stream, "      <%s gml:id=\"%s.%s\">\n", layerName, lp->name, shape.values[featureIdIndex]);
         } else
           msIO_fprintf(stream, "      <%s>\n", layerName);
-                    
+              
+        if (bSwapAxis)
+          msAxisSwapShape(&shape);
+
         /* write the feature geometry and bounding box */
         if(!(geometryList && geometryList->numgeometries == 1 && strcasecmp(geometryList->geometries[0].name, "none") == 0)) {
 #ifdef USE_PROJ
@@ -1525,7 +1590,7 @@ int msGMLWriteWFSQuery(mapObj *map, FILE *stream, int maxfeatures, char *default
       msGMLFreeItems(itemList);
       msGMLFreeGeometries(geometryList);
 
-      msLayerClose(lp);
+      /* msLayerClose(lp); */
     }
 
     if (maxfeatures > 0 && features == maxfeatures)
