@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: maplayer.c 8901 2009-04-12 16:49:02Z tamas $
+ * $Id: maplayer.c 9417 2009-10-08 16:56:13Z sdlime $
  *
  * Project:  MapServer
  * Purpose:  Implementation of most layerObj functions.
@@ -32,7 +32,7 @@
 #include "mapogcfilter.h"
 
 #include <assert.h>
-MS_CVSID("$Id: maplayer.c 8901 2009-04-12 16:49:02Z tamas $")
+MS_CVSID("$Id: maplayer.c 9417 2009-10-08 16:56:13Z sdlime $")
 
 
 static int populateVirtualTable(layerVTableObj *vtable);
@@ -145,6 +145,21 @@ int msLayerNextShape(layerObj *layer, shapeObj *shape)
   return layer->vtable->LayerNextShape(layer, shape);
 }
 
+/*
+** Used to retrieve a shape from a result set by index. Result sets are created by the various
+** msQueryBy...() functions. The index is assigned by the data source.
+*/
+int msLayerResultsGetShape(layerObj *layer, shapeObj *shape, int tile, long record)
+{
+  if ( ! layer->vtable) {
+    int rv =  msInitializeVirtualTable(layer);
+    if (rv != MS_SUCCESS)
+      return rv;
+  }
+
+  return layer->vtable->LayerResultsGetShape(layer, shape, tile, record);
+}
+
 /*
 ** Used to retrieve a shape by index. All data sources must be capable of random access using
 ** a record number of some sort.
@@ -279,267 +294,174 @@ static int string2list(char **list, int *listsize, char *string)
 {
   int i;
 
-  for(i=0; i<(*listsize); i++)
-    if(strcasecmp(list[i], string) == 0) { 
-        /* printf("string2list (duplicate): %s %d\n", string, i); */
-      return(i);
-    }
+  for(i=0; i<(*listsize); i++) {
+    if(strcasecmp(list[i], string) == 0) return(i);
+  }
 
   list[i] = strdup(string);
   (*listsize)++;
 
-  /* printf("string2list: %s %d\n", string, i); */
-
   return(i);
 }
 
-/* TO DO: this function really needs to use the lexer */
-static void expression2list(char **list, int *listsize, expressionObj *expression) 
+static void expression2list(char **items, int numitems, char **list, int *listsize, expressionObj *exp)
 {
-  int i, j, l;
-  char tmpstr1[1024], tmpstr2[1024];
-  short in=MS_FALSE;
-  int tmpint;
+  int i, t, index;
+  char *tag, *expptr;
+  int taglen;
 
-  j = 0;
-  l = strlen(expression->string);
-  for(i=0; i<l; i++) {
-    if(expression->string[i] == '[') {
-      in = MS_TRUE;
-      tmpstr2[j] = expression->string[i];
-      j++;
-      continue;
-    }
-    if(expression->string[i] == ']') {
-      in = MS_FALSE;
-
-      tmpint = expression->numitems;
-
-      tmpstr2[j] = expression->string[i];
-      tmpstr2[j+1] = '\0';
-      string2list(expression->items, &(expression->numitems), tmpstr2);
-
-      if(tmpint != expression->numitems) { /* not a duplicate, so no need to calculate the index */
-        tmpstr1[j-1] = '\0';
-        expression->indexes[expression->numitems - 1] = string2list(list, listsize, tmpstr1);
-      }
-
-      j = 0; /* reset */
-
-      continue;
-    }
-
-    if(in) {
-      tmpstr2[j] = expression->string[i];
-      tmpstr1[j-1] = expression->string[i];
-      j++;
+  for(i=0; i<numitems; i++) {
+    tag = (char *) malloc(sizeof(char)*strlen(items[i])+3);
+    sprintf(tag, "[%s]", items[i]); /* there is enough room in tag */
+    if((expptr = (char *) msCaseFindSubstring(exp->string, tag)) != 0) {      
+      index = string2list(list, listsize, items[i]); /* add to overall list */
+
+      t = exp->numitems; /* we need to know if the tag list changes */
+      string2list(exp->items, &(exp->numitems), tag); /* add tag to expression list, use tag so we don't have to reconstruct it later */
+      if(t != exp->numitems) exp->indexes[exp->numitems-1] = index; /* index to overall list */
+
+      /* normalize expression */
+      taglen = strlen(tag);
+      do {
+	memcpy(expptr, tag, taglen);
+      } while((expptr = (char *) msCaseFindSubstring(expptr+taglen, tag)) != 0);
     }
+    free(tag);
   }
 }
 
-int msLayerWhichItemsNew(layerObj *layer, int classify, int annotate, char *metadata) 
+static int allocateExpressionItems(expressionObj *exp, int numitems)
 {
-  int status;
-  int numchars;
-/* int i; */
-
-  status = msLayerGetItems(layer); /* get a list of all attributes available for this layer (including JOINs) */
-  if(status != MS_SUCCESS) return(status);
-
-  /* allocate space for the various item lists */
-  if(classify && layer->filter.type == MS_EXPRESSION) { 
-    numchars = msCountChars(layer->filter.string, '[');
-    if(numchars > 0) {
-      layer->filter.items = (char **)calloc(numchars, sizeof(char *)); /* should be more than enough space */
-      if(!(layer->filter.items)) {
-	msSetError(MS_MEMERR, NULL, "msLayerWhichItems()");
-	return(MS_FAILURE);
-      }
-      layer->filter.indexes = (int *)malloc(numchars*sizeof(int));
-      if(!(layer->filter.indexes)) {
-	msSetError(MS_MEMERR, NULL, "msLayerWhichItems()");
-	return(MS_FAILURE);
-      }
-      layer->filter.numitems = 0;     
-    }
+  exp->items = (char **) calloc(numitems, sizeof(char *));
+  if(!(exp->items)) {
+    msSetError(MS_MEMERR, NULL, "allocateExpressionItems()");
+    return MS_FAILURE;
   }
+  exp->indexes = (int *) malloc(numitems*sizeof(int));
+  if(!(exp->indexes)) {
+    msSetError(MS_MEMERR, NULL, "allocateExpressionItems()");
+    return MS_FAILURE;
+  }
+  exp->numitems = 0; /* initially empty */
 
-  /* for(i=0; i<layer->numitems; i++) { } */
-
-  return(MS_SUCCESS);
+  return MS_SUCCESS; 
 }
 
-/*
-** This function builds a list of items necessary to draw or query a particular layer by
-** examining the contents of the various xxxxitem parameters and expressions. That list is
-** then used to set the iteminfo variable. 
-*/
-int msLayerWhichItems(layerObj *layer, int classify, int annotate, char *metadata) 
+static int isValidItem(char **list, int listsize, char *item) 
 {
+  int i;
+
+  for(i=0; i<listsize; i++) {
+    if(strcasecmp(list[i], item) == 0) return MS_TRUE; /* found it */
+  }
+
+  msSetError(MS_MISCERR, "Item %s not found", "isValidItem()", item);
+  return MS_FALSE;
+}
+
+int msLayerWhichItems(layerObj *layer, int get_all, char *metadata)
+{
+  char **items;
+  int numitems;
   int i, j, k, rv;
-  int nt=0, ne=0;
+  int freeitems=MS_FALSE;
 
   if (!layer->vtable) {
     rv =  msInitializeVirtualTable(layer);
-    if (rv != MS_SUCCESS) {
-      return rv;
-    }
+    if (rv != MS_SUCCESS) return rv;
   }
 
-  /*  */
-  /* TO DO! I have a better algorithm. */
-  /*  */
-  /* 1) call msLayerGetItems to get a complete list (including joins) */
-  /* 2) loop though item-based parameters and expressions to identify items to keep */
-  /* 3) based on 2) build a list of items */
-  /*  */
-  /* This is more straight forward and robust, fixes the problem of [...] regular expressions */
-  /* embeded in logical expressions. It also opens up to using dynamic joins anywhere... */
-  /*  */
-
-  /* Cleanup any previous item selection */
-  msLayerFreeItemInfo(layer);
-  if(layer->items) {
-    msFreeCharArray(layer->items, layer->numitems);
-    layer->items = NULL;
-    layer->numitems = 0;
-  }
+  /* force get_all=MS_TRUE in some cases */
+  if(layer->connectiontype == MS_INLINE || layer->connectiontype == MS_SDE)
+    get_all=MS_TRUE;
 
   /*
-  ** layer level counts
+  ** The algorithm:
+  **   1) call msLayerGetItems to get a complete list (including joins potentially)
+  **   2) loop though item-based parameters and expressions to identify items to keep 
+  **   3) based on 2) build a list of items 
   */
-  if(classify) {
-    if(layer->classitem) nt++;
-    if(layer->filteritem) nt++;
 
-    ne = 0;
-    if(layer->filter.type == MS_EXPRESSION) {
-      ne = msCountChars(layer->filter.string, '[');
-      if(ne > 0) {
-        layer->filter.items = (char **) calloc(ne, sizeof(char *)); /* should be more than enough space */
-        if(!(layer->filter.items)) {
-          msSetError(MS_MEMERR, NULL, "msLayerWhichItems()");
-          return(MS_FAILURE);
-        }
-        layer->filter.indexes = (int *) malloc(ne*sizeof(int));
-        if(!(layer->filter.indexes)) {
-          msSetError(MS_MEMERR, NULL, "msLayerWhichItems()");
-          return(MS_FAILURE);
-        }
-        layer->filter.numitems = 0;
-        nt += ne;
-      }
-    }
-  }
+  rv = msLayerGetItems(layer);
+  if(rv != MS_SUCCESS) return rv;
 
-  if(annotate) {
-    if(layer->labelitem) nt++;
-  }
+  if(layer->numitems == 0) return MS_SUCCESS; /* nothing to do but not an error */
+
+  items = layer->items; /* save for later */
+  numitems = layer->numitems;
 
   /*
-  ** class level counts
+  ** reset things (if necessary)
+  **   note: if we don't reset then the items array is fully populated will ALL items
   */
+  if(!get_all) {    
+    rv = layer->vtable->LayerCreateItems(layer, numitems);
+    if(rv != MS_SUCCESS) return rv;
+    freeitems = MS_TRUE;
+  }
+
+  /* layer-level item use */
+  if(layer->classitem) {
+    if(!isValidItem(items, numitems, layer->classitem)) return MS_FAILURE;
+    layer->classitemindex = string2list(layer->items, &(layer->numitems), layer->classitem);
+  }
+  if(layer->filteritem) {
+    if(!isValidItem(items, numitems, layer->filteritem)) return MS_FAILURE;
+    layer->filteritemindex = string2list(layer->items, &(layer->numitems), layer->filteritem);
+  }
+  if(layer->labelitem) {
+    if(!isValidItem(items, numitems, layer->labelitem)) return MS_FAILURE;
+    layer->labelitemindex = string2list(layer->items, &(layer->numitems), layer->labelitem);
+  }
+
+  if(layer->filter.type == MS_EXPRESSION) {
+    if(allocateExpressionItems(&layer->filter, numitems) != MS_SUCCESS) return MS_FAILURE;
+    expression2list(items, numitems, layer->items, &(layer->numitems), &(layer->filter));
+  }
+
+  /* class-level item use */
   for(i=0; i<layer->numclasses; i++) {
-    
-    if(classify) {
-      for(j=0; j<layer->class[i]->numstyles; j++) {
-        if(layer->class[i]->styles[j]->rangeitem) nt++;
-        nt += layer->class[i]->styles[j]->numbindings;
-      }
+    if(layer->class[i]->expression.type == MS_EXPRESSION) {
+      if(allocateExpressionItems(&layer->class[i]->expression, numitems) != MS_SUCCESS) return MS_FAILURE;
+      expression2list(items, numitems, layer->items, &(layer->numitems), &(layer->class[i]->expression));
+    }
 
-      ne = 0;
-      if(layer->class[i]->expression.type == MS_EXPRESSION) { 
-        ne = msCountChars(layer->class[i]->expression.string, '[');
-        if(ne > 0) {
-          layer->class[i]->expression.items = (char **) calloc(ne, sizeof(char *)); /* should be more than enough space */
-          if(!(layer->class[i]->expression.items)) {
-            msSetError(MS_MEMERR, NULL, "msLayerWhichItems()");
-            return(MS_FAILURE);
-          }
-          layer->class[i]->expression.indexes = (int *) malloc(ne*sizeof(int));
-          if(!(layer->class[i]->expression.indexes)) {
-            msSetError(MS_MEMERR, NULL, "msLayerWhichItems()");
-            return(MS_FAILURE);
-          }
-          layer->class[i]->expression.numitems = 0;
-          nt += ne;
-        }
+    if(layer->class[i]->text.type == MS_EXPRESSION) {
+      if(allocateExpressionItems(&layer->class[i]->text, numitems) != MS_SUCCESS) return MS_FAILURE;
+      expression2list(items, numitems, layer->items, &(layer->numitems), &(layer->class[i]->text));
+    }
+
+    for(k=0; k<MS_LABEL_BINDING_LENGTH; k++) {      
+      if(layer->class[i]->label.bindings[k].item) {
+        if(!isValidItem(items, numitems, layer->class[i]->label.bindings[k].item)) return MS_FAILURE;
+        layer->class[i]->label.bindings[k].index = string2list(layer->items, &(layer->numitems), layer->class[i]->label.bindings[k].item);
       }
     }
 
-    if(annotate) {
-      nt += layer->class[i]->label.numbindings;
-
-      ne = 0;
-      if(layer->class[i]->text.type == MS_EXPRESSION) { 
-        ne = msCountChars(layer->class[i]->text.string, '[');
-        if(ne > 0) {
-          layer->class[i]->text.items = (char **) calloc(ne, sizeof(char *)); /* should be more than enough space */
-          if(!(layer->class[i]->text.items)) {
-            msSetError(MS_MEMERR, NULL, "msLayerWhichItems()");
-            return(MS_FAILURE);
-          }
-          layer->class[i]->text.indexes = (int *) malloc(ne*sizeof(int));
-          if(!(layer->class[i]->text.indexes)) {
-            msSetError(MS_MEMERR, NULL, "msLayerWhichItems()");
-            return(MS_FAILURE);
-          }
-          layer->class[i]->text.numitems = 0;
-          nt += ne;
+    /* style-level item use */
+    for(j=0; j<layer->class[i]->numstyles; j++) {
+      if(layer->class[i]->styles[j]->rangeitem) {
+        if(!isValidItem(items, numitems, layer->class[i]->styles[j]->rangeitem)) return MS_FAILURE;
+        layer->class[i]->styles[j]->rangeitemindex = string2list(layer->items, &(layer->numitems), layer->class[i]->styles[j]->rangeitem);
+      }
+      for(k=0; k<MS_STYLE_BINDING_LENGTH; k++) {
+	if(layer->class[i]->styles[j]->bindings[k].item) {
+          if(!isValidItem(items, numitems, layer->class[i]->styles[j]->bindings[k].item)) return MS_FAILURE;
+          layer->class[i]->styles[j]->bindings[k].index = string2list(layer->items, &(layer->numitems), layer->class[i]->styles[j]->bindings[k].item);
         }
       }
     }
   }
 
-  /* Always retrieve all items in some cases */
-  if (layer->connectiontype == MS_INLINE)
-  {
-     msLayerGetItems(layer);
-     /* need to realloc the array to accept the possible new items*/
-     if (nt > 0)
-        layer->items = (char **)realloc(layer->items, sizeof(char *)*(layer->numitems + nt));
-  }
-  else
-  {
-    rv = layer->vtable->LayerCreateItems(layer, nt);
-    if (rv != MS_SUCCESS) {
-      return rv;
-    }
-  }
-
-  if(nt > 0) {
-    if(classify) {
-      if(layer->classitem) layer->classitemindex = string2list(layer->items, &(layer->numitems), layer->classitem);
-      if(layer->filteritem) layer->filteritemindex = string2list(layer->items, &(layer->numitems), layer->filteritem);
-
-      for(i=0; i<layer->numclasses; i++) {
-        if(layer->class[i]->expression.type == MS_EXPRESSION) expression2list(layer->items, &(layer->numitems), &(layer->class[i]->expression));
-        for(j=0; j<layer->class[i]->numstyles; j++) {
-          if(layer->class[i]->styles[j]->rangeitem) layer->class[i]->styles[j]->rangeitemindex = string2list(layer->items, &(layer->numitems), layer->class[i]->styles[j]->rangeitem); 
-	  for(k=0; k<MS_STYLE_BINDING_LENGTH; k++)
-	    if(layer->class[i]->styles[j]->bindings[k].item) layer->class[i]->styles[j]->bindings[k].index = string2list(layer->items, &(layer->numitems), layer->class[i]->styles[j]->bindings[k].item);
-        }
-      }
-
-      if(layer->filter.type == MS_EXPRESSION) expression2list(layer->items, &(layer->numitems), &(layer->filter));      
-    }
-
-    if(annotate) {
-      if(layer->labelitem) layer->labelitemindex = string2list(layer->items, &(layer->numitems), layer->labelitem);
-      for(i=0; i<layer->numclasses; i++) {
-        if(layer->class[i]->text.type == MS_EXPRESSION) expression2list(layer->items, &(layer->numitems), &(layer->class[i]->text));
-        for(k=0; k<MS_LABEL_BINDING_LENGTH; k++)
-          if(layer->class[i]->label.bindings[k].item) layer->class[i]->label.bindings[k].index = string2list(layer->items, &(layer->numitems), layer->class[i]->label.bindings[k].item);
-      }
-    }
-  }
-
+  /*
+  ** need a description here of what this code is doing...
+  */ 
   if(metadata) {
     char **tokens;
     int n = 0;
     int j;
     int bFound = 0;
-      
+
     tokens = msStringSplit(metadata, ',', &n);
     if(tokens) {
       for(i=0; i<n; i++) {
@@ -550,7 +472,7 @@ int msLayerWhichItems(layerObj *layer, int classify, int annotate, char *metadat
             break;
           }
         }
-              
+
         if(!bFound) {
           layer->numitems++;
           layer->items =  (char **)realloc(layer->items, sizeof(char *)*(layer->numitems));
@@ -561,11 +483,16 @@ int msLayerWhichItems(layerObj *layer, int classify, int annotate, char *metadat
     }
   }
 
+  /*
+  ** Clean up (if necessary)
+  */
+  if(freeitems) msFreeCharArray(items, numitems);
+
   /* populate the iteminfo array */
   if(layer->numitems == 0)
-    return(MS_SUCCESS);
-
-  return(msLayerInitItemInfo(layer));
+    return MS_SUCCESS;
+  else
+    return msLayerInitItemInfo(layer);
 }
 
 /*
@@ -988,59 +915,54 @@ msLayerMakePlainTimeFilter(layerObj *lp, const char *timestring,
 /*
  * Dummies / default actions for layers
  */
-int 
-LayerDefaultInitItemInfo(layerObj *layer)
+int LayerDefaultInitItemInfo(layerObj *layer)
 {
-    return MS_SUCCESS;
+  return MS_SUCCESS;
 }
 
-void 
-LayerDefaultFreeItemInfo(layerObj *layer)
+void LayerDefaultFreeItemInfo(layerObj *layer)
 {
-    return;
+  return;
 }
 
 int LayerDefaultOpen(layerObj *layer)
 {
-    return MS_FAILURE;
+  return MS_FAILURE;
 }
 
-int 
-LayerDefaultIsOpen(layerObj *layer)
+int LayerDefaultIsOpen(layerObj *layer)
 {
-    return MS_FALSE;
+  return MS_FALSE;
 }
 
-int 
-LayerDefaultWhichShapes(layerObj *layer, rectObj rect)
+int LayerDefaultWhichShapes(layerObj *layer, rectObj rect)
 {
-    return MS_SUCCESS;
+  return MS_SUCCESS;
 }
 
-int 
-LayerDefaultNextShape(layerObj *layer, shapeObj *shape)
+int LayerDefaultNextShape(layerObj *layer, shapeObj *shape)
 {
-    return MS_FAILURE;
+  return MS_FAILURE;
 }
 
-
-int 
-LayerDefaultGetShape(layerObj *layer, shapeObj *shape, 
-                     int tile, long record)
+int LayerDefaultResultsGetShape(layerObj *layer, shapeObj *shape, int tile, long record)
 {
-    return MS_FAILURE;
+  return MS_FAILURE;
 }
 
-int 
-LayerDefaultClose(layerObj *layer)
+int LayerDefaultGetShape(layerObj *layer, shapeObj *shape, int tile, long record)
 {
-    return MS_SUCCESS;
+  return MS_FAILURE;
 }
 
-int 
-LayerDefaultGetItems(layerObj *layer)
+int LayerDefaultClose(layerObj *layer)
 {
-    return MS_FAILURE;
+  return MS_SUCCESS;
+}
+
+int LayerDefaultGetItems(layerObj *layer)
+{
+  return MS_SUCCESS; /* returning no items is legit */
 }
 
 int 
@@ -1067,47 +989,39 @@ msLayerApplyPlainFilterToLayer(FilterEncodingNode *psNode, mapObj *map,
 #endif
 }
 
-int 
-LayerDefaultGetExtent(layerObj *layer, rectObj *extent)
+int LayerDefaultGetExtent(layerObj *layer, rectObj *extent)
 {
-    return MS_FAILURE;
+  return MS_FAILURE;
 }
 
-int 
-LayerDefaultGetAutoStyle(mapObj *map, layerObj *layer, classObj *c, 
-                         int tile, long record)
+int LayerDefaultGetAutoStyle(mapObj *map, layerObj *layer, classObj *c, int tile, long record)
 {
-    msSetError(MS_MISCERR, "'STYLEITEM AUTO' not supported for this data source.", "msLayerGetAutoStyle()");
-    return MS_FAILURE; 
+  msSetError(MS_MISCERR, "'STYLEITEM AUTO' not supported for this data source.", "msLayerGetAutoStyle()");
+  return MS_FAILURE; 
 }
 
-int 
-LayerDefaultCloseConnection(layerObj *layer)
+int LayerDefaultCloseConnection(layerObj *layer)
 {
-    return MS_SUCCESS;
+  return MS_SUCCESS;
 }
 
-
-int 
-LayerDefaultCreateItems(layerObj *layer,
-                      const int nt)
+int LayerDefaultCreateItems(layerObj *layer, const int nt)
 {
-    if (nt > 0) {
-        layer->items = (char **)calloc(nt, sizeof(char *)); /* should be more than enough space */
-        if(!layer->items) {
-            msSetError(MS_MEMERR, NULL, "LayerDefaultCreateItems()");
-            return(MS_FAILURE);
-        }
-        layer->numitems = 0;
+  if (nt > 0) {
+    layer->items = (char **)calloc(nt, sizeof(char *)); /* should be more than enough space */
+    if(!layer->items) {
+      msSetError(MS_MEMERR, NULL, "LayerDefaultCreateItems()");
+      return(MS_FAILURE);
     }
-    return MS_SUCCESS;
+    layer->numitems = 0;
+  }
+  return MS_SUCCESS;
 }
 
-int 
-LayerDefaultGetNumFeatures(layerObj *layer)
+int LayerDefaultGetNumFeatures(layerObj *layer)
 {
-    msSetError(MS_SHPERR, "Not an inline layer", "msLayerGetNumFeatures()");
-    return MS_FAILURE;
+  msSetError(MS_SHPERR, "Not an inline layer", "msLayerGetNumFeatures()");
+  return MS_FAILURE;
 }
 
 /*
@@ -1140,122 +1054,116 @@ int msConnectLayer(layerObj *layer,
     return msInitializeVirtualTable(layer) ;   
 }
 
-static int
-populateVirtualTable(layerVTableObj *vtable)
+static int populateVirtualTable(layerVTableObj *vtable)
 {
-    assert(vtable != NULL);
+  assert(vtable != NULL);
 
-    vtable->LayerInitItemInfo = LayerDefaultInitItemInfo;
-    vtable->LayerFreeItemInfo = LayerDefaultFreeItemInfo;
-    vtable->LayerOpen = LayerDefaultOpen;
-    vtable->LayerIsOpen = LayerDefaultIsOpen;
-    vtable->LayerWhichShapes = LayerDefaultWhichShapes;
+  vtable->LayerInitItemInfo = LayerDefaultInitItemInfo;
+  vtable->LayerFreeItemInfo = LayerDefaultFreeItemInfo;
+  vtable->LayerOpen = LayerDefaultOpen;
+  vtable->LayerIsOpen = LayerDefaultIsOpen;
+  vtable->LayerWhichShapes = LayerDefaultWhichShapes;
 
-    vtable->LayerNextShape = LayerDefaultNextShape;
-    vtable->LayerGetShape = LayerDefaultGetShape;
-    vtable->LayerClose = LayerDefaultClose;
-    vtable->LayerGetItems = LayerDefaultGetItems;
-    vtable->LayerGetExtent = LayerDefaultGetExtent;
+  vtable->LayerNextShape = LayerDefaultNextShape;
+  vtable->LayerResultsGetShape = LayerDefaultResultsGetShape;
+  vtable->LayerGetShape = LayerDefaultGetShape;
+  vtable->LayerClose = LayerDefaultClose;
+  vtable->LayerGetItems = LayerDefaultGetItems;
+  vtable->LayerGetExtent = LayerDefaultGetExtent;
 
-    vtable->LayerGetAutoStyle = LayerDefaultGetAutoStyle;
-    vtable->LayerCloseConnection = LayerDefaultCloseConnection;
-    vtable->LayerSetTimeFilter = msLayerMakePlainTimeFilter;
+  vtable->LayerGetAutoStyle = LayerDefaultGetAutoStyle;
+  vtable->LayerCloseConnection = LayerDefaultCloseConnection;
+  vtable->LayerSetTimeFilter = msLayerMakePlainTimeFilter;
 
-    vtable->LayerApplyFilterToLayer = msLayerApplyPlainFilterToLayer;
+  vtable->LayerApplyFilterToLayer = msLayerApplyPlainFilterToLayer;
 
-    vtable->LayerCreateItems = LayerDefaultCreateItems;
+  vtable->LayerCreateItems = LayerDefaultCreateItems;
     
-    vtable->LayerGetNumFeatures = LayerDefaultGetNumFeatures;
+  vtable->LayerGetNumFeatures = LayerDefaultGetNumFeatures;
 
-    return MS_SUCCESS;
+  return MS_SUCCESS;
 }
 
-static int
-createVirtualTable(layerVTableObj **vtable)
+static int createVirtualTable(layerVTableObj **vtable)
 {
-    *vtable = malloc(sizeof(**vtable));
-    if ( ! *vtable) {
-        return MS_FAILURE;
-    }
-    return populateVirtualTable(*vtable);
+  *vtable = malloc(sizeof(**vtable));
+  if ( ! *vtable) {
+    return MS_FAILURE;
+  }
+  return populateVirtualTable(*vtable);
 }
 
-static int
-destroyVirtualTable(layerVTableObj **vtable)
+static int destroyVirtualTable(layerVTableObj **vtable)
 {
-    memset(*vtable, 0, sizeof(**vtable));
-    msFree(*vtable);
-    *vtable = NULL;
-    return MS_SUCCESS;
+  memset(*vtable, 0, sizeof(**vtable));
+  msFree(*vtable);
+  *vtable = NULL;
+  return MS_SUCCESS;
 }
 
-int
-msInitializeVirtualTable(layerObj *layer)
+int msInitializeVirtualTable(layerObj *layer)
 {
-    if (layer->vtable) {
-        destroyVirtualTable(&layer->vtable);
+  if (layer->vtable) {
+    destroyVirtualTable(&layer->vtable);
+  }
+  createVirtualTable(&layer->vtable);
+   
+  if(layer->features && layer->connectiontype != MS_GRATICULE ) 
+    layer->connectiontype = MS_INLINE;
+
+  if(layer->tileindex && layer->connectiontype == MS_SHAPEFILE)
+    layer->connectiontype = MS_TILED_SHAPEFILE;
+
+  if(layer->type == MS_LAYER_RASTER && layer->connectiontype != MS_WMS)
+    layer->connectiontype = MS_RASTER;
+
+  switch(layer->connectiontype) {
+    case(MS_INLINE):
+      return(msINLINELayerInitializeVirtualTable(layer));
+      break;
+    case(MS_SHAPEFILE):
+      return(msShapeFileLayerInitializeVirtualTable(layer));
+      break;
+    case(MS_TILED_SHAPEFILE):
+      return(msTiledSHPLayerInitializeVirtualTable(layer));
+      break;
+    case(MS_SDE):
+      return(msSDELayerInitializeVirtualTable(layer));
+      break;
+    case(MS_OGR):
+      return(msOGRLayerInitializeVirtualTable(layer));
+      break;
+    case(MS_POSTGIS):
+      return(msPostGISLayerInitializeVirtualTable(layer));
+      break;
+    case(MS_WMS):
+      /* WMS should be treated as a raster layer */
+      return(msRASTERLayerInitializeVirtualTable(layer));
+      break;
+    case(MS_ORACLESPATIAL):
+      return(msOracleSpatialLayerInitializeVirtualTable(layer));
+      break;
+    case(MS_WFS):
+      return(msWFSLayerInitializeVirtualTable(layer));
+      break;
+    case(MS_GRATICULE):
+      return(msGraticuleLayerInitializeVirtualTable(layer));
+      break;
+    case(MS_MYGIS):
+      return(msMYGISLayerInitializeVirtualTable(layer));
+      break;         
+    case(MS_RASTER):
+      return(msRASTERLayerInitializeVirtualTable(layer));
+      break;
+    case(MS_PLUGIN):
+      return(msPluginLayerInitializeVirtualTable(layer));
+      break;
+    default:
+      msSetError(MS_MISCERR, "Unknown connectiontype, it was %d", "msInitializeVirtualTable()", layer->connectiontype);
+      return MS_FAILURE;
+      break;
     }
-    createVirtualTable(&layer->vtable);
 
-    
-    if(layer->features && layer->connectiontype != MS_GRATICULE ) 
-      layer->connectiontype = MS_INLINE;
-
-    if(layer->tileindex && layer->connectiontype == MS_SHAPEFILE)
-      layer->connectiontype = MS_TILED_SHAPEFILE;
-
-    if(layer->type == MS_LAYER_RASTER && layer->connectiontype != MS_WMS)
-      layer->connectiontype = MS_RASTER;
-    
-
-    switch(layer->connectiontype) {
-        case(MS_INLINE):
-            return(msINLINELayerInitializeVirtualTable(layer));
-            break;
-        case(MS_SHAPEFILE):
-            return(msShapeFileLayerInitializeVirtualTable(layer));
-            break;
-        case(MS_TILED_SHAPEFILE):
-            return(msTiledSHPLayerInitializeVirtualTable(layer));
-            break;
-        case(MS_SDE):
-            return(msSDELayerInitializeVirtualTable(layer));
-            break;
-        case(MS_OGR):
-            return(msOGRLayerInitializeVirtualTable(layer));
-            break;
-        case(MS_POSTGIS):
-            return(msPostGISLayerInitializeVirtualTable(layer));
-            break;
-        case(MS_WMS):
-            /* WMS should be treated as a raster layer */
-              return(msRASTERLayerInitializeVirtualTable(layer));
-              break;
-        case(MS_ORACLESPATIAL):
-            return(msOracleSpatialLayerInitializeVirtualTable(layer));
-            break;
-        case(MS_WFS):
-            return(msWFSLayerInitializeVirtualTable(layer));
-            break;
-        case(MS_GRATICULE):
-            return(msGraticuleLayerInitializeVirtualTable(layer));
-            break;
-        case(MS_MYGIS):
-            return(msMYGISLayerInitializeVirtualTable(layer));
-            break;         
-        case(MS_RASTER):
-            return(msRASTERLayerInitializeVirtualTable(layer));
-            break;
-        case(MS_PLUGIN):
-            return(msPluginLayerInitializeVirtualTable(layer));
-            break;
-        default:
-            msSetError(MS_MISCERR, 
-                       "Unknown connectiontype, it was %d", 
-                       "msInitializeVirtualTable()", layer->connectiontype);
-            return MS_FAILURE;
-            break;
-    }
     /* not reached */
     return MS_FAILURE;
 }
@@ -1357,6 +1265,7 @@ msINLINELayerInitializeVirtualTable(layerObj *layer)
     layer->vtable->LayerIsOpen = msINLINELayerIsOpen;
     /* layer->vtable->LayerWhichShapes, use default */
     layer->vtable->LayerNextShape = msINLINELayerNextShape;
+    layer->vtable->LayerResultsGetShape = msINLINELayerGetShape; /* no special version, use ...GetShape() */
     layer->vtable->LayerGetShape = msINLINELayerGetShape;
     /* layer->vtable->LayerClose, use default */
     /* layer->vtable->LayerGetItems, use default */
