@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: maplegend.c 8959 2009-04-24 15:07:54Z assefa $
+ * $Id: maplegend.c 9432 2009-10-09 13:07:50Z tbonfort $
  *
  * Project:  MapServer
  * Purpose:  Legend generation.
@@ -29,7 +29,7 @@
 
 #include "mapserver.h"
 
-MS_CVSID("$Id: maplegend.c 8959 2009-04-24 15:07:54Z assefa $")
+MS_CVSID("$Id: maplegend.c 9432 2009-10-09 13:07:50Z tbonfort $")
 
 #define PSF .8
 #define VMARGIN 5 /* margin at top and bottom of legend graphic */
@@ -40,17 +40,152 @@ MS_CVSID("$Id: maplegend.c 8959 2009-04-24 15:07:54Z assefa $")
  * renderer specific drawing functions shouldn't be called directly, but through
  * this function
  */
-int msDrawLegendIcon(mapObj *map, layerObj *lp, classObj *class, int width, int height, imageObj *img, int dstX, int dstY)
+int msDrawLegendIcon(mapObj *map, layerObj *lp, classObj *theclass, 
+        int width, int height, imageObj *image, int dstX, int dstY)
 {
-#ifdef USE_AGG
-  if(MS_RENDERER_AGG(map->outputformat))
-    return msDrawLegendIconAGG(map,lp,class,width,height,img,dstX,dstY);
-  else
-#endif
-    /*default is to draw with GD*/
-    return msDrawLegendIconGD(map,lp,class,width,height,img->img.gd,dstX,dstY);
+  int i, type;
+  shapeObj box, zigzag;
+  pointObj marker;
+  char szPath[MS_MAXPATHLEN];
+  styleObj outline_style;
+  
+  if(MS_RENDERER_GD(map->outputformat))
+    /* keep GD specific code here for now as it supports clipping */
+	if(MS_VALID_COLOR(map->legend.outlinecolor))
+	  gdImageSetClip(image->img.gd, dstX, dstY, dstX + width - 1, dstY + height - 1);
+  
+
+  /* initialize the box used for polygons and for outlines */
+  box.line = (lineObj *)malloc(sizeof(lineObj));
+  box.numlines = 1;
+  box.line[0].point = (pointObj *)malloc(sizeof(pointObj)*5);
+  box.line[0].numpoints = 5;
+
+  box.line[0].point[0].x = dstX;
+  box.line[0].point[0].y = dstY;
+  box.line[0].point[1].x = dstX + width - 1;
+  box.line[0].point[1].y = dstY;
+  box.line[0].point[2].x = dstX + width - 1;
+  box.line[0].point[2].y = dstY + height - 1;
+  box.line[0].point[3].x = dstX;
+  box.line[0].point[3].y = dstY + height - 1;
+  box.line[0].point[4].x = box.line[0].point[0].x;
+  box.line[0].point[4].y = box.line[0].point[0].y;
+  box.line[0].numpoints = 5;
+    
+  /* if the class has a keyimage, treat it as a point layer
+   * (the keyimage will be treated there) */
+  if(theclass->keyimage != NULL) {
+    type = MS_LAYER_POINT;
+  } else {        
+    /* some polygon layers may be better drawn using zigzag if there is no fill */
+    type = lp->type;
+    if(type == MS_LAYER_POLYGON) {
+      type = MS_LAYER_LINE;
+      for(i=0; i<theclass->numstyles; i++) {
+       if(MS_VALID_COLOR(theclass->styles[i]->color)) { /* there is a fill */
+      type = MS_LAYER_POLYGON;
+      break;
+        }
+      }
+    }
+  }
+
+  /* 
+  ** now draw the appropriate color/symbol/size combination 
+  */     
+  switch(type) {
+  case MS_LAYER_ANNOTATION:
+    marker.x = dstX + MS_NINT(width / 2.0);
+    marker.y = dstY + MS_NINT(height / 2.0);
+    if (theclass->numstyles > 0) {
+      for(i=0; i<theclass->numstyles; i++)
+        msDrawMarkerSymbol(&map->symbolset, image, &marker, theclass->styles[i], lp->scalefactor);          
+    } else if (theclass->label.size!=-1) {
+      labelObj label = theclass->label;
+      double lsize = label.size;
+      double langle = label.angle;
+      int lpos = label.position;
+      label.angle = 0;
+      label.position = MS_CC;
+      if (label.type == MS_TRUETYPE) label.size = height;
+      msDrawLabel(map, image, marker, (char*)"Az", &label,1.0);
+      label.size = lsize;
+      label.position = lpos;
+      label.angle = langle;
+    }
+    break;
+  case MS_LAYER_POINT:
+    marker.x = dstX + MS_NINT(width / 2.0);
+    marker.y = dstY + MS_NINT(height / 2.0);
+    if(theclass->keyimage != NULL) {
+      int symbolNum;
+      styleObj imgStyle;
+      symbolNum = msAddImageSymbol(&(map->symbolset), msBuildPath(szPath, map->mappath, theclass->keyimage));
+      if(symbolNum == -1) { 
+          msSetError(MS_GDERR, "Failed to open legend key image", "msCreateLegendIcon()");
+          return(MS_FAILURE);
+      }
+      initStyle(&imgStyle);
+      imgStyle.size = width;
+      imgStyle.symbol = symbolNum;
+      msDrawMarkerSymbol(&map->symbolset,image,&marker,&imgStyle,lp->scalefactor);
+      /* TO DO: we may want to handle this differently depending on the relative size of the keyimage */
+    } else {
+      for(i=0; i<theclass->numstyles; i++)
+        msDrawMarkerSymbol(&map->symbolset, image, &marker, theclass->styles[i], lp->scalefactor);
+    }
+    break;
+  case MS_LAYER_LINE:
+    zigzag.line = (lineObj *)malloc(sizeof(lineObj));
+    zigzag.numlines = 1;
+    zigzag.line[0].point = (pointObj *)malloc(sizeof(pointObj)*4);
+    zigzag.line[0].numpoints = 4;
+
+    zigzag.line[0].point[0].x = dstX;
+    zigzag.line[0].point[0].y = dstY + height - 1;
+    zigzag.line[0].point[1].x = dstX + MS_NINT(width / 3.0) - 1; 
+    zigzag.line[0].point[1].y = dstY;
+    zigzag.line[0].point[2].x = dstX + MS_NINT(2.0 * width / 3.0) - 1; 
+    zigzag.line[0].point[2].y = dstY + height - 1; 
+    zigzag.line[0].point[3].x = dstX + width - 1; 
+    zigzag.line[0].point[3].y = dstY; 
+
+    for(i=0; i<theclass->numstyles; i++)
+      msDrawLineSymbol(&map->symbolset, image, &zigzag, theclass->styles[i], lp->scalefactor); 
+
+    free(zigzag.line[0].point);
+    free(zigzag.line);    
+    break;
+  case MS_LAYER_CIRCLE:
+  case MS_LAYER_RASTER:
+  case MS_LAYER_CHART:
+  case MS_LAYER_POLYGON:
+    for(i=0; i<theclass->numstyles; i++)     
+      msDrawShadeSymbol(&map->symbolset, image, &box, theclass->styles[i], lp->scalefactor);
+    break;
+  default:
+    return MS_FAILURE;
+    break;
+  } /* end symbol drawing */
+
+  /* handle an outline if necessary */
+  if(MS_VALID_COLOR(map->legend.outlinecolor)) {
+    initStyle(&outline_style);
+    outline_style.color = map->legend.outlinecolor;
+    msDrawLineSymbol(&map->symbolset, image, &box, &outline_style, 1.0);
+    /* reset clipping rectangle */
+    if(MS_RENDERER_GD(map->outputformat))
+    	gdImageSetClip(image->img.gd, 0, 0, image->width - 1, image->height - 1);
+  }
+
+  free(box.line[0].point);
+  free(box.line);
+  
+  return MS_SUCCESS;
 }
 
+
 imageObj *msCreateLegendIcon(mapObj* map, layerObj* lp, classObj* class, int width, int height)
 {
   imageObj *image;
@@ -68,10 +203,10 @@ imageObj *msCreateLegendIcon(mapObj* map, layerObj* lp, classObj* class, int wid
   /* create image */
 #ifdef USE_AGG
   if(MS_RENDERER_AGG(map->outputformat))
-    image = msImageCreateAGG(width, height, map->outputformat, map->web.imagepath, map->web.imageurl);        
+     image = msImageCreateAGG(width, height, map->outputformat, map->web.imagepath, map->web.imageurl, map->resolution, map->defresolution);        
   else
 #endif
-    image = msImageCreateGD(width, height, map->outputformat, map->web.imagepath, map->web.imageurl);
+    image = msImageCreateGD(width, height, map->outputformat, map->web.imagepath, map->web.imageurl, map->resolution, map->defresolution);
 
   /* drop this reference to output format */
   msApplyOutputFormat( &format, NULL, MS_NOOVERRIDE, MS_NOOVERRIDE, MS_NOOVERRIDE );
@@ -243,7 +378,12 @@ imageObj *msDrawLegend(mapObj *map, int scale_independent)
   };
   typedef struct legend_struct legendlabel;
   legendlabel *head=NULL,*cur=NULL;
-
+  
+  if(MS_RENDERER_PLUGIN(map->outputformat)) {
+    msSetError(MS_MISCERR, "Scalebar not supported yet", "msDrawScalebar()");
+    return(NULL);
+  }
+  
   if(msValidateContexts(map) != MS_SUCCESS) return NULL; /* make sure there are no recursive REQUIRES or LABELREQUIRES expressions */
   if(msLegendCalcSize(map, scale_independent, &size_x, &size_y, NULL) != MS_SUCCESS) return NULL;
 
@@ -318,10 +458,10 @@ imageObj *msDrawLegend(mapObj *map, int scale_independent)
   /* initialize the legend image */
 #ifdef USE_AGG
   if(MS_RENDERER_AGG(map->outputformat))
-    image = msImageCreateAGG(size_x, size_y, format, map->web.imagepath, map->web.imageurl);        
+    image = msImageCreateAGG(size_x, size_y, format, map->web.imagepath, map->web.imageurl, map->resolution, map->defresolution);        
   else
 #endif
-    image = msImageCreateGD(size_x, size_y, format, map->web.imagepath, map->web.imageurl);
+    image = msImageCreateGD(size_x, size_y, format, map->web.imagepath, map->web.imageurl, map->resolution, map->defresolution);
 
   /* drop this reference to output format */
   msApplyOutputFormat(&format, NULL, MS_NOOVERRIDE, MS_NOOVERRIDE, MS_NOOVERRIDE);
@@ -352,7 +492,7 @@ imageObj *msDrawLegend(mapObj *map, int scale_independent)
     /* set the scale factor so that scale dependant symbols are drawn in the legend with their default size */
     if(cur->layer->sizeunits != MS_PIXELS)
     {
-      map->cellsize = msAdjustExtent(&(map->extent), map->width, map->height);
+       map->cellsize = msAdjustExtent(&(map->extent), map->width, map->height);
       cur->layer->scalefactor = (msInchesPerUnit(cur->layer->sizeunits,0)/msInchesPerUnit(map->units,0)) / map->cellsize;
     }
     if(msDrawLegendIcon(map, cur->layer, cur->theclass,  map->legend.keysizex,  map->legend.keysizey, image, HMARGIN, (int) pnt.y) != MS_SUCCESS)
@@ -403,6 +543,11 @@ int msEmbedLegend(mapObj *map, imageObj *img)
   pointObj point;
   imageObj *image = NULL;
 
+  if(MS_RENDERER_PLUGIN(map->outputformat)) {
+    msSetError(MS_MISCERR, "Scalebar not supported yet", "msDrawScalebar()");
+    return MS_FAILURE;
+  }
+
   s = msGetSymbolIndex(&(map->symbolset), "legend", MS_FALSE);
   if(s != -1) 
     msRemoveSymbol(&(map->symbolset), s); /* solves some caching issues in AGG with long-running processes */
