@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: mapmssql2008.c 7985 2008-10-18 20:49:28Z tamas $
+ * $Id: mapmssql2008.c 9659 2010-01-03 16:53:22Z tamas $
  *
  * Project:  MapServer
  * Purpose:  MS SQL 2008 (Katmai) Layer Connector
@@ -34,7 +34,7 @@
 
 #define _CRT_SECURE_NO_WARNINGS 1
 
-/* $Id: mapmssql2008.c 7985 2008-10-18 20:49:28Z tamas $ */
+/* $Id: mapmssql2008.c 9659 2010-01-03 16:53:22Z tamas $ */
 #include <assert.h>
 #include "mapserver.h"
 #include "maptime.h"
@@ -49,7 +49,7 @@
 #include <string.h>
 #include <ctype.h> /* tolower() */
 
-MS_CVSID("$Id: mapmssql2008.c 7985 2008-10-18 20:49:28Z tamas $")
+MS_CVSID("$Id: mapmssql2008.c 9659 2010-01-03 16:53:22Z tamas $")
 
 /* Structure for connection to an ODBC database (Microsoft preferred way to connect to SQL Server 2005 from c/c++) */
 typedef struct msODBCconn_t
@@ -1264,22 +1264,28 @@ int msMSSQL2008LayerGetShapeRandom(layerObj *layer, shapeObj *shape, long *recor
 				/* figure out how big the buffer needs to be */
                 rc = SQLGetData(layerinfo->conn->hstmt, t + 1, SQL_C_BINARY, dummyBuffer, 0, &needLen);
 
-				/* allocate the buffer - this will be a null-terminated string so alloc for the null too */
-				valueBuffer = (char*) malloc( needLen + 1 );
-				if ( valueBuffer == NULL )
-				{
-					msSetError( MS_QUERYERR, "Could not allocate value buffer.", "msMSSQL2008LayerGetShapeRandom()" );
-					return MS_FAILURE;
-				}
+				if (needLen > 0)
+                {
+                    /* allocate the buffer - this will be a null-terminated string so alloc for the null too */
+				    valueBuffer = (char*) malloc( needLen + 1 );
+				    if ( valueBuffer == NULL )
+				    {
+					    msSetError( MS_QUERYERR, "Could not allocate value buffer.", "msMSSQL2008LayerGetShapeRandom()" );
+					    return MS_FAILURE;
+				    }
 
-				/* Now grab the data */
-                rc = SQLGetData(layerinfo->conn->hstmt, t + 1, SQL_C_BINARY, valueBuffer, needLen, &retLen);
+				    /* Now grab the data */
+                    rc = SQLGetData(layerinfo->conn->hstmt, t + 1, SQL_C_BINARY, valueBuffer, needLen, &retLen);
 
-				/* Terminate the buffer */
-                valueBuffer[retLen] = 0; /* null terminate it */
+				    /* Terminate the buffer */
+                    valueBuffer[retLen] = 0; /* null terminate it */
 
-				/* Pop the value into the shape's value array */
-                shape->values[t] = valueBuffer;
+				    /* Pop the value into the shape's value array */
+                    shape->values[t] = valueBuffer;
+                }
+                else
+                    /* Copy empty sting for NULL values */
+                    shape->values[t] = strdup("");
             }
 
             /* Get shape geometry */
@@ -1436,7 +1442,7 @@ int msMSSQL2008LayerGetShape(layerObj *layer, shapeObj *shape, long record)
 
     if(layer->numitems == 0) 
     {
-        snprintf(buffer, sizeof(buffer), "%s", layerinfo->geom_column);
+        snprintf(buffer, sizeof(buffer), "%s.STAsBinary()", layerinfo->geom_column);
         columns_wanted = _strdup(buffer);
     } 
     else 
@@ -1445,7 +1451,7 @@ int msMSSQL2008LayerGetShape(layerObj *layer, shapeObj *shape, long record)
             snprintf(buffer + strlen(buffer), sizeof(buffer) - strlen(buffer), "convert(varchar(max), %s),", layer->items[t]);
         }
 
-        snprintf(buffer + strlen(buffer), sizeof(buffer) - strlen(buffer), "%s", layerinfo->geom_column);
+        snprintf(buffer + strlen(buffer), sizeof(buffer) - strlen(buffer), "%s.STAsBinary()", layerinfo->geom_column);
 
         columns_wanted = _strdup(buffer);
     }
