@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: mapoutput.c 8821 2009-03-25 14:26:21Z assefa $
+ * $Id: mapoutput.c 9434 2009-10-09 21:06:41Z sdlime $
  *
  * Project:  MapServer
  * Purpose:  Various support code related to the outputFormatObj.
@@ -30,7 +30,7 @@
 #include <assert.h>
 #include "mapserver.h"
 
-MS_CVSID("$Id: mapoutput.c 8821 2009-03-25 14:26:21Z assefa $")
+MS_CVSID("$Id: mapoutput.c 9434 2009-10-09 21:06:41Z sdlime $")
 
 static outputFormatObj *msAllocOutputFormat( mapObj *map, const char *name, 
                                              const char *driver );
@@ -219,6 +219,57 @@ outputFormatObj *msCreateDefaultOutputFormat( mapObj *map,
         format->renderer = MS_RENDER_WITH_AGG;
     }
 #endif
+    
+#if defined(USE_CAIRO)
+    if( strcasecmp(driver,"CAIRO/PNG") == 0 )
+    {
+        format = msAllocOutputFormat( map, "cairopng", driver );
+        format->mimetype = strdup("image/png; mode=24bit");
+        format->imagemode = MS_IMAGEMODE_RGB;
+        format->extension = strdup("png");
+        format->renderer = MS_RENDER_WITH_CAIRO_RASTER;
+    }
+    if( strcasecmp(driver,"CAIRO/JPEG") == 0 )
+    {
+        format = msAllocOutputFormat( map, "cairojpeg", driver );
+        format->mimetype = strdup("image/jpeg");
+        format->imagemode = MS_IMAGEMODE_RGB;
+        format->extension = strdup("jpg");
+        format->renderer = MS_RENDER_WITH_CAIRO_RASTER;
+    }
+    if( strcasecmp(driver,"CAIRO/PDF") == 0 )
+	{
+		format = msAllocOutputFormat( map, "cairopdf", driver );
+		format->mimetype = strdup("application/x-pdf");
+		format->imagemode = MS_IMAGEMODE_RGB;
+		format->extension = strdup("pdf");
+		format->renderer = MS_RENDER_WITH_CAIRO_PDF;
+	}
+    if( strcasecmp(driver,"CAIRO/SVG") == 0 )
+	{
+		format = msAllocOutputFormat( map, "cairosvg", driver );
+		format->mimetype = strdup("image/svg+xml");
+		format->imagemode = MS_IMAGEMODE_RGB;
+		format->extension = strdup("svg");
+		format->renderer = MS_RENDER_WITH_CAIRO_SVG;
+	}
+    
+    
+#endif
+
+#if defined(USE_OGL) 
+    if( strcasecmp(driver,"OGL/PNG") == 0 ) 
+    { 
+        format = msAllocOutputFormat( map, "oglpng24", driver ); 
+        format->mimetype = strdup("image/png; mode=24bit"); 
+        format->imagemode = MS_IMAGEMODE_RGB; 
+        format->extension = strdup("png"); 
+        format->renderer = MS_RENDER_WITH_OGL; 
+    } 
+#endif 
+ 
+    
+
 #ifdef USE_MING_FLASH
     if( strcasecmp(driver,"swf") == 0 )
     {
@@ -259,7 +310,7 @@ outputFormatObj *msCreateDefaultOutputFormat( mapObj *map,
     if( strcasecmp(driver,"imagemap") == 0 )
     {
         format = msAllocOutputFormat( map, "imagemap", driver );
-        format->mimetype = strdup("text/html");
+        format->mimetype = strdup("text/html; driver=imagemap");
         format->extension = strdup("html");
         format->imagemode = 0;
         format->renderer = MS_RENDER_WITH_IMAGEMAP;
@@ -283,7 +334,10 @@ outputFormatObj *msCreateDefaultOutputFormat( mapObj *map,
 
     if( format != NULL )
       format->inmapfile = MS_FALSE;
-
+    
+    if( format!= NULL && MS_RENDERER_PLUGIN(format) ) {
+        msInitializeRendererVTable(format);
+    }
     return format;
 }
 
@@ -321,6 +375,19 @@ void msApplyDefaultOutputFormats( mapObj *map )
 
     if( msSelectOutputFormat( map, "aggjpeg" ) == NULL )
         msCreateDefaultOutputFormat( map, "AGG/JPEG" );
+    
+    if( msSelectOutputFormat( map, "cairopng" ) == NULL )
+        msCreateDefaultOutputFormat( map, "CAIRO/PNG" );
+
+    if( msSelectOutputFormat( map, "cairojpeg" ) == NULL )
+        msCreateDefaultOutputFormat( map, "CAIRO/JPEG" );
+    
+    if( msSelectOutputFormat( map, "cairopdf" ) == NULL )
+        msCreateDefaultOutputFormat( map, "CAIRO/PDF" );
+    
+    if( msSelectOutputFormat( map, "cairosvg" ) == NULL )
+        msCreateDefaultOutputFormat( map, "CAIRO/SVG" );
+
 
     if( msSelectOutputFormat( map, "swf" ) == NULL )
         msCreateDefaultOutputFormat( map, "swf" );
@@ -337,6 +404,17 @@ void msApplyDefaultOutputFormats( mapObj *map )
     if( msSelectOutputFormat( map, "svg" ) == NULL )
         msCreateDefaultOutputFormat( map, "svg" );
 
+#if defined(USE_CAIRO)
+    if( msSelectOutputFormat( map, "cairopng24" ) == NULL )
+        msCreateDefaultOutputFormat( map, "CAIRO/PNG" );
+    if( msSelectOutputFormat( map, "cairojpeg" ) == NULL )
+        msCreateDefaultOutputFormat( map, "CAIRO/JPEG" );
+    if( msSelectOutputFormat( map, "cairopdf" ) == NULL )
+        msCreateDefaultOutputFormat( map, "CAIRO/PDF" );
+    if( msSelectOutputFormat( map, "cairosvg" ) == NULL )
+        msCreateDefaultOutputFormat( map, "CAIRO/SVG" );
+#endif
+
     if( map->imagetype != NULL )
         free( map->imagetype );
     map->imagetype = saved_imagetype;
@@ -357,7 +435,8 @@ void msFreeOutputFormat( outputFormatObj *format )
     msFree( format->driver );
     msFree( format->extension );
     msFreeCharArray( format->formatoptions, format->numformatoptions );
-
+    /* msFreeRendererVTable( format->vtable ); */
+    msFree( format->vtable );
     msFree( format );
 }
 
@@ -388,7 +467,7 @@ static outputFormatObj *msAllocOutputFormat( mapObj *map, const char *name,
     format->name = strdup(name);
     format->driver = strdup(driver);
     format->refcount = 0;
-
+    format->vtable = NULL;
     format->imagemode = MS_IMAGEMODE_PC256;
 
 /* -------------------------------------------------------------------- */
@@ -519,34 +598,20 @@ outputFormatObj *msSelectOutputFormat( mapObj *map,
                                        const char *imagetype )
 
 {
-    int i, index;
+    int index;
     outputFormatObj *format = NULL;
 
     if( map == NULL || imagetype == NULL || strlen(imagetype) == 0 )
         return NULL;
     
-    index = msGetOutputFormatIndex(map, imagetype);
-    if (index >= 0)
-        format = map->outputformatlist[index];
-
 /* -------------------------------------------------------------------- */
 /*      Try to find the format in the maps list of formats, first by    */
 /*      mime type, and then by output format name.                      */
 /* -------------------------------------------------------------------- */
-    for( i = 0; i < map->numoutputformats && format == NULL; i++ )
-    {
-        if( map->outputformatlist[i]->mimetype != NULL
-            && strcasecmp(imagetype,
-                          map->outputformatlist[i]->mimetype) == 0 )
-            format = map->outputformatlist[i];
-    }
-    
-    for( i = 0; i < map->numoutputformats && format == NULL; i++ )
-    {
-        if( strcasecmp(imagetype,map->outputformatlist[i]->name) == 0 )
-            format = map->outputformatlist[i];
-    }
-    
+    index = msGetOutputFormatIndex(map, imagetype);
+    if (index >= 0)
+        format = map->outputformatlist[index];
+
     if (format)
     {
         if (map->imagetype)
@@ -672,7 +737,7 @@ outputFormatObj *msCloneOutputFormat( outputFormatObj *src )
 
     dst->imagemode = src->imagemode;
     dst->renderer = src->renderer;
-
+    
     dst->transparent = src->transparent;
     dst->bands = src->bands;
 
@@ -685,6 +750,11 @@ outputFormatObj *msCloneOutputFormat( outputFormatObj *src )
 
     dst->inmapfile = src->inmapfile;
 
+    if( MS_RENDERER_PLUGIN(dst) ) 
+    {
+        msInitializeRendererVTable(dst);
+    }
+    
     return dst;
 }
 
@@ -802,12 +872,32 @@ void msGetOutputFormatMimeList( mapObj *map, char **mime_list, int max_mime )
 /************************************************************************/
 /*                     msGetOutputFormatMimeList()                      */
 /************************************************************************/
-
-void msGetOutputFormatMimeListGD( mapObj *map, char **mime_list, int max_mime )
+void msGetOutputFormatMimeListImg( mapObj *map, char **mime_list, int max_mime )
 
 {
-    int mime_count = 0, i;
+  int mime_count = 0, i,j;
+    const char *format_list = NULL;
+    char **tokens = NULL;
+    int numtokens = 0;
+    outputFormatObj *format;
 
+    format_list = msOWSLookupMetadata(&(map->web.metadata), "M","getlegendgraphic_formatlist");
+    if ( format_list && strlen(format_list) > 0)
+      tokens = msStringSplit(format_list,  ',', &numtokens);
+
+    if (tokens && numtokens > 0)
+    {
+        for(j = 0; j < numtokens; j++ )
+        {
+            format = msSelectOutputFormat(map, tokens[j]);
+            if (format != NULL)
+            {
+                mime_list[mime_count++] = format->mimetype;
+            }
+        }
+        msFreeCharArray(tokens, numtokens);
+    }
+    else
     for( i = 0; i < map->numoutputformats && mime_count < max_mime; i++ )
     {
         int  j;
@@ -832,42 +922,63 @@ void msGetOutputFormatMimeListGD( mapObj *map, char **mime_list, int max_mime )
         mime_list[mime_count] = NULL;
 }
 
-
 /************************************************************************/
-/*                  msGetOutputFormatMimeListRaster()                   */
+/*                  msGetOutputFormatMimeListWMS()                      */
 /************************************************************************/
 
 void msGetOutputFormatMimeListWMS( mapObj *map, char **mime_list, int max_mime )
-
 {
-    int mime_count = 0, i;
+    int mime_count = 0, i,j;
+    const char *format_list = NULL;
+    char **tokens = NULL;
+    int numtokens = 0;
+    outputFormatObj *format;
 
-    for( i = 0; i < map->numoutputformats && mime_count < max_mime; i++ )
+    format_list = msOWSLookupMetadata(&(map->web.metadata), "M","getmap_formatlist");
+    if ( format_list && strlen(format_list) > 0)
+      tokens = msStringSplit(format_list,  ',', &numtokens);
+
+    if (tokens && numtokens > 0)
     {
-        int  j;
-        
-        if( map->outputformatlist[i]->mimetype == NULL )
-            continue;
-
-        for( j = 0; j < mime_count; j++ )
+        for(j = 0; j < numtokens; j++ )
         {
-            if( strcasecmp(mime_list[j],
-                           map->outputformatlist[i]->mimetype) == 0 )
-                break;
+            format = msSelectOutputFormat(map, tokens[j]);
+            if (format != NULL)
+            {
+                mime_list[mime_count++] = format->mimetype;
+            }
         }
-
-        if( j == mime_count && map->outputformatlist[i]->driver &&
-            (strncasecmp(map->outputformatlist[i]->driver, "GD/", 3)==0 ||
-             strncasecmp(map->outputformatlist[i]->driver, "AGG/", 4)==0 ||
-             strncasecmp(map->outputformatlist[i]->driver, "GDAL/", 5)==0 ||
-             strcasecmp(map->outputformatlist[i]->driver, "svg")==0)) 
-            mime_list[mime_count++] = map->outputformatlist[i]->mimetype;
+        msFreeCharArray(tokens, numtokens);
     }
+    else
+    {
+        for( i = 0; i < map->numoutputformats && mime_count < max_mime; i++ )
+        {
+            int  j;
+        
+            if( map->outputformatlist[i]->mimetype == NULL )
+              continue;
 
+            for( j = 0; j < mime_count; j++ )
+            {
+                if( strcasecmp(mime_list[j],
+                               map->outputformatlist[i]->mimetype) == 0 )
+                  break;
+            }
+
+            if( j == mime_count && map->outputformatlist[i]->driver &&
+               (strncasecmp(map->outputformatlist[i]->driver, "GD/", 3)==0 ||
+                 strncasecmp(map->outputformatlist[i]->driver, "GDAL/", 5)==0 ||
+                 strncasecmp(map->outputformatlist[i]->driver, "AGG/", 4)==0 ||
+                 strcasecmp(map->outputformatlist[i]->driver, "svg")==0)) 
+              mime_list[mime_count++] = map->outputformatlist[i]->mimetype;
+        }
+    }
     if( mime_count < max_mime )
         mime_list[mime_count] = NULL;
 }
 
+
 /************************************************************************/
 /*                       msOutputFormatValidate()                       */
 /*                                                                      */
@@ -922,3 +1033,31 @@ int msOutputFormatValidate( outputFormatObj *format )
 
     return result;
 }
+
+int msInitializeRendererVTable(outputFormatObj *format) {
+    assert(format);
+    if(format->vtable) {
+        /* TODO?: clean up caches before switch 
+        msFreeRendererVTable(format->vtable); */
+        msFree(format->vtable);
+    }
+    format->vtable = (rendererVTableObj*)malloc(sizeof(rendererVTableObj));
+    
+    switch(format->renderer) {
+        case MS_RENDER_WITH_CAIRO_RASTER:
+            return msPopulateRendererVTableCairoRaster(format->vtable);
+        case MS_RENDER_WITH_CAIRO_PDF:
+            return msPopulateRendererVTableCairoPDF(format->vtable);
+        case MS_RENDER_WITH_CAIRO_SVG:
+            return msPopulateRendererVTableCairoSVG(format->vtable);
+        case MS_RENDER_WITH_OGL:
+            return msPopulateRendererVTableOGL(format->vtable);
+        default:
+            msSetError(MS_MISCERR, "unsupported RendererVtable renderer %d",
+                    "msInitializeRendererVTable()",format->renderer);
+            return MS_FAILURE;            
+    }
+    /* this code should never be executed */
+    return MS_FAILURE;
+}
+
