@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: mapows.c 8545 2009-02-13 00:37:18Z assefa $
+ * $Id: mapows.c 9513 2009-10-26 17:18:18Z warmerdam $
  *
  * Project:  MapServer
  * Purpose:  OGC Web Services (WMS, WFS) support functions
@@ -34,7 +34,7 @@
 #include <stdarg.h> 
 #include <assert.h>
 
-MS_CVSID("$Id: mapows.c 8545 2009-02-13 00:37:18Z assefa $")
+MS_CVSID("$Id: mapows.c 9513 2009-10-26 17:18:18Z warmerdam $")
 
 /*
 ** msOWSDispatch() is the entry point for any OWS request (WMS, WFS, ...)
@@ -134,6 +134,115 @@ int msOWSDispatch(mapObj *map, cgiRequestObj *request, int force_ows_mode)
                       * since we're not in force_ows_mode*/
 }
 
+
+/*
+** msOWSLookupMetadata()
+**
+** Attempts to lookup a given metadata name in multiple OWS namespaces.
+**
+** 'namespaces' is a string with a letter for each namespace to lookup 
+** in the order they should be looked up. e.g. "MO" to lookup wms_ and ows_
+** If namespaces is NULL then this function just does a regular metadata
+** lookup.
+*/
+const char *msOWSLookupMetadata(hashTableObj *metadata, 
+                                const char *namespaces, const char *name)
+{
+    const char *value = NULL;
+
+    if (namespaces == NULL)
+    {
+        value = msLookupHashTable(metadata, (char*)name);
+    }
+    else
+    {
+        char buf[100] = "ows_";
+
+        strncpy(buf+4, name, 95);
+        buf[99] = '\0';
+
+        while (value == NULL && *namespaces != '\0')
+        {
+            switch (*namespaces)
+            {
+              case 'O':         /* ows_... */
+                buf[0] = 'o';
+                buf[1] = 'w';
+                buf[2] = 's';
+                break;
+              case 'M':         /* wms_... */
+                buf[0] = 'w';
+                buf[1] = 'm';
+                buf[2] = 's';
+                break;
+              case 'F':         /* wfs_... */
+                buf[0] = 'w';
+                buf[1] = 'f';
+                buf[2] = 's';
+                break;
+              case 'C':         /* wcs_... */
+                buf[0] = 'w';
+                buf[1] = 'c';
+                buf[2] = 's';
+                break;
+              case 'G':         /* gml_... */
+                buf[0] = 'g';
+                buf[1] = 'm';
+                buf[2] = 'l';
+                break;
+                case 'S':         /* sos_... */
+                buf[0] = 's';
+                buf[1] = 'o';
+                buf[2] = 's';
+                break;
+              default:
+                /* We should never get here unless an invalid code (typo) is */
+                /* present in the code, but since this happened before... */
+                msSetError(MS_WMSERR, 
+                           "Unsupported metadata namespace code (%c).",
+                           "msOWSLookupMetadata()", *namespaces );
+                assert(MS_FALSE);
+                return NULL;
+            }
+
+            value = msLookupHashTable(metadata, buf);
+            namespaces++;
+        }
+    }
+
+    return value;
+}
+
+/*
+** msOWSLookupMetadata2()
+**
+** Attempts to lookup a given metadata name in multiple hashTables, and
+** in multiple OWS namespaces within each. First searches the primary
+** table and if no result is found, attempts the search using the 
+** secondary (fallback) table.
+**
+** 'namespaces' is a string with a letter for each namespace to lookup 
+** in the order they should be looked up. e.g. "MO" to lookup wms_ and ows_
+** If namespaces is NULL then this function just does a regular metadata
+** lookup.
+*/
+const char *msOWSLookupMetadata2(hashTableObj *pri,
+                                        hashTableObj *sec,
+                                        const char *namespaces,
+                                        const char *name)
+{
+    const char *result;
+    
+    if ((result = msOWSLookupMetadata(pri, namespaces, name)) == NULL)
+    {
+        /* Try the secondary table */
+        result = msOWSLookupMetadata(sec, namespaces, name);
+    }
+
+    return result;
+}
+
+
 #if defined(USE_WMS_SVR) || defined (USE_WFS_SVR) || defined (USE_WCS_SVR) || defined(USE_SOS_SVR) || defined(USE_WMS_LYR) || defined(USE_WFS_LYR)
 
 #if !defined(USE_PROJ)
@@ -466,113 +575,6 @@ const char *msOWSGetVersionString(int nVersion, char *pszBuffer)
 }
 
 
-/*
-** msOWSLookupMetadata()
-**
-** Attempts to lookup a given metadata name in multiple OWS namespaces.
-**
-** 'namespaces' is a string with a letter for each namespace to lookup 
-** in the order they should be looked up. e.g. "MO" to lookup wms_ and ows_
-** If namespaces is NULL then this function just does a regular metadata
-** lookup.
-*/
-const char *msOWSLookupMetadata(hashTableObj *metadata, 
-                                const char *namespaces, const char *name)
-{
-    const char *value = NULL;
-
-    if (namespaces == NULL)
-    {
-        value = msLookupHashTable(metadata, (char*)name);
-    }
-    else
-    {
-        char buf[100] = "ows_";
-
-        strncpy(buf+4, name, 95);
-        buf[99] = '\0';
-
-        while (value == NULL && *namespaces != '\0')
-        {
-            switch (*namespaces)
-            {
-              case 'O':         /* ows_... */
-                buf[0] = 'o';
-                buf[1] = 'w';
-                buf[2] = 's';
-                break;
-              case 'M':         /* wms_... */
-                buf[0] = 'w';
-                buf[1] = 'm';
-                buf[2] = 's';
-                break;
-              case 'F':         /* wfs_... */
-                buf[0] = 'w';
-                buf[1] = 'f';
-                buf[2] = 's';
-                break;
-              case 'C':         /* wcs_... */
-                buf[0] = 'w';
-                buf[1] = 'c';
-                buf[2] = 's';
-                break;
-              case 'G':         /* gml_... */
-                buf[0] = 'g';
-                buf[1] = 'm';
-                buf[2] = 'l';
-                break;
-                case 'S':         /* sos_... */
-                buf[0] = 's';
-                buf[1] = 'o';
-                buf[2] = 's';
-                break;
-              default:
-                /* We should never get here unless an invalid code (typo) is */
-                /* present in the code, but since this happened before... */
-                msSetError(MS_WMSERR, 
-                           "Unsupported metadata namespace code (%c).",
-                           "msOWSLookupMetadata()", *namespaces );
-                assert(MS_FALSE);
-                return NULL;
-            }
-
-            value = msLookupHashTable(metadata, buf);
-            namespaces++;
-        }
-    }
-
-    return value;
-}
-
-/*
-** msOWSLookupMetadata2()
-**
-** Attempts to lookup a given metadata name in multiple hashTables, and
-** in multiple OWS namespaces within each. First searches the primary
-** table and if no result is found, attempts the search using the 
-** secondary (fallback) table.
-**
-** 'namespaces' is a string with a letter for each namespace to lookup 
-** in the order they should be looked up. e.g. "MO" to lookup wms_ and ows_
-** If namespaces is NULL then this function just does a regular metadata
-** lookup.
-*/
-const char *msOWSLookupMetadata2(hashTableObj *pri,
-                                        hashTableObj *sec,
-                                        const char *namespaces,
-                                        const char *name)
-{
-    const char *result;
-    
-    if ((result = msOWSLookupMetadata(pri, namespaces, name)) == NULL)
-    {
-        /* Try the secondary table */
-        result = msOWSLookupMetadata(sec, namespaces, name);
-    }
-
-    return result;
-}
-
 /*
 ** msOWSPrintMetadata()
 **
@@ -1211,7 +1213,11 @@ void msOWSPrintLatLonBoundingBox(FILE *stream, const char *tabspace,
 
   if (nService == OWS_WMS) { /* always project to lat long */
     if (srcproj->numargs > 0 && !pj_is_latlong(srcproj->proj)) {
-      msProjectRect(srcproj, NULL, &ext);
+        projectionObj wgs84;
+        msInitProjection(&wgs84);
+        msLoadProjectionString(&wgs84, "+proj=longlat +datum=WGS84");
+        msProjectRect(srcproj, &wgs84, &ext);
+        msFreeProjection(&wgs84);
     }
   }
 
@@ -1416,7 +1422,7 @@ int msOWSGetLayerExtent(mapObj *map, layerObj *lp, const char *namespaces, rectO
     msFreeCharArray(tokens, n);
     return MS_SUCCESS;
   }
-  else if( lp->type != MS_LAYER_RASTER )
+  else
   {
       return msLayerGetExtent(lp, ext);
   }
@@ -1616,6 +1622,9 @@ const char *msOWSGetEPSGProj(projectionObj *proj, hashTableObj *metadata, const
   } else if (proj && proj->numargs > 0 && (value = strstr(proj->args[0], "init=epsg:")) != NULL && strlen(value) < 20) {
     sprintf(epsgCode, "EPSG:%s", value+10);
     return epsgCode;
+  } else if (proj && proj->numargs > 0 && (value = strstr(proj->args[0], "init=crs:")) != NULL && strlen(value) < 20) {
+    sprintf(epsgCode, "CRS:%s", value+9);
+    return epsgCode;
   } else if (proj && proj->numargs > 0 && (strncasecmp(proj->args[0], "AUTO:", 5) == 0 ||
                                            strncasecmp(proj->args[0], "AUTO2:", 6) == 0)) {
     return proj->args[0];
