@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: mappostgis.c 9052 2009-05-28 18:56:59Z pramsey $
+ * $Id: mappostgis.c 9312 2009-09-15 22:34:46Z pramsey $
  *
  * Project:  MapServer
  * Purpose:  PostGIS CONNECTIONTYPE support.
@@ -68,7 +68,7 @@
 
 #ifdef USE_POSTGIS
 
-MS_CVSID("$Id: mappostgis.c 9052 2009-05-28 18:56:59Z pramsey $")
+MS_CVSID("$Id: mappostgis.c 9312 2009-09-15 22:34:46Z pramsey $")
 
 /*
 ** msPostGISCloseConnection()
@@ -677,6 +677,10 @@ int msPostGISParseData(layerObj *layer) {
     ** primary key on the table.
     */
     if ( ! (layerinfo->uid) ) {
+        if ( strstr(layerinfo->fromsource, " ") ) {
+            msSetError(MS_QUERYERR, "Error parsing PostGIS DATA variable.  You must specifiy 'using unique' when supplying a subselect in the data definition.", "msPostGISParseData()");
+            return MS_FAILURE;
+        }
         if ( msPostGISRetrievePK(layer) != MS_SUCCESS ) {
             /* No user specified unique id so we will use the PostgreSQL oid */
             /* TODO: Deprecate this, oids are deprecated in PostgreSQL */
@@ -1041,7 +1045,6 @@ char *msPostGISBuildSQLSRID(layerObj *layer) {
         }
         strSRID = malloc(strlen(strSRIDTemplate) + strlen(f_table_name) + strlen(layerinfo->geomcolumn));
         sprintf(strSRID, strSRIDTemplate, f_table_name, layerinfo->geomcolumn);
-
         if ( f_table_name ) free(f_table_name);
     }
     return strSRID;
@@ -1203,7 +1206,6 @@ char *msPostGISBuildSQLWhere(layerObj *layer, rectObj *rect, long *uid) {
         strFilterLength = strlen(strFilter);
     }
 
-
     /* Populate strUid, if necessary. */
     if ( uid ) {
         static char *strUidTemplate = "\"%s\" = %ld";
@@ -1293,7 +1295,7 @@ char *msPostGISBuildSQL(layerObj *layer, rectObj *rect, long *uid) {
 
 }
 
-int msPostGISReadShape(layerObj *layer, shapeObj *shape) {
+int msPostGISReadShape(layerObj *layer, shapeObj *shape, int random) {
 
     char *wkbstr = NULL;
     unsigned char *wkb = NULL;
@@ -1390,16 +1392,26 @@ int msPostGISReadShape(layerObj *layer, shapeObj *shape) {
                 msDebug("msPostGISReadShape: [%s] \"%s\"\n", layer->items[t], shape->values[t]);
             }
         }
-        /* t is the geometry, t+1 is the uid */
-        tmp = PQgetvalue(layerinfo->pgresult, layerinfo->rownum, t + 1);
-        if( tmp ) {
-            uid = strtol( tmp, NULL, 10 );
+        
+        if( random ) {
+            /* t is the geometry, t+1 is the uid */
+            tmp = PQgetvalue(layerinfo->pgresult, layerinfo->rownum, t + 1);
+            if( tmp ) {
+                uid = strtol( tmp, NULL, 10 );
+            }
+            else {
+                uid = 0;
+            }
+            if( layer->debug > 4 ) {
+                msDebug("msPostGISReadShape: Setting shape->index = %d\n", uid);
+            }
+            shape->index = uid;
+        } else {
+            if( layer->debug > 4 ) {
+                msDebug("msPostGISReadShape: Setting shape->index = %d\n", layerinfo->rownum);
+            }
+            shape->index = layerinfo->rownum;
         }
-        else {
-            uid = 0;
-        }
-
-        shape->index = uid;
 
         if( layer->debug > 2 ) {
             msDebug("msPostGISReadShape: [index] %d\n",  shape->index);
@@ -1535,7 +1547,6 @@ int msPostGISLayerOpen(layerObj *layer) {
         }
     }
 
-
     /* Save the layerinfo in the layerObj. */
     layer->layerinfo = (void*)layerinfo;
 
@@ -1703,7 +1714,7 @@ int msPostGISLayerWhichShapes(layerObj *layer, rectObj rect) {
     pgresult = PQexec(layerinfo->pgconn, strSQL);
 
     if ( layer->debug > 1 ) {
-        msDebug("msPostGISLayerWhichShapes query status: %d\n", PQresultStatus(pgresult));
+        msDebug("msPostGISLayerWhichShapes query status: %s (%d)\n", PQresStatus(PQresultStatus(pgresult)), PQresultStatus(pgresult)); 
     }
 
     /* Something went wrong. */
@@ -1765,8 +1776,8 @@ int msPostGISLayerNextShape(layerObj *layer, shapeObj *shape) {
     while (shape->type == MS_SHAPE_NULL) {
         if (layerinfo->rownum < PQntuples(layerinfo->pgresult)) {
             int rv;
-            /* Retrieve this shape. */
-            rv = msPostGISReadShape(layer, shape);
+            /* Retrieve this shape, cursor access mode. */
+            rv = msPostGISReadShape(layer, shape, 0);
             if( shape->type != MS_SHAPE_NULL ) {
                 (layerinfo->rownum)++; /* move to next shape */
                 return MS_SUCCESS;
@@ -1790,6 +1801,74 @@ int msPostGISLayerNextShape(layerObj *layer, shapeObj *shape) {
 #endif
 }
 
+/*
+** msPostGISLayerResultsGetShape()
+**
+** Registered vtable->LayerGetShape function. For pulling from a prepared and 
+** undisposed result set. We ignore the 'tile' parameter, as it means nothing to us.
+*/
+int msPostGISLayerResultsGetShape(layerObj *layer, shapeObj *shape, int tile, long record) {
+#ifdef USE_POSTGIS
+    
+    PGresult *pgresult = NULL;
+    msPostGISLayerInfo *layerinfo = NULL;
+	int result = MS_SUCCESS;
+	int status;
+
+    assert(layer != NULL);
+    assert(layer->layerinfo != NULL);
+
+    if (layer->debug) {
+        msDebug("msPostGISLayerGetShape called for record = %i\n", record);
+    }
+
+    layerinfo = (msPostGISLayerInfo*) layer->layerinfo;
+
+    /* Check the validity of the open result. */
+    pgresult = layerinfo->pgresult;
+    if ( ! pgresult ) {
+        msSetError( MS_MISCERR,
+                    "PostgreSQL result set is null.",
+                    "msPostGISLayerResultsGetShape()");
+        return MS_FAILURE;
+    }
+    status = PQresultStatus(pgresult);
+    if ( layer->debug > 1 ) {
+        msDebug("msPostGISLayerResultsGetShape query status: %s (%d)\n", PQresStatus(status), status);
+    }    
+    if( ! ( status == PGRES_COMMAND_OK || status == PGRES_TUPLES_OK) ) {
+        msSetError( MS_MISCERR,
+                    "PostgreSQL result set is not ready.",
+                    "msPostGISLayerResultsGetShape()");
+        return MS_FAILURE;
+    }
+
+    /* Check the validity of the requested record number. */
+    if( record >= PQntuples(pgresult) ) {
+        msDebug("msPostGISLayerResultsGetShape got record request (%d) but only has %d tuples.\n", record, PQntuples(pgresult));
+        msSetError( MS_MISCERR,
+                    "Got request larger than result set.",
+                    "msPostGISLayerResultsGetShape()");
+        return MS_FAILURE;
+    }
+
+    layerinfo->rownum = record; /* Only return one result. */
+
+    /* We don't know the shape type until we read the geometry. */
+    shape->type = MS_SHAPE_NULL;
+
+	/* Return the shape, cursor access mode. */
+    result = msPostGISReadShape(layer, shape, 0);
+
+    return (shape->type == MS_SHAPE_NULL) ? MS_FAILURE : MS_SUCCESS;
+#else
+    msSetError( MS_MISCERR,
+                "PostGIS support is not available.",
+                "msPostGISLayerResultsGetShape()");
+    return MS_FAILURE;
+#endif
+}
+
 /*
 ** msPostGISLayerGetShape()
 **
@@ -1865,7 +1944,8 @@ int msPostGISLayerGetShape(layerObj *layer, shapeObj *shape, int tile, long reco
     }
 
     if (num_tuples > 0) {
-         result = msPostGISReadShape(layer, shape);
+        /* Get shape in random access mode. */
+        result = msPostGISReadShape(layer, shape, 1);
     }
 
     return (shape->type == MS_SHAPE_NULL) ? MS_FAILURE : ( (num_tuples > 0) ? MS_SUCCESS : MS_DONE );
@@ -2400,25 +2480,17 @@ int msPostGISLayerInitializeVirtualTable(layerObj *layer) {
     layer->vtable->LayerIsOpen = msPostGISLayerIsOpen;
     layer->vtable->LayerWhichShapes = msPostGISLayerWhichShapes;
     layer->vtable->LayerNextShape = msPostGISLayerNextShape;
+    layer->vtable->LayerResultsGetShape = msPostGISLayerResultsGetShape; 
     layer->vtable->LayerGetShape = msPostGISLayerGetShape;
-
     layer->vtable->LayerClose = msPostGISLayerClose;
-
     layer->vtable->LayerGetItems = msPostGISLayerGetItems;
     layer->vtable->LayerGetExtent = msPostGISLayerGetExtent;
-
     layer->vtable->LayerApplyFilterToLayer = msLayerApplyCondSQLFilterToLayer;
-
     /* layer->vtable->LayerGetAutoStyle, not supported for this layer */
     layer->vtable->LayerCloseConnection = msPostGISLayerClose;
-
     layer->vtable->LayerSetTimeFilter = msPostGISLayerSetTimeFilter; 
     /* layer->vtable->LayerCreateItems, use default */
     /* layer->vtable->LayerGetNumFeatures, use default */
 
-
     return MS_SUCCESS;
 }
-
-
-
