@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: mapprojhack.c 7418 2008-02-29 00:02:49Z nsavard $
+ * $Id: mapprojhack.c 9500 2009-10-21 17:55:16Z aboudreault $
  *
  * Project:  MapServer
  * Purpose:  Implementation of utility functions related to the PROJ4 library
@@ -43,8 +43,8 @@
 #include <string.h>
 #include "mapproject.h"
 
-enum MS_UNITS {MS_INCHES, MS_FEET, MS_MILES, MS_METERS, MS_KILOMETERS, MS_DD, 
-               MS_PIXELS};
+enum MS_UNITS {MS_INCHES, MS_FEET, MS_MILES, MS_METERS, MS_KILOMETERS,
+               MS_NAUTICALMILES, MS_DD, MS_PIXELS};
 
 
 /************************************************************************/
@@ -77,6 +77,10 @@ static int ConvertProjUnitStringToMS(const char *pszProjUnit)
     {
         return MS_FEET;
     }
+    else if (strcmp(pszProjUnit, "kmi") == 0)
+    {
+        return MS_NAUTICALMILES;
+    }
     
     return -1;              
 }
