@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: mapraster.c 9055 2009-05-29 03:29:47Z sdlime $
+ * $Id: mapraster.c 9440 2009-10-11 21:27:28Z warmerdam $
  *
  * Project:  MapServer
  * Purpose:  msDrawRasterLayer(): generic raster layer drawing, including
@@ -36,7 +36,7 @@
 #include "mapresample.h"
 #include "mapthread.h"
 
-MS_CVSID("$Id: mapraster.c 9055 2009-05-29 03:29:47Z sdlime $")
+MS_CVSID("$Id: mapraster.c 9440 2009-10-11 21:27:28Z warmerdam $")
 
 extern int msyyparse(void);
 extern int msyylex(void);
@@ -332,40 +332,6 @@ int msAddColorAGG(mapObj *map, gdImagePtr img, int cmt, int r, int g, int b)
 }
 #endif
 
-/************************************************************************/
-/*                        resetExtension()                              */
-/*                                                                      */
-/*      Adapted from GDAL's CPLResetExtension().                        */
-/************************************************************************/
-static char *resetExtension( const char *pszFilename, const char *pszNewExt )
-{
-  char *pszResult = (char*) malloc(strlen(pszFilename) + 1 + strlen(pszNewExt) + 1);
-  size_t  i;
-
-  strcpy(pszResult, pszFilename);
-
-  /* -------------------------------------------------------------------- */
-  /*      First, try and strip off any existing extension.                */
-  /* -------------------------------------------------------------------- */
-  for( i = strlen(pszResult) - 1; i > 0; i-- ) {
-    if( pszResult[i] == '.' ) {
-      pszResult[i] = '\0';
-      break;
-    }
-
-    if( pszResult[i] == '/' || pszResult[i] == '\\' || pszResult[i] == ':' )
-      break;
-  }
-
-  /* -------------------------------------------------------------------- */
-  /*      Append the new extension.                                       */
-  /* -------------------------------------------------------------------- */
-  strcat(pszResult, ".");
-  strcat(pszResult, pszNewExt);
-
-  return pszResult;
-}
-
 /************************************************************************/
 /*                           readWorldFile()                            */
 /*                                                                      */
@@ -373,29 +339,28 @@ static char *resetExtension( const char *pszFilename, const char *pszNewExt )
 /*      from an ESRI world file.                                        */
 /************************************************************************/
 
-static int readWorldFile(const char *filename, double *ulx, double *uly, double *cx, double *cy) {
+static int readWorldFile(char *filename, double *ulx, double *uly, double *cx, double *cy) {
   FILE *stream;
   char *wld_filename;
   int i=0;
   char buffer[BUFLEN];
 
-  wld_filename = resetExtension(filename, "wld");
+  wld_filename = strdup(filename);
+
+  strcpy(strrchr(wld_filename, '.'), ".wld");
   stream = fopen(wld_filename, "r");
-  free(wld_filename);
   if(!stream) {
-    wld_filename = resetExtension(filename, "tfw");
+    strcpy(strrchr(wld_filename, '.'), ".tfw");
     stream = fopen(wld_filename, "r");
-    free(wld_filename);
     if(!stream) {
-      wld_filename = resetExtension(filename, "jgw");
+      strcpy(strrchr(wld_filename, '.'), ".jgw");
       stream = fopen(wld_filename, "r");
-      free(wld_filename);
       if(!stream) {
-        wld_filename = resetExtension(filename, "gfw");
+        strcpy(strrchr(wld_filename, '.'), ".gfw");
         stream = fopen(wld_filename, "r");
-        free(wld_filename);
         if(!stream) {	
 	  msSetError(MS_IOERR, "Unable to open world file for reading.", "readWorldFile()");
+	  free(wld_filename);
 	  return(-1);
         }
       }
@@ -424,6 +389,7 @@ static int readWorldFile(const char *filename, double *ulx, double *uly, double
   }
     
   fclose(stream);
+  free(wld_filename);
 
   return(0);
 }
@@ -1024,6 +990,7 @@ static int drawJPEG(mapObj *map, layerObj *layer, gdImagePtr img, char *filename
   skipy = map->cellsize/cy;
 
   /* muck with scaling */
+  cinfo.scale_num = 1;
   if(MS_MIN(skipx, skipy) >= 8)
     cinfo.scale_denom = 8;
   else
@@ -1381,8 +1348,7 @@ int msDrawRasterLayerLow(mapObj *map, layerObj *layer, imageObj *image)
         goto cleanup;
     }
 
-    /* build item list (no annotation) since we may have to classify the shape, plus we want the tileitem */
-    status = msLayerWhichItems(tlp, MS_TRUE, MS_FALSE, layer->tileitem);
+    status = msLayerWhichItems(tlp, MS_FALSE, layer->tileitem);
     if(status != MS_SUCCESS)
     {
         final_status = status;
@@ -1432,12 +1398,10 @@ int msDrawRasterLayerLow(mapObj *map, layerObj *layer, imageObj *image)
 
       if(status == MS_DONE) break; /* no more tiles/images */
        
-      if(layer->data == NULL || strlen(layer->data) == 0 ) { /* assume whole filename is in attribute field */
-	(void) strncpy(tilename, tshp.values[tileitemindex], sizeof(tilename)-1);
-	tilename[sizeof(tilename)-1] = '\0';
-      } else {
-	snprintf(tilename, sizeof(tilename), "%s/%s", tshp.values[tileitemindex], layer->data);
-      }
+      if(layer->data == NULL || strlen(layer->data) == 0 ) /* assume whole filename is in attribute field */
+          strcpy( tilename, tshp.values[tileitemindex] );
+      else
+          sprintf(tilename, "%s/%s", tshp.values[tileitemindex], layer->data);
       filename = tilename;
       
       msFreeShape(&tshp); /* done with the shape */
@@ -1531,10 +1495,11 @@ int msDrawRasterLayerLow(mapObj *map, layerObj *layer, imageObj *image)
       msGDALInitialize();
 
       msAcquireLock( TLOCK_GDAL );
-      hDS = GDALOpen(szPath, GA_ReadOnly );
+      hDS = GDALOpenShared(szPath, GA_ReadOnly );
 
       if(hDS != NULL) {
-        double	adfGeoTransform[6];
+          double	adfGeoTransform[6];
+          const char *close_connection;
 
             if (layer->projection.numargs > 0 && 
                 EQUAL(layer->projection.args[0], "auto"))
@@ -1568,7 +1533,9 @@ int msDrawRasterLayerLow(mapObj *map, layerObj *layer, imageObj *image)
                 }
             }
 
+            msReleaseLock( TLOCK_GDAL );
             msGetGDALGeoTransform( hDS, map, layer, adfGeoTransform );
+            msAcquireLock( TLOCK_GDAL );
 
             /* 
             ** We want to resample if the source image is rotated, if
@@ -1609,7 +1576,23 @@ int msDrawRasterLayerLow(mapObj *map, layerObj *layer, imageObj *image)
                 break;
             }
 
-            GDALClose( hDS );
+            close_connection = msLayerGetProcessingKey( layer, 
+                                                        "CLOSE_CONNECTION" );
+            
+            /* default to keeping open for single data files, and 
+               to closing for tile indexes */
+            if( close_connection == NULL && layer->tileindex == NULL )
+                close_connection = "DEFER";
+
+            if( close_connection != NULL
+                && strcasecmp(close_connection,"DEFER") == 0 )
+            {
+                GDALDereferenceDataset( hDS );
+            }
+            else
+            {
+                GDALClose( hDS );
+            }
             msReleaseLock( TLOCK_GDAL );
             continue;
         }
