@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: maprasterquery.c 8901 2009-04-12 16:49:02Z tamas $
+ * $Id: maprasterquery.c 9502 2009-10-22 20:37:44Z warmerdam $
  *
  * Project:  MapServer
  * Purpose:  Implementation of query operations on rasters. 
@@ -32,10 +32,10 @@
 #include "mapresample.h"
 #include "mapthread.h"
 
-MS_CVSID("$Id: maprasterquery.c 8901 2009-04-12 16:49:02Z tamas $")
+MS_CVSID("$Id: maprasterquery.c 9502 2009-10-22 20:37:44Z warmerdam $")
 
-int msRASTERLayerGetShape(layerObj *layer, shapeObj *shape, int tile, 
-                          long record);
+int msRASTERLayerGetShape(layerObj *layer, shapeObj *shape, int tile, long record);
+int msRASTERLayerGetItems(layerObj *layer);
 
 #ifdef USE_GDAL
 
@@ -783,7 +783,7 @@ int msRasterQueryByRect(mapObj *map, layerObj *layer, rectObj queryRect)
             msBuildPath(szPath, tiAbsDirPath, filename); 
             free(tiAbsDirPath);
         } else {
-            msBuildPath3(szPath, map->mappath, map->shapepath, filename);
+            msTryBuildPath3(szPath, map->mappath, map->shapepath, filename);
         }
 
         msAcquireLock( TLOCK_GDAL );
@@ -865,6 +865,10 @@ int msRasterQueryByRect(mapObj *map, layerObj *layer, rectObj queryRect)
     if( status == MS_FAILURE || rlinfo->query_results == 0 )
         msRasterLayerInfoFree( layer );
 
+    /* populate the items/numitems layer-level values */
+    if( layer->layerinfo != NULL )
+        msRASTERLayerGetItems(layer);
+
     return status;
 #endif /* def USE_GDAL */
 }
@@ -906,7 +910,7 @@ int msRasterQueryByShape(mapObj *map, layerObj *layer, shapeObj *selectshape)
 /************************************************************************/
 
 int msRasterQueryByPoint(mapObj *map, layerObj *layer, int mode, pointObj p, 
-                         double buffer)
+                         double buffer, int maxresults)
 {
 #ifndef USE_GDAL
     msSetError( MS_IMGERR, 
@@ -915,6 +919,7 @@ int msRasterQueryByPoint(mapObj *map, layerObj *layer, int mode, pointObj p,
     return MS_FAILURE;
 #else
     int result;
+    int previous_maxresults;
     double layer_tolerance;
     rectObj bufferRect;
     rasterLayerInfo *rlinfo = NULL;
@@ -978,7 +983,6 @@ int msRasterQueryByPoint(mapObj *map, layerObj *layer, int mode, pointObj p,
 
         rlinfo->range_mode = MS_SINGLE;
         result = msRasterQueryByRect( map, layer, pointRect );
-
         if( rlinfo->query_results > 0 )
             return result;
     }
@@ -993,8 +997,18 @@ int msRasterQueryByPoint(mapObj *map, layerObj *layer, int mode, pointObj p,
     bufferRect.maxy = p.y + buffer;
 
     rlinfo->range_mode = mode;
+
+    if( maxresults != 0 )
+    {
+        previous_maxresults = rlinfo->query_result_hard_max;
+        rlinfo->query_result_hard_max = maxresults;
+    }
+
     result = msRasterQueryByRect( map, layer, bufferRect );
 
+    if( rlinfo != NULL && maxresults != 0 )
+        rlinfo->query_result_hard_max = previous_maxresults;
+
     return result;
 #endif /* USE_GDAL  */
 }
@@ -1252,13 +1266,6 @@ int msRASTERLayerGetShape(layerObj *layer, shapeObj *shape, int tile,
 #endif /* def USE_GDAL */
 }
 
-/************************************************************************/
-/*                       msRASTERLayerGetExtent()                       */
-/************************************************************************/
-
-int msRASTERLayerGetExtent(layerObj *layer, rectObj *extent)
-	{ return MS_FAILURE; }
-
 /************************************************************************/
 /*                       msRASTERLayerGetItems()                        */
 /************************************************************************/
@@ -1270,6 +1277,9 @@ int msRASTERLayerGetItems(layerObj *layer)
 #else
     rasterLayerInfo *rlinfo = (rasterLayerInfo *) layer->layerinfo;
 
+    if( rlinfo == NULL )
+        return MS_FAILURE;
+
     layer->items = (char **) calloc(sizeof(char *),10);
 
     layer->numitems = 0;
@@ -1303,6 +1313,74 @@ int msRASTERLayerGetItems(layerObj *layer)
 #endif /* def USE_GDAL */
 }
 
+/************************************************************************/
+/*                       msRASTERLayerGetExtent()                       */
+/************************************************************************/
+
+int msRASTERLayerGetExtent(layerObj *layer, rectObj *extent)
+
+{ 
+#ifndef USE_GDAL
+    return MS_FAILURE;
+#else
+  char szPath[MS_MAXPATHLEN];
+  mapObj *map = layer->map;
+  double adfGeoTransform[6];
+  int nXSize, nYSize;
+  GDALDatasetH hDS;
+
+  /*
+  ** For the time being we only automatically derive extents from
+  ** single raster files.  Opening many raster files from a tile index
+  ** in order to get the extent would be potentially very expensive.  We
+  ** could - in theory - scan the tile index and build up an extent from
+  ** the polygons but that is quite complicated code wise, so I am leaving
+  ** that till someone cares more about the issue.  (#79)
+  */
+  if( !layer->data 
+      || strlen(layer->data) == 0 
+      || layer->tileindex != NULL )
+  {
+    /* should we be issuing a specific error about not supporting 
+       extents for tileindexed raster layers? */
+    return MS_FAILURE;
+  }
+
+  if( map == NULL )
+    return MS_FAILURE;
+
+  msTryBuildPath3(szPath, map->mappath, map->shapepath, layer->data);
+
+  msAcquireLock( TLOCK_GDAL );
+  hDS = GDALOpen(szPath, GA_ReadOnly );
+  
+  if( hDS != NULL )
+  {
+    nXSize = GDALGetRasterXSize( hDS );
+    nYSize = GDALGetRasterYSize( hDS );
+    GDALGetGeoTransform( hDS, adfGeoTransform );
+    
+    GDALClose( hDS );
+  }
+  
+  msReleaseLock( TLOCK_GDAL );
+
+  if( hDS == NULL )
+  {
+      return MS_FAILURE;
+  }
+  
+  extent->minx = adfGeoTransform[0];
+  extent->maxy = adfGeoTransform[3];
+  
+  extent->maxx = adfGeoTransform[0] + nXSize * adfGeoTransform[1];
+  extent->miny = adfGeoTransform[3] + nYSize * adfGeoTransform[5];
+  
+  return MS_SUCCESS;
+#endif /* def USE_GDAL */
+}
+
+
 /************************************************************************/
 /*                     msRASTERLayerSetTimeFilter()                     */
 /*                                                                      */
@@ -1370,20 +1448,16 @@ msRASTERLayerInitializeVirtualTable(layerObj *layer)
     layer->vtable->LayerIsOpen = msRASTERLayerIsOpen;
     layer->vtable->LayerWhichShapes = msRASTERLayerWhichShapes;
     layer->vtable->LayerNextShape = msRASTERLayerNextShape;
+    layer->vtable->LayerResultsGetShape = msRASTERLayerGetShape; /* no special version, use ...GetShape() */
     layer->vtable->LayerGetShape = msRASTERLayerGetShape;
-
     layer->vtable->LayerClose = msRASTERLayerClose;
     layer->vtable->LayerGetItems = msRASTERLayerGetItems;
     layer->vtable->LayerGetExtent = msRASTERLayerGetExtent;
-
     /* layer->vtable->LayerGetAutoStyle, use default */
     /* layer->vtable->LayerApplyFilterToLayer, use default */
-
     layer->vtable->LayerCloseConnection = msRASTERLayerClose;
-
     /* we use backtics for proper tileindex shapefile functioning */
     layer->vtable->LayerSetTimeFilter = msRASTERLayerSetTimeFilter;
-
     /* layer->vtable->LayerCreateItems, use default */
     /* layer->vtable->LayerGetNumFeatures, use default */
 
