@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: mapresample.c 7765 2008-06-30 18:01:38Z tbonfort $
+ * $Id: mapresample.c 9296 2009-09-11 04:06:38Z warmerdam $
  *
  * Project:  MapServer
  * Purpose:  Assorted code related to resampling rasters.
@@ -31,7 +31,7 @@
 #include "mapresample.h"
 #include "mapthread.h"
 
-MS_CVSID("$Id: mapresample.c 7765 2008-06-30 18:01:38Z tbonfort $")
+MS_CVSID("$Id: mapresample.c 9296 2009-09-11 04:06:38Z warmerdam $")
 
 #ifndef MAX
 #  define MIN(a,b)      ((a<b) ? a : b)
@@ -132,8 +132,10 @@ msNearestRasterResampler( imageObj *psSrcImage, colorObj offsite,
             /*
              * We test the original floating point values to 
              * avoid errors related to asymmetric rounding around zero.
+             * (Also note bug #3120 regarding nearly redundent x/y < 0 checks).
              */
             if( x[nDstX] < 0.0 || y[nDstX] < 0.0
+                || nSrcX < 0 || nSrcY < 0 
                 || nSrcX >= nSrcXSize || nSrcY >= nSrcYSize )
             {
                 continue;
@@ -1563,7 +1565,7 @@ about AGG here, and create a GD imageObj
 /* -------------------------------------------------------------------- */
     srcImage = msImageCreate( nLoadImgXSize, nLoadImgYSize,
                               sDummyMap.outputformat, NULL, NULL, 
-                              &sDummyMap );
+                              &sDummyMap);
 
     if (srcImage == NULL)
         return -1; /* msSetError() should have been called already */
