@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: mapresample.h 1895 2003-02-24 21:25:00Z frank $
+ * $Id: mapresample.h 6428 2007-07-31 19:58:45Z dmorissette $
  *
  * Project:  CFS OGC MapServer
  * Purpose:  Definitions related to raster resampling support.
@@ -15,8 +15,8 @@
  * and/or sell copies of the Software, and to permit persons to whom the
  * Software is furnished to do so, subject to the following conditions:
  *
- * The above copyright notice and this permission notice shall be included
- * in all copies or substantial portions of the Software.
+ * The above copyright notice and this permission notice shall be included in 
+ * all copies of this Software or works derived from this Software.
  *
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
  * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
@@ -25,28 +25,13 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log$
- * Revision 1.4  2003/02/24 21:25:00  frank
- * include cpl_string.h
- *
- * Revision 1.3  2002/10/29 16:40:59  frank
- * removed prototype for function that is now static
- *
- * Revision 1.2  2002/06/21 18:33:45  frank
- * added INT16 and FLOAT image mode support
- *
- * Revision 1.1  2001/03/14 17:39:16  frank
- * New
- *
- */
+ ****************************************************************************/
 
 #ifndef RESAMPLE_H
 #define RESAMPLE_H
 
 #include <gd.h>
-#include "map.h"
+#include "mapserver.h"
 #include "mapproject.h"
 
 #ifdef USE_GDAL
