@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: mapscale.c 8633 2009-03-05 06:32:31Z sdlime $
+ * $Id: mapscale.c 9500 2009-10-21 17:55:16Z aboudreault $
  *
  * Project:  MapServer
  * Purpose:  Scale object rendering. 
@@ -29,7 +29,7 @@
 
 #include "mapserver.h"
 
-MS_CVSID("$Id: mapscale.c 8633 2009-03-05 06:32:31Z sdlime $")
+MS_CVSID("$Id: mapscale.c 9500 2009-10-21 17:55:16Z aboudreault $")
 
 #define VMARGIN 3 /* buffer around the scalebar */
 #define HMARGIN 3
@@ -39,8 +39,8 @@ MS_CVSID("$Id: mapscale.c 8633 2009-03-05 06:32:31Z sdlime $")
 /*
 ** Match this with with unit enumerations is mapserver.h
 */
-static char *unitText[5]={"in", "ft", "mi", "m", "km"};
-double inchesPerUnit[6]={1, 12, 63360.0, 39.3701, 39370.1, 4374754};
+static char *unitText[6]={"in", "ft", "mi", "m", "km", "NM"};
+double inchesPerUnit[7]={1, 12, 63360.0, 39.3701, 39370.1, 72913.3858, 4374754};
 
 
 static double roundInterval(double d)
@@ -95,6 +95,7 @@ int msCalculateScale(rectObj extent, int units, int width, int height, double re
   case(MS_METERS):    
   case(MS_KILOMETERS):
   case(MS_MILES):
+  case(MS_NAUTICALMILES):
   case(MS_INCHES):  
   case(MS_FEET):
     center_y = (extent.miny+extent.maxy)/2.0;
@@ -118,6 +119,7 @@ double msInchesPerUnit(int units, double center_lat)
   case(MS_METERS):    
   case(MS_KILOMETERS):
   case(MS_MILES):
+  case(MS_NAUTICALMILES):
   case(MS_INCHES):  
   case(MS_FEET):
     ipu = inchesPerUnit[units]; 
@@ -167,6 +169,11 @@ imageObj *msDrawScalebar(mapObj *map)
     msSetError(MS_MISCERR, "Map units not set.", "msDrawScalebar()");
     return(NULL);
   }
+
+  if(MS_RENDERER_PLUGIN(map->outputformat)) {
+    msSetError(MS_MISCERR, "Scalebar not supported yet", "msDrawScalebar()");
+    return(NULL);
+  }
 /*
  *  Allow scalebars to use TrueType fonts for labels (jnovak@novacell.com)
  *
@@ -258,7 +265,7 @@ imageObj *msDrawScalebar(mapObj *map)
 
    /* create image */
   image = msImageCreateGD(map->scalebar.width, sy, format,
-                          map->web.imagepath, map->web.imageurl);
+                          map->web.imagepath, map->web.imageurl, map->resolution, map->defresolution);
 
   /* drop this reference to output format */
   msApplyOutputFormat( &format, NULL, 
@@ -317,7 +324,7 @@ imageObj *msDrawScalebar(mapObj *map)
       if(msGetLabelSize(NULL,label,&(map->scalebar.label), &r, 
               &(map->fontset), 1, MS_FALSE,NULL) == -1) return(NULL);
       p2 = get_metrics(&p, MS_CC, r, 0,0, 0, 0, NULL);
-      msDrawTextGD(image->img.gd, p2, label, &(map->scalebar.label), &(map->fontset), 1.0);
+      msDrawTextGD(image, p2, label, &(map->scalebar.label), &(map->fontset), 1.0);
 
       state = -state;
     }
@@ -330,7 +337,7 @@ imageObj *msDrawScalebar(mapObj *map)
     if(msGetLabelSize(NULL,label,&(map->scalebar.label), &r, 
             &(map->fontset), 1, MS_FALSE,NULL) == -1) return(NULL);
     p2 = get_metrics(&p, MS_CR, r, 0,0, 0, 0, NULL);
-    msDrawTextGD(image->img.gd, p2, label, &(map->scalebar.label), &(map->fontset), 1.0);
+    msDrawTextGD(image, p2, label, &(map->scalebar.label), &(map->fontset), 1.0);
 
     break;
 
@@ -350,7 +357,7 @@ imageObj *msDrawScalebar(mapObj *map)
       if(msGetLabelSize(NULL,label,&(map->scalebar.label), &r, 
               &(map->fontset), 1, MS_FALSE,NULL) == -1) return(NULL);
       p2 = get_metrics(&p, MS_CC, r, 0,0, 0, 0, NULL);
-      msDrawTextGD(image->img.gd, p2, label, &(map->scalebar.label), &(map->fontset), 1.0);
+      msDrawTextGD(image, p2, label, &(map->scalebar.label), &(map->fontset), 1.0);
 
       state = -state;
     }
@@ -366,7 +373,7 @@ imageObj *msDrawScalebar(mapObj *map)
     if(msGetLabelSize(NULL,label,&(map->scalebar.label), &r, 
             &(map->fontset), 1, MS_FALSE,NULL) == -1) return(NULL);
     p2 = get_metrics(&p, MS_CR, r, 0,0, 0, 0, NULL);
-    msDrawTextGD(image->img.gd, p2, label, &(map->scalebar.label), &(map->fontset), 1.0);
+    msDrawTextGD(image, p2, label, &(map->scalebar.label), &(map->fontset), 1.0);
 
     break;
   default:
@@ -497,3 +504,85 @@ int msEmbedScalebar(mapObj *map, imageObj *img)
 
   return(0);
 }
+
+/************************************************************************/
+/* These two functions are used in PHP/Mapscript and Swig/Mapscript     */
+/************************************************************************/
+
+/************************************************************************/
+/*  double GetDeltaExtentsUsingScale(double scale, int units,           */
+/*                                   double centerLat, int width,       */
+/*                                   double resolution)                 */
+/*                                                                      */
+/*      Utility function to return the maximum extent using the         */
+/*      scale and the width of the image.                               */
+/*                                                                      */
+/*      Base on the function msCalculateScale (mapscale.c)              */
+/************************************************************************/
+double GetDeltaExtentsUsingScale(double scale, int units, double centerLat, int width, double resolution)
+{
+    double md = 0.0;
+    double dfDelta = -1.0;
+
+    if (scale <= 0 || width <=0)
+      return -1;
+
+    switch (units) 
+    {
+      case(MS_DD):
+      case(MS_METERS):    
+      case(MS_KILOMETERS):
+      case(MS_MILES):
+      case(MS_NAUTICALMILES):
+      case(MS_INCHES):  
+      case(MS_FEET):
+        /* remember, we use a pixel-center to pixel-center extent, hence the width-1 */
+        md = (width-1)/(resolution*msInchesPerUnit(units,centerLat));
+        dfDelta = md * scale;
+        break;
+          
+      default:
+        break;
+    }
+
+    return dfDelta;
+}
+
+/************************************************************************/
+/*    static double Pix2Georef(int nPixPos, int nPixMin, double nPixMax,*/
+/*                              double dfGeoMin, double dfGeoMax,       */
+/*                              bool bULisYOrig)                        */
+/*                                                                      */
+/*      Utility function to convert a pixel pos to georef pos. If       */
+/*      bULisYOrig parameter is set to true then the upper left is      */
+/*      considered to be the Y origin.                                  */
+/*                                                                      */
+/************************************************************************/
+double Pix2Georef(int nPixPos, int nPixMin, int nPixMax, 
+                         double dfGeoMin, double dfGeoMax, int bULisYOrig)
+{
+    double      dfWidthGeo = 0.0;
+    int         nWidthPix = 0;
+    double      dfPixToGeo = 0.0;
+    double      dfPosGeo = 0.0;
+    double      dfDeltaGeo = 0.0;
+    int         nDeltaPix = 0;
+
+    dfWidthGeo = dfGeoMax - dfGeoMin;
+    nWidthPix = nPixMax - nPixMin;
+   
+    if (dfWidthGeo > 0.0 && nWidthPix > 0)
+    {
+        dfPixToGeo = dfWidthGeo / (double)nWidthPix;
+
+        if (!bULisYOrig)
+            nDeltaPix = nPixPos - nPixMin;
+        else
+            nDeltaPix = nPixMax - nPixPos;
+        
+        dfDeltaGeo = nDeltaPix * dfPixToGeo;
+
+        dfPosGeo = dfGeoMin + dfDeltaGeo;
+    }
+    return (dfPosGeo);
+}
