@@ -1,12 +1,35 @@
+/******************************************************************************
+ * $Id: drawquery.cs 7418 2008-02-29 00:02:49Z nsavard $
+ *
+ * Project:  MapServer
+ * Purpose:  A C# based mapscript example to use the atribute query and 
+ *           highlight the results.
+ * Author:   Tamas Szekeres, szekerest@gmail.com
+ *
+ ******************************************************************************
+ * Copyright (c) 1996-2008 Regents of the University of Minnesota.
+ *
+ * Permission is hereby granted, free of charge, to any person obtaining a
+ * copy of this software and associated documentation files (the "Software"),
+ * to deal in the Software without restriction, including without limitation
+ * the rights to use, copy, modify, merge, publish, distribute, sublicense,
+ * and/or sell copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following conditions:
+ *
+ * The above copyright notice and this permission notice shall be included in 
+ * all copies of this Software or works derived from this Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
+ * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
+ * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+ * DEALINGS IN THE SOFTWARE.
+ *****************************************************************************/
+
 using System;
 using OSGeo.MapServer;
-	
-/**
- * <p>Title: Mapscript query map example.</p>
- * <p>Description: A C# based mapscript example to use the arribute query an highlight the results.</p>
- * @author Tamas Szekeres	szekerest@gmail.com
- * @version 1.0  
-*/
 
 /// <summary>
 /// A C# based mapscript example to use the arribute query an highlight the results.
@@ -21,7 +44,8 @@ class DrawQuery
     		  
     public static void Main(string[] args)
     {
-        if (args.Length < 3 || args.Length > 4) usage();
+        Console.WriteLine("");
+		if (args.Length < 3 || args.Length > 4) usage();
         
         bool ZoomToResults = (args.Length == 4 && args[3] == "-zoom");
 
@@ -95,11 +119,17 @@ class DrawQuery
                 }
             }
             // setting the map extent to the result bounds
-			if (zoomToResults) 
+			if (query_bounds != null) 
 			{
-				map.setExtent(query_bounds.minx, query_bounds.miny, query_bounds.maxx, query_bounds.maxy);
-				Console.WriteLine("Current map scale: 1:" + (int)map.scaledenom);
+				if (zoomToResults) 
+				{
+					map.setExtent(query_bounds.minx, query_bounds.miny, query_bounds.maxx, query_bounds.maxy);
+					map.scaleExtent(1.2, 0, 0); // increasing the visible area
+					Console.WriteLine("Current map scale: 1:" + (int)map.scaledenom);
+				}
 			}
+			else
+				Console.WriteLine("The query returned 0 results ...");
         }
         catch (Exception e)
         {
