@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: mapscript_i.c 9062 2009-06-02 12:38:19Z dmorissette $
+ * $Id: mapscript_i.c 9558 2009-11-20 18:11:50Z sdlime $
  *
  * Project:  MapServer
  * Purpose:  Interface file for MapServer PHP scripting extension 
@@ -154,29 +154,55 @@ int mapObj_drawLabelCache(mapObj* self, imageObj *img) {
     return msDrawLabelCache(img, self);
   }
 
-int mapObj_queryByPoint(mapObj* self, pointObj *point, 
-                         int mode, double buffer) {
-    return msQueryByPoint(self, -1, mode, *point, buffer, 0);
+int mapObj_queryByPoint(mapObj* self, pointObj *point, int mode, double buffer) {
+    msInitQuery(&(self->query));
+
+    self->query.type = MS_QUERY_BY_POINT;
+    self->query.mode = mode;
+    self->query.point = *point;
+    self->query.buffer = buffer;
+
+    return msQueryByPoint(self);
   }
 
 int mapObj_queryByRect(mapObj* self, rectObj rect) {
-    return msQueryByRect(self, -1, rect);
+    msInitQuery(&(self->query));
+
+    self->query.type = MS_QUERY_BY_RECT;
+    self->query.mode = MS_QUERY_MULTIPLE;
+    self->query.rect = rect;
+
+    return msQueryByRect(self);
   }
 
 int mapObj_queryByFeatures(mapObj* self, int slayer) {
-    return msQueryByFeatures(self, -1, slayer);
+    self->query.slayer = slayer;
+    return msQueryByFeatures(self);
   }
 
 int mapObj_queryByShape(mapObj *self, shapeObj *shape) {
-    return msQueryByShape(self, -1, shape);
+    msInitQuery(&(self->query));
+
+    self->query.type = MS_QUERY_BY_SHAPE;
+    self->query.mode = MS_QUERY_MULTIPLE;
+    self->query.shape = (shapeObj *) malloc(sizeof(shapeObj));
+    msInitShape(self->query.shape);
+    msCopyShape(shape, self->query.shape);
+
+    return msQueryByShape(self);
   }
 
-int mapObj_queryByIndex(mapObj *self, int qlayer, int tileindex, int shapeindex,
-                        int bAddToQuery) {
-    if (bAddToQuery)
-      return msQueryByIndexAdd(self, qlayer, tileindex, shapeindex);
-    else
-      return msQueryByIndex(self, qlayer, tileindex, shapeindex);
+int mapObj_queryByIndex(mapObj *self, int qlayer, int tileindex, int shapeindex, int bAddToQuery) {
+    msInitQuery(&(self->query));
+
+    self->query.type = MS_QUERY_BY_INDEX;
+    self->query.mode = MS_QUERY_SINGLE;
+    self->query.tileindex = tileindex;
+    self->query.shapeindex = shapeindex;
+    self->query.clear_resultcache = !bAddToQuery;
+    self->query.layer = qlayer;
+
+    return msQueryByIndex(self);
 }
 
 int mapObj_saveQuery(mapObj *self, char *filename) {
@@ -358,6 +384,22 @@ layerObj *mapObj_removeLayer(mapObj *self, int layerindex)
     return msRemoveLayer(self, layerindex);
 }
 
+int mapObj_setCenter(mapObj *self, pointObj *center)
+{
+    return msMapSetCenter(self, center);    
+}
+
+int mapObj_offsetExtent(mapObj *self, double x, double y)
+{
+    return msMapOffsetExtent(self, x, y);    
+}
+
+int mapObj_scaleExtent(mapObj *self, double zoomfactor, double minscaledenom, 
+                       double maxscaledenom)
+{
+    return msMapScaleExtent(self, zoomfactor, minscaledenom, maxscaledenom);    
+}
+
 /**********************************************************************
  * class extensions for layerObj, always within the context of a map
  **********************************************************************/
@@ -403,8 +445,8 @@ int layerObj_open(layerObj *self) {
 int layerObj_whichShapes(layerObj *self, rectObj *poRect) {
     int oldconnectiontype = self->connectiontype;
     self->connectiontype = MS_INLINE;
-
-    if(msLayerWhichItems(self, MS_TRUE, MS_FALSE, NULL) != MS_SUCCESS) {
+    
+    if(msLayerWhichItems(self, MS_FALSE, NULL) != MS_SUCCESS) {
         self->connectiontype = oldconnectiontype;
         return MS_FAILURE;
     }
@@ -467,23 +509,38 @@ int layerObj_drawQuery(layerObj *self, mapObj *map, imageObj *img) {
 int layerObj_queryByAttributes(layerObj *self, mapObj *map, char *qitem, char *qstring, int mode) {
     int status;
     int retval;
+
+    msInitQuery(&(map->query));
+
+    map->query.type = MS_QUERY_BY_ATTRIBUTE;
+    map->query.mode = mode;
+    if(qitem) map->query.item = strdup(qitem);
+    if(qstring) map->query.str = strdup(qstring);
+    map->query.layer = self->index;
         
     status = self->status;
     self->status = MS_ON;
-    retval = msQueryByAttributes(map, self->index, qitem, qstring, mode);
+    retval = msQueryByAttributes(map);
     self->status = status;
 
     return retval;
   }
 
-int layerObj_queryByPoint(layerObj *self, mapObj *map, 
-                          pointObj *point, int mode, double buffer) {
+int layerObj_queryByPoint(layerObj *self, mapObj *map, pointObj *point, int mode, double buffer) {
     int status;
     int retval;
-        
+
+    msInitQuery(&(map->query));
+
+    map->query.type = MS_QUERY_BY_POINT;
+    map->query.mode = mode;
+    map->query.point = *point;
+    map->query.buffer = buffer;
+    map->query.layer = self->index;
+
     status = self->status;
     self->status = MS_ON;
-    retval = msQueryByPoint(map, self->index, mode, *point, buffer, 0);
+    retval = msQueryByPoint(map);
     self->status = status;
 
     return retval;
@@ -492,10 +549,17 @@ int layerObj_queryByPoint(layerObj *self, mapObj *map,
 int layerObj_queryByRect(layerObj *self, mapObj *map, rectObj rect) {
     int status;
     int retval;
-        
+
+    msInitQuery(&(map->query));
+
+    map->query.type = MS_QUERY_BY_RECT;
+    map->query.mode = MS_QUERY_MULTIPLE;
+    map->query.rect = rect;
+    map->query.layer = self->index;
+
     status = self->status;
     self->status = MS_ON;
-    retval = msQueryByRect(map, self->index, rect);
+    retval = msQueryByRect(map);
     self->status = status;
 
     return retval;
@@ -504,10 +568,13 @@ int layerObj_queryByRect(layerObj *self, mapObj *map, rectObj rect) {
 int layerObj_queryByFeatures(layerObj *self, mapObj *map, int slayer) {
     int status;
     int retval;
-        
+
+    map->query.slayer = slayer;
+    map->query.layer = self->index;
+
     status = self->status;
     self->status = MS_ON;
-    retval = msQueryByFeatures(map, self->index, slayer);
+    retval = msQueryByFeatures(map);
     self->status = status;
 
     return retval;
@@ -516,10 +583,19 @@ int layerObj_queryByFeatures(layerObj *self, mapObj *map, int slayer) {
 int layerObj_queryByShape(layerObj *self, mapObj *map, shapeObj *shape) {
     int status;
     int retval;
-        
+
+    msInitQuery(&(map->query));
+
+    map->query.type = MS_QUERY_BY_SHAPE;
+    map->query.mode = MS_QUERY_MULTIPLE;
+    map->query.shape = (shapeObj *) malloc(sizeof(shapeObj));
+    msInitShape(map->query.shape);
+    msCopyShape(shape, map->query.shape);
+    map->query.layer = self->index;
+
     status = self->status;
     self->status = MS_ON;
-    retval = msQueryByShape(map, self->index, shape);
+    retval = msQueryByShape(map);
     self->status = status;
 
     return retval;
@@ -556,8 +632,10 @@ int layerObj_setProjection(layerObj *self, char *string) {
   }
 
 int layerObj_addFeature(layerObj *self, shapeObj *shape) {
-    if (self->features == NULL) shape->index = 0;
-    else shape->index = self->features->shape.index + 1;
+    if(self->features != NULL && self->features->tailifhead != NULL) 
+          shape->index = self->features->tailifhead->shape.index + 1;
+      else 
+          shape->index = 0;
     if(insertFeatureList(&(self->features), shape) == NULL) 
       return -1;
     else
@@ -720,6 +798,10 @@ char *classObj_getExpressionString(classObj *self) {
 }
 
 int classObj_setText(classObj *self, layerObj *layer, char *string) {
+    if (!string || strlen(string) == 0) {
+        freeExpression(&self->text);
+        return MS_SUCCESS;
+    }
     return msLoadExpressionString(&self->text, string);
 }
 
@@ -998,6 +1080,18 @@ shapeObj *shapeObj_buffer(shapeObj *self, double width)
     return msGEOSBuffer(self, width);
 }
 
+
+shapeObj *shapeObj_simplify(shapeObj *self, double tolerance)
+{
+    return msGEOSSimplify(self, tolerance);
+}
+
+
+shapeObj *shapeObj_topologypreservingsimplify(shapeObj *self, double tolerance)
+{
+    return msGEOSTopologyPreservingSimplify(self, tolerance);
+}
+
  
 shapeObj *shapeObj_convexHull(shapeObj *self)
 {
