@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: mapsde.c 8503 2009-02-05 16:20:16Z hobu $
+ * $Id: mapsde.c 9200 2009-07-22 04:20:44Z sdlime $
  *    
  * Project:  MapServer
  * Purpose:  Implements SDE CONNECTIONTYPE.
@@ -604,14 +604,23 @@ static int sdeGetRecord(layerObj *layer, shapeObj *shape) {
     
     SE_COLUMN_DEF *itemdefs;
     SE_SHAPE shapeval=0;
-    SE_BLOB_INFO blobval;
-    /* blobval = (SE_BLOB_INFO *) malloc(sizeof(SE_BLOB_INFO)); */
     msSDELayerInfo *sde;
 
+    SE_BLOB_INFO blobval;
+
+#ifdef SE_CLOB_TYPE
+    SE_CLOB_INFO clobval;
+#endif
+
+#ifdef SE_NCLOB_TYPE
+    SE_NCLOB_INFO nclobval;
+#endif
+
 #ifdef SE_NSTRING_TYPE
     SE_WCHAR* wide=NULL;
 #endif
 
+
     if(!msSDELayerIsOpen(layer)) {
         msSetError( MS_SDEERR, 
                     "SDE layer has not been opened.", 
@@ -775,6 +784,53 @@ static int sdeGetRecord(layerObj *layer, shapeObj *shape) {
             break;
             
 #endif
+
+#ifdef SE_CLOB_TYPE
+
+        case SE_CLOB_TYPE:
+            status = SE_stream_get_clob(sde->connPoolInfo->stream, (short) (i+1), &clobval);
+            if(status == SE_SUCCESS) {
+                shape->values[i] = (char *)malloc(sizeof(char)*clobval.clob_length);
+                shape->values[i] = memcpy(  shape->values[i],
+                                            clobval.clob_buffer, 
+                                            clobval.clob_length);
+                SE_clob_free(&clobval);
+            }
+            else if (status == SE_NULL_VALUE) {
+                shape->values[i] = strdup(MS_SDE_NULLSTRING);
+            }
+            else {
+                sde_error(  status,  
+                            "sdeGetRecord()", 
+                            "SE_stream_get_clob()");
+                return(MS_FAILURE);
+            }
+            break;    
+#endif
+
+#ifdef SE_CLOB_TYPE
+
+        case SE_NCLOB_TYPE:
+            status = SE_stream_get_nclob(sde->connPoolInfo->stream, (short) (i+1), &nclobval);
+            if(status == SE_SUCCESS) {
+                shape->values[i] = (char *)malloc(sizeof(char)*nclobval.nclob_length);
+                shape->values[i] = memcpy(  shape->values[i],
+                                            nclobval.nclob_buffer, 
+                                            nclobval.nclob_length);
+                SE_nclob_free(&nclobval);
+            }
+            else if (status == SE_NULL_VALUE) {
+                shape->values[i] = strdup(MS_SDE_NULLSTRING);
+            }
+            else {
+                sde_error(  status,  
+                            "sdeGetRecord()", 
+                            "SE_stream_get_nclob()");
+                return(MS_FAILURE);
+            }
+            break;    
+#endif
+
         case SE_BLOB_TYPE:
             status = SE_stream_get_blob(sde->connPoolInfo->stream, (short) (i+1), &blobval);
             if(status == SE_SUCCESS) {
@@ -794,6 +850,7 @@ static int sdeGetRecord(layerObj *layer, shapeObj *shape) {
                 return(MS_FAILURE);
             }
             break;
+            
         case SE_DATE_TYPE:
             status = SE_stream_get_date(sde->connPoolInfo->stream, (short)(i+1), &dateval);
             if(status == SE_SUCCESS) {
@@ -2122,6 +2179,7 @@ PluginInitializeVirtualTable(layerVTableObj* vtable, layerObj *layer)
     vtable->LayerIsOpen = msSDELayerIsOpen;
     vtable->LayerWhichShapes = msSDELayerWhichShapes;
     vtable->LayerNextShape = msSDELayerNextShape;
+    vtable->LayerResultsGetShape = msSDELayerGetShapeVT; /* no special version, use ...GetShape() */
     vtable->LayerGetShape = msSDELayerGetShapeVT;
     vtable->LayerClose = msSDELayerClose;
     vtable->LayerGetItems = msSDELayerGetItems;
@@ -2154,17 +2212,14 @@ msSDELayerInitializeVirtualTable(layerObj *layer)
     layer->vtable->LayerIsOpen = msSDELayerIsOpen;
     layer->vtable->LayerWhichShapes = msSDELayerWhichShapes;
     layer->vtable->LayerNextShape = msSDELayerNextShape;
+    layer->vtable->LayerResultsGetShape = msSDELayerGetShapeVT; /* no special version, use ...GetShape() */
     layer->vtable->LayerGetShape = msSDELayerGetShapeVT;
     layer->vtable->LayerClose = msSDELayerClose;
     layer->vtable->LayerGetItems = msSDELayerGetItems;
     layer->vtable->LayerGetExtent = msSDELayerGetExtent;
-
     /* layer->vtable->LayerGetAutoStyle, use default */
     /* layer->vtable->LayerApplyFilterToLayer, use default */
-
-    /* SDE uses pooled connections, close from msCloseConnections */
-    layer->vtable->LayerCloseConnection = msSDELayerCloseConnection;
-
+    layer->vtable->LayerCloseConnection = msSDELayerCloseConnection; /* SDE uses pooled connections, close from msCloseConnections */
     layer->vtable->LayerSetTimeFilter = msLayerMakePlainTimeFilter;
     layer->vtable->LayerCreateItems = msSDELayerCreateItems;
     /* layer->vtable->LayerGetNumFeatures, use default */
