@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: mapsearch.c 7995 2008-10-23 03:19:03Z sdlime $
+ * $Id: mapsearch.c 9514 2009-10-28 14:43:04Z sdlime $
  *
  * Project:  MapServer
  * Purpose:  Various geospatial search operations.
@@ -35,7 +35,7 @@
 
 #include "mapserver.h"
 
-MS_CVSID("$Id: mapsearch.c 7995 2008-10-23 03:19:03Z sdlime $")
+MS_CVSID("$Id: mapsearch.c 9514 2009-10-28 14:43:04Z sdlime $")
 
 #define LASTVERT(v,n)  ((v) == 0 ? n-2 : v-1)
 #define NEXTVERT(v,n)  ((v) == n-2 ? 0 : v+1)
@@ -228,56 +228,48 @@ int msIntersectPolylines(shapeObj *line1, shapeObj *line2) {
   for(c1=0; c1<line1->numlines; c1++)
     for(v1=1; v1<line1->line[c1].numpoints; v1++)
       for(c2=0; c2<line2->numlines; c2++)
-  for(v2=1; v2<line2->line[c2].numpoints; v2++)
-    if(msIntersectSegments(&(line1->line[c1].point[v1-1]), &(line1->line[c1].point[v1]), &(line2->line[c2].point[v2-1]), &(line2->line[c2].point[v2])) ==  MS_TRUE)
-      return(MS_TRUE);
+        for(v2=1; v2<line2->line[c2].numpoints; v2++)
+          if(msIntersectSegments(&(line1->line[c1].point[v1-1]), &(line1->line[c1].point[v1]),
+              &(line2->line[c2].point[v2-1]), &(line2->line[c2].point[v2])) ==  MS_TRUE)
+            return(MS_TRUE);
 
   return(MS_FALSE);
 }
 
 int msIntersectPolylinePolygon(shapeObj *line, shapeObj *poly) {
-  int c1,v1,c2,v2;
+  int i;
 
   /* STEP 1: polygon might competely contain the polyline or one of it's parts (only need to check one point from each part) */
-  for(c1=0; c1<line->numlines; c1++) {
-    if(msIntersectPointPolygon(&(line->line[c1].point[0]), poly) == MS_TRUE) /* this considers holes and multiple parts */
+  for(i=0; i<line->numlines; i++) {
+    if(msIntersectPointPolygon(&(line->line[i].point[0]), poly) == MS_TRUE) /* this considers holes and multiple parts */
       return(MS_TRUE);
   }
 
-
   /* STEP 2: look for intersecting line segments */
-  for(c1=0; c1<line->numlines; c1++)
-    for(v1=1; v1<line->line[c1].numpoints; v1++)
-      for(c2=0; c2<poly->numlines; c2++)
-  for(v2=1; v2<poly->line[c2].numpoints; v2++)
-    if(msIntersectSegments(&(line->line[c1].point[v1-1]), &(line->line[c1].point[v1]), &(poly->line[c2].point[v2-1]), &(poly->line[c2].point[v2])) ==  MS_TRUE)
-      return(MS_TRUE);
-  
+  if (msIntersectPolylines(line, poly) == MS_TRUE)
+    return (MS_TRUE);
+
   return(MS_FALSE);
 }
 
 int msIntersectPolygons(shapeObj *p1, shapeObj *p2) {
-  int c1,v1,c2,v2;
+  int i;
 
   /* STEP 1: polygon 1 completely contains 2 (only need to check one point from each part) */
-  for(c2=0; c2<p2->numlines; c2++) {
-    if(msIntersectPointPolygon(&(p2->line[c2].point[0]), p1) == MS_TRUE) /* this considers holes and multiple parts */
+  for(i=0; i<p2->numlines; i++) {
+    if(msIntersectPointPolygon(&(p2->line[i].point[0]), p1) == MS_TRUE) /* this considers holes and multiple parts */
       return(MS_TRUE);
   }
 
   /* STEP 2: polygon 2 completely contains 1 (only need to check one point from each part) */
-  for(c1=0; c1<p1->numlines; c1++) {
-    if(msIntersectPointPolygon(&(p1->line[c1].point[0]), p2) == MS_TRUE) /* this considers holes and multiple parts */
+  for(i=0; i<p1->numlines; i++) {
+    if(msIntersectPointPolygon(&(p1->line[i].point[0]), p2) == MS_TRUE) /* this considers holes and multiple parts */
       return(MS_TRUE);
   }
 
   /* STEP 3: look for intersecting line segments */
-  for(c1=0; c1<p1->numlines; c1++)
-    for(v1=1; v1<p1->line[c1].numpoints; v1++)
-      for(c2=0; c2<p2->numlines; c2++)
-  for(v2=1; v2<p2->line[c2].numpoints; v2++)
-    if(msIntersectSegments(&(p1->line[c1].point[v1-1]), &(p1->line[c1].point[v1]), &(p2->line[c2].point[v2-1]), &(p2->line[c2].point[v2])) ==  MS_TRUE)     
-      return(MS_TRUE);
+  if (msIntersectPolylines(p1, p2) == MS_TRUE)
+    return(MS_TRUE);
 
   /*
   ** At this point we know there are are no intersections between edges. There may be other tests necessary
@@ -295,34 +287,50 @@ int msIntersectPolygons(shapeObj *p1, shapeObj *p2) {
 double msDistancePointToPoint(pointObj *a, pointObj *b)
 {
   double d;
+
+  d = sqrt(msSquareDistancePointToPoint(a, b));
+
+  return(d);
+}
+
+/*
+** Quickly compute the square of the distance; avoids expensive sqrt() call on each invocation
+*/
+double msSquareDistancePointToPoint(pointObj *a, pointObj *b)
+{
   double dx, dy;
   
   dx = a->x - b->x;
   dy = a->y - b->y;
-  d = sqrt(dx*dx + dy*dy);
-  return(d);
+
+  return(dx*dx + dy*dy);
 }
 
 double msDistancePointToSegment(pointObj *p, pointObj *a, pointObj *b)
 {
-  double l; /* length of line ab */
+	return (sqrt(msSquareDistancePointToSegment(p, a, b)));
+}
+
+double msSquareDistancePointToSegment(pointObj *p, pointObj *a, pointObj *b)
+{
+  double l_squared; /* squared length of line ab */
   double r,s;
 
-  l = msDistancePointToPoint(a,b);
+  l_squared = msSquareDistancePointToPoint(a,b);
 
-  if(l == 0.0) /* a = b */
-    return( msDistancePointToPoint(a,p));
+  if(l_squared == 0.0) /* a = b */
+    return(msSquareDistancePointToPoint(a,p));
 
-  r = ((a->y - p->y)*(a->y - b->y) - (a->x - p->x)*(b->x - a->x))/(l*l);
+  r = ((a->y - p->y)*(a->y - b->y) - (a->x - p->x)*(b->x - a->x))/(l_squared);
 
   if(r > 1) /* perpendicular projection of P is on the forward extention of AB */
-    return(MS_MIN(msDistancePointToPoint(p, b),msDistancePointToPoint(p, a)));
+    return(MS_MIN(msSquareDistancePointToPoint(p, b),msSquareDistancePointToPoint(p, a)));
   if(r < 0) /* perpendicular projection of P is on the backward extention of AB */
-    return(MS_MIN(msDistancePointToPoint(p, b),msDistancePointToPoint(p, a)));
+    return(MS_MIN(msSquareDistancePointToPoint(p, b),msSquareDistancePointToPoint(p, a)));
 
-  s = ((a->y - p->y)*(b->x - a->x) - (a->x - p->x)*(b->y - a->y))/(l*l);
+  s = ((a->y - p->y)*(b->x - a->x) - (a->x - p->x)*(b->y - a->y))/l_squared;
 
-  return(fabs(s*l));
+  return(fabs(s*s*l_squared));
 }
 
 #define SMALL_NUMBER 0.00000001
@@ -429,6 +437,18 @@ double msDistanceSegmentToSegment(pointObj *pa, pointObj *pb, pointObj *pc, poin
 }
 
 double msDistancePointToShape(pointObj *point, shapeObj *shape)
+{
+  double d;
+
+  d = msSquareDistancePointToShape(point, shape);
+
+  return(sqrt(d));
+}
+
+/*
+** As msDistancePointToShape; avoid expensive sqrt calls
+*/
+double msSquareDistancePointToShape(pointObj *point, shapeObj *shape)
 {
   int i, j;
   double dist, minDist=-1;
@@ -437,7 +457,7 @@ double msDistancePointToShape(pointObj *point, shapeObj *shape)
   case(MS_SHAPE_POINT):
     for(j=0;j<shape->numlines;j++) {
       for(i=0; i<shape->line[j].numpoints; i++) {
-        dist = msDistancePointToPoint(point, &(shape->line[j].point[i]));
+        dist = msSquareDistancePointToPoint(point, &(shape->line[j].point[i]));
         if((dist < minDist) || (minDist < 0)) minDist = dist;
       }
     }
@@ -445,7 +465,7 @@ double msDistancePointToShape(pointObj *point, shapeObj *shape)
   case(MS_SHAPE_LINE):
     for(j=0;j<shape->numlines;j++) {
       for(i=1; i<shape->line[j].numpoints; i++) {
-        dist = msDistancePointToSegment(point, &(shape->line[j].point[i-1]), &(shape->line[j].point[i]));
+        dist = msSquareDistancePointToSegment(point, &(shape->line[j].point[i-1]), &(shape->line[j].point[i]));
         if((dist < minDist) || (minDist < 0)) minDist = dist;
       }
     }
@@ -456,7 +476,7 @@ double msDistancePointToShape(pointObj *point, shapeObj *shape)
     else { /* treat shape just like a line */
       for(j=0;j<shape->numlines;j++) {
         for(i=1; i<shape->line[j].numpoints; i++) {
-          dist = msDistancePointToSegment(point, &(shape->line[j].point[i-1]), &(shape->line[j].point[i]));
+          dist = msSquareDistancePointToSegment(point, &(shape->line[j].point[i-1]), &(shape->line[j].point[i]));
           if((dist < minDist) || (minDist < 0)) minDist = dist;
         }
       }
@@ -478,22 +498,24 @@ double msDistanceShapeToShape(shapeObj *shape1, shapeObj *shape2)
   case(MS_SHAPE_POINT): /* shape1 */
     for(i=0;i<shape1->numlines;i++) {
       for(j=0; j<shape1->line[i].numpoints; j++) {
-        dist = msDistancePointToShape(&(shape1->line[i].point[j]), shape2);
+        dist = msSquareDistancePointToShape(&(shape1->line[i].point[j]), shape2);
         if((dist < minDist) || (minDist < 0)) 
     minDist = dist;
       }
     }
+    minDist = sqrt(minDist);
     break;
   case(MS_SHAPE_LINE): /* shape1 */
     switch(shape2->type) {
     case(MS_SHAPE_POINT):
       for(i=0;i<shape2->numlines;i++) {
         for(j=0; j<shape2->line[i].numpoints; j++) {
-          dist = msDistancePointToShape(&(shape2->line[i].point[j]), shape1);
+          dist = msSquareDistancePointToShape(&(shape2->line[i].point[j]), shape1);
           if((dist < minDist) || (minDist < 0)) 
       minDist = dist;
         }
       }
+      minDist = sqrt(minDist);
       break;
     case(MS_SHAPE_LINE):
       for(i=0;i<shape1->numlines;i++) {
@@ -545,11 +567,12 @@ double msDistanceShapeToShape(shapeObj *shape1, shapeObj *shape2)
     case(MS_SHAPE_POINT):
       for(i=0;i<shape2->numlines;i++) {
         for(j=0; j<shape2->line[i].numpoints; j++) {
-          dist = msDistancePointToShape(&(shape2->line[i].point[j]), shape1);
+          dist = msSquareDistancePointToShape(&(shape2->line[i].point[j]), shape1);
           if((dist < minDist) || (minDist < 0)) 
       minDist = dist;
         }
       }
+      minDist = sqrt(minDist);
       break;
     case(MS_SHAPE_LINE):
       /* shape1 (the polygon) could contain shape2 or one of it's parts       */
