@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: mapserv.c 9014 2009-05-14 19:52:59Z sdlime $
+ * $id: mapserv.c 9470 2009-10-16 16:09:31Z sdlime $
  *
  * Project:  MapServer
  * Purpose:  MapServer CGI mainline.
@@ -39,8 +39,7 @@
 #include <signal.h>
 #endif
 
-MS_CVSID("$Id: mapserv.c 9014 2009-05-14 19:52:59Z sdlime $")
-
+MS_CVSID("$Id: mapserv.c 9649 2010-01-02 17:21:10Z sdlime $")
 
 mapservObj* mapserv;
 
@@ -176,13 +175,15 @@ static double getNumeric(char *s)
   return rv;
 }
 
+
+
 /*
 ** Extract Map File name from params and load it.  
 ** Returns map object or NULL on error.
 */
 mapObj *loadMap(void)
 {
-  int i,j,k;
+  int i,j;
   mapObj *map = NULL;
   char *tmpstr, *key, *value=NULL;
 
@@ -243,32 +244,23 @@ mapObj *loadMap(void)
     sprintf(key,"%s_validation_pattern", mapserv->request->ParamNames[i]);
         
     for(j=0; j<map->numlayers; j++) {
-      value = msLookupHashTable(&(GET_LAYER(map, j)->metadata), key);
+      layerObj *layer = GET_LAYER(map, j);
+      value = msLookupHashTable(&(layer->metadata), key);
       if(value) { /* validate parameter value */
         if(msEvalRegex(value, mapserv->request->ParamValues[i]) == MS_FALSE) {
           msSetError(MS_WEBERR, "Parameter '%s' value fails to validate.", "loadMap()", mapserv->request->ParamNames[i]);
           writeError();
         }
       }
-
-      if(GET_LAYER(map, j)->data && (strstr(GET_LAYER(map, j)->data, tmpstr) != NULL)) 
-        GET_LAYER(map, j)->data = msReplaceSubstring(GET_LAYER(map, j)->data, tmpstr, mapserv->request->ParamValues[i]);
-      if(GET_LAYER(map, j)->tileindex && (strstr(GET_LAYER(map, j)->tileindex, tmpstr) != NULL)) 
-        GET_LAYER(map, j)->tileindex = msReplaceSubstring(GET_LAYER(map, j)->tileindex, tmpstr, mapserv->request->ParamValues[i]);
-      if(GET_LAYER(map, j)->connection && (strstr(GET_LAYER(map, j)->connection, tmpstr) != NULL)) 
-        GET_LAYER(map, j)->connection = msReplaceSubstring(GET_LAYER(map, j)->connection, tmpstr, mapserv->request->ParamValues[i]);
-      if(GET_LAYER(map, j)->filter.string && (strstr(GET_LAYER(map, j)->filter.string, tmpstr) != NULL)) 
-        GET_LAYER(map, j)->filter.string = msReplaceSubstring(GET_LAYER(map, j)->filter.string, tmpstr, mapserv->request->ParamValues[i]);
-      for(k=0; k<GET_LAYER(map, j)->numclasses; k++) {
-        if(GET_LAYER(map, j)->class[k]->expression.string && (strstr(GET_LAYER(map, j)->class[k]->expression.string, tmpstr) != NULL)) 
-          GET_LAYER(map, j)->class[k]->expression.string = msReplaceSubstring(GET_LAYER(map, j)->class[k]->expression.string, tmpstr, mapserv->request->ParamValues[i]);
-      }
+      msLayerSubstituteString(layer, tmpstr, mapserv->request->ParamValues[i]);
     }
     
     free(tmpstr);
     free(key);
   }
 
+  msApplyDefaultSubstitutions(map);
+
   /* check to see if a ogc map context is passed as argument. if there */
   /* is one load it */
 
@@ -287,6 +279,7 @@ mapObj *loadMap(void)
   return map;
 }
 
+
 /*
 ** Set operation mode. First look in MS_MODE env. var. as a
 ** default value that can be overridden by the mode=... CGI param.
@@ -533,41 +526,64 @@ void loadForm(void)
       continue;
     }
 
-    if(strcasecmp(mapserv->request->ParamNames[i],"mapshape") == 0) { /* query shape */
-      lineObj line={0,NULL};
-      char **tmp=NULL;
-      int n, j;
+    /*
+    ** Query shape consisting of map or image coordinates. It's almost identical processing so we'll do either in this block...
+    */
+    if(strcasecmp(mapserv->request->ParamNames[i], "mapshape") == 0 || strcasecmp(mapserv->request->ParamNames[i], "imgshape") == 0) {
+      if(strcasecmp(mapserv->request->ParamNames[i],"mapshape") == 0)
+        QueryCoordSource = FROMUSERSHAPE;
+      else
+        QueryCoordSource = FROMIMGSHAPE;
+
+      if(strchr(mapserv->request->ParamValues[i], '(') != NULL) { /* try WKT */
+        if((mapserv->map->query.shape = msShapeFromWKT(mapserv->request->ParamValues[i])) == NULL) {
+          msSetError(MS_WEBERR, "WKT parse failed for mapshape/imgshape.", "loadForm()");
+          writeError();
+        }
+      } else {
+        lineObj line={0,NULL};
+        char **tmp=NULL;
+        int n, j;
       
-      tmp = msStringSplit(mapserv->request->ParamValues[i], ' ', &n);
+        tmp = msStringSplit(mapserv->request->ParamValues[i], ' ', &n);
 
-      if((line.point = (pointObj *)malloc(sizeof(pointObj)*(n/2))) == NULL) {
-        msSetError(MS_MEMERR, NULL, "loadForm()");
-        writeError();
-      }
-      line.numpoints = n/2;
+        if(n%2 != 0 || n<8) { /* n must be even and be at least 8 */
+          msSetError(MS_WEBERR, "Malformed polygon geometry for mapshape/imgshape.", "loadForm()");
+          writeError();
+        }
 
-      msInitShape(&(mapserv->SelectShape));
-      mapserv->SelectShape.type = MS_SHAPE_POLYGON;
+        line.numpoints = n/2;
+        if((line.point = (pointObj *)malloc(sizeof(pointObj)*line.numpoints)) == NULL) {
+          msSetError(MS_MEMERR, NULL, "loadForm()");
+          writeError();
+        }
 
-      for(j=0; j<n/2; j++) {
-        line.point[j].x = atof(tmp[2*j]);
-        line.point[j].y = atof(tmp[2*j+1]);
+        if((mapserv->map->query.shape = (shapeObj *) malloc(sizeof(shapeObj))) == NULL) {
+          msSetError(MS_MEMERR, NULL, "loadForm()");
+          writeError();
+	}
+        msInitShape(mapserv->map->query.shape);
+        mapserv->map->query.shape->type = MS_SHAPE_POLYGON;
+
+        for(j=0; j<line.numpoints; j++) {
+          line.point[j].x = atof(tmp[2*j]);
+          line.point[j].y = atof(tmp[2*j+1]);
 
 #ifdef USE_PROJ
-        if(mapserv->map->projection.proj && !pj_is_latlong(mapserv->map->projection.proj)
-           && (line.point[j].x >= -180.0 && line.point[j].x <= 180.0) 
-           && (line.point[j].y >= -90.0 && line.point[j].y <= 90.0)) {
-          msProjectPoint(&(mapserv->map->latlon), &(mapserv->map->projection), &line.point[j]); /* point is a in lat/lon */
-        }
+          if(QueryCoordSource == FROMUSERSHAPE && mapserv->map->projection.proj && !pj_is_latlong(mapserv->map->projection.proj)
+             && (line.point[j].x >= -180.0 && line.point[j].x <= 180.0) 
+             && (line.point[j].y >= -90.0 && line.point[j].y <= 90.0)) {
+            msProjectPoint(&(mapserv->map->latlon), &(mapserv->map->projection), &line.point[j]); /* point is a in lat/lon */
+          }
 #endif
+        }
+
+        if(msAddLine(mapserv->map->query.shape, &line) == -1) writeError();
+
+        msFree(line.point);
+        msFreeCharArray(tmp, n);
       }
 
-      if(msAddLine(&mapserv->SelectShape, &line) == -1) writeError();
-
-      msFree(line.point);    
-      msFreeCharArray(tmp, n);
-
-      QueryCoordSource = FROMUSERSHAPE;
       continue;
     }
 
@@ -652,36 +668,6 @@ void loadForm(void)
       continue;
     }
 
-    if(strcasecmp(mapserv->request->ParamNames[i],"imgshape") == 0) { /* shape given in image coordinates */
-      lineObj line={0,NULL};
-      char **tmp=NULL;
-      int n, j;
-      
-      tmp = msStringSplit(mapserv->request->ParamValues[i], ' ', &n);
-
-      if((line.point = (pointObj *)malloc(sizeof(pointObj)*(n/2))) == NULL) {
-        msSetError(MS_MEMERR, NULL, "loadForm()");
-        writeError();
-      }
-      line.numpoints = n/2;
-
-      msInitShape(&mapserv->SelectShape);
-      mapserv->SelectShape.type = MS_SHAPE_POLYGON;
-
-      for(j=0; j<n/2; j++) {
-        line.point[j].x = atof(tmp[2*j]);
-        line.point[j].y = atof(tmp[2*j+1]);
-      }
-
-      if(msAddLine(&mapserv->SelectShape, &line) == -1) writeError();
-
-      msFree(line.point);
-      msFreeCharArray(tmp, n);
-
-      QueryCoordSource = FROMIMGSHAPE;
-      continue;
-    }
-
     if(strcasecmp(mapserv->request->ParamNames[i],"ref.x") == 0) { /* mouse click in reference image, in pieces */
       mapserv->RefPnt.x = getNumeric(mapserv->request->ParamValues[i]);      
       if((mapserv->RefPnt.x > (2*mapserv->map->maxsize)) || (mapserv->RefPnt.x < (-2*mapserv->map->maxsize))) {
@@ -1179,6 +1165,10 @@ int main(int argc, char *argv[]) {
       /* Debugging hook... pass "QUERY_STRING=..." on the command-line */
       putenv( "REQUEST_METHOD=GET" );
       putenv( argv[iArg] );
+    } else if( strncmp(argv[iArg], "MS_ERRORFILE=", 13) == 0 ) {
+      msSetErrorFile( argv[iArg] + 13 );
+    } else if( strncmp(argv[iArg], "MS_DEBUGLEVEL=", 14) == 0) {
+      msSetGlobalDebugLevel( atoi(argv[iArg] + 14) );
     } else {
       /* we don't produce a usage message as some web servers pass junk arguments */
     }
@@ -1375,7 +1365,7 @@ int main(int argc, char *argv[]) {
       /* -------------------------------------------------------------------- */
       /*      generate map, legend, scalebar and refernce images.             */
       /* -------------------------------------------------------------------- */
-      if(msGenerateImages(mapserv, MS_FALSE, MS_TRUE) == MS_FALSE)
+      if(msGenerateImages(mapserv, MS_FALSE, MS_TRUE) != MS_SUCCESS)
         writeError();
       
       if(QueryFile) {
@@ -1511,10 +1501,10 @@ int main(int argc, char *argv[]) {
       /* initialize the legend image */
 #ifdef USE_AGG
       if(MS_RENDERER_AGG(mapserv->map->outputformat))
-        img = msImageCreateAGG(mapserv->map->legend.keysizex, mapserv->map->legend.keysizey, format, mapserv->map->web.imagepath, mapserv->map->web.imageurl);        
+         img = msImageCreateAGG(mapserv->map->legend.keysizex, mapserv->map->legend.keysizey, format, mapserv->map->web.imagepath, mapserv->map->web.imageurl, mapserv->map->resolution, mapserv->map->defresolution);        
       else
 #endif
-        img = msImageCreateGD(mapserv->map->legend.keysizex, mapserv->map->legend.keysizey, format, mapserv->map->web.imagepath, mapserv->map->web.imageurl);
+        img = msImageCreateGD(mapserv->map->legend.keysizex, mapserv->map->legend.keysizey, format, mapserv->map->web.imagepath, mapserv->map->web.imageurl, mapserv->map->resolution, mapserv->map->defresolution);
 
   /* allocate the background color */
 #ifdef USE_AGG
@@ -1538,7 +1528,6 @@ int main(int argc, char *argv[]) {
       msFreeImage(img);
 
     } else if(mapserv->Mode >= QUERY) { /* query modes */
-
       if(QueryFile) { /* already got a completed query */
         status = msLoadQuery(mapserv->map, QueryFile);
         if(status != MS_SUCCESS) writeError();
@@ -1569,14 +1558,16 @@ int main(int argc, char *argv[]) {
           if(QueryCoordSource != NONE && !mapserv->UseShapes)
             setExtent(mapserv); /* set user area of interest */
 
-          if(mapserv->Mode == ITEMFEATUREQUERY || mapserv->Mode == ITEMFEATUREQUERYMAP) {
-            if((status = msQueryByAttributes(mapserv->map, SelectLayerIndex, QueryItem, QueryString, MS_SINGLE)) != MS_SUCCESS) writeError();
-          } else {
-            if((status = msQueryByAttributes(mapserv->map, SelectLayerIndex, QueryItem, QueryString, MS_MULTIPLE)) != MS_SUCCESS) writeError();
-          }
+	  mapserv->map->query.type = MS_QUERY_BY_ATTRIBUTE;
+          if(QueryItem) mapserv->map->query.item = strdup(QueryItem);
+          if(QueryString) mapserv->map->query.str = strdup(QueryString);
 
-          if(msQueryByFeatures(mapserv->map, QueryLayerIndex, SelectLayerIndex) != MS_SUCCESS) writeError();
+          mapserv->map->query.mode = MS_QUERY_MULTIPLE;
+          if(mapserv->Mode == ITEMFEATUREQUERY || mapserv->Mode == ITEMFEATUREQUERYMAP)
+            mapserv->map->query.mode = MS_QUERY_SINGLE;
 
+          mapserv->map->query.layer = QueryLayerIndex;
+	  mapserv->map->query.slayer = SelectLayerIndex; /* this will trigger the feature query eventually */
           break;
         case FEATUREQUERY:
         case FEATURENQUERY:
@@ -1593,40 +1584,50 @@ int main(int argc, char *argv[]) {
             case FROMIMGPNT:
               mapserv->map->extent = mapserv->ImgExt; /* use the existing map extent */    
               setCoordinate();
-              if((status = msQueryByPoint(mapserv->map, SelectLayerIndex, MS_SINGLE, mapserv->mappnt, 0, 0)) != MS_SUCCESS) writeError();
               break;
-            case FROMUSERPNT: /* only a buffer makes sense */
-              if(mapserv->Buffer == -1) {
-                msSetError(MS_WEBERR, "Point given but no search buffer specified.", "mapserv()");
-                writeError();
-              }
-              if((status = msQueryByPoint(mapserv->map, SelectLayerIndex, MS_SINGLE, mapserv->mappnt, mapserv->Buffer, 0)) != MS_SUCCESS) writeError();
+            case FROMUSERPNT:
               break;
             default:
-              msSetError(MS_WEBERR, "No way to the initial search, not enough information.", "mapserv()");
+              msSetError(MS_WEBERR, "No way to perform the initial search, not enough information.", "mapserv()");
               writeError();
               break;
             }      
+
+            mapserv->map->query.type = MS_QUERY_BY_POINT;
+            mapserv->map->query.mode = MS_QUERY_SINGLE;
+
+            mapserv->map->query.point = mapserv->mappnt;
+	    mapserv->map->query.buffer = mapserv->Buffer;
+
+            mapserv->map->query.layer = QueryLayerIndex;
+	    mapserv->map->query.slayer = SelectLayerIndex; /* this will trigger the feature query eventually */
           } else { /* FEATURENQUERY/FEATURENQUERYMAP */
             switch(QueryCoordSource) {
             case FROMIMGPNT:
               mapserv->map->extent = mapserv->ImgExt; /* use the existing map extent */    
               setCoordinate();
-              if((status = msQueryByPoint(mapserv->map, SelectLayerIndex, MS_MULTIPLE, mapserv->mappnt, 0, 0)) != MS_SUCCESS) writeError();
+              mapserv->map->query.type = MS_QUERY_BY_POINT;
               break;     
             case FROMIMGBOX:
+              /* TODO: this option was present but with no code to leverage the image box... */
               break;
-            case FROMUSERPNT: /* only a buffer makes sense */
-              if((status = msQueryByPoint(mapserv->map, SelectLayerIndex, MS_MULTIPLE, mapserv->mappnt, mapserv->Buffer, 0)) != MS_SUCCESS) writeError();
+            case FROMUSERPNT:
+              mapserv->map->query.type = MS_QUERY_BY_POINT;
             default:
               setExtent(mapserv);
-              if((status = msQueryByRect(mapserv->map, SelectLayerIndex, mapserv->map->extent)) != MS_SUCCESS) writeError();
+              mapserv->map->query.type = MS_QUERY_BY_RECT;
               break;
             }
           }
-    
-          if(msQueryByFeatures(mapserv->map, QueryLayerIndex, SelectLayerIndex) != MS_SUCCESS) writeError();
-      
+
+          mapserv->map->query.mode = MS_QUERY_MULTIPLE;
+
+          mapserv->map->query.rect = mapserv->map->extent;
+          mapserv->map->query.point = mapserv->mappnt;
+          mapserv->map->query.buffer = mapserv->Buffer;
+
+          mapserv->map->query.layer = QueryLayerIndex;
+	  mapserv->map->query.slayer = SelectLayerIndex;
           break;
         case ITEMQUERY:
         case ITEMNQUERY:
@@ -1648,46 +1649,53 @@ int main(int argc, char *argv[]) {
           if(QueryCoordSource != NONE && !mapserv->UseShapes)
             setExtent(mapserv); /* set user area of interest */
 
-          if(mapserv->Mode == ITEMQUERY || mapserv->Mode == ITEMQUERYMAP) {
-            if((status = msQueryByAttributes(mapserv->map, QueryLayerIndex, QueryItem, QueryString, MS_SINGLE)) != MS_SUCCESS) writeError();
-          } else {
-            if((status = msQueryByAttributes(mapserv->map, QueryLayerIndex, QueryItem, QueryString, MS_MULTIPLE)) != MS_SUCCESS) writeError();
-          }
+	  mapserv->map->query.type = MS_QUERY_BY_ATTRIBUTE;
+	  mapserv->map->query.layer = QueryLayerIndex;
+          if(QueryItem) mapserv->map->query.item = strdup(QueryItem);
+          if(QueryString) mapserv->map->query.str = strdup(QueryString);
 
+	  mapserv->map->query.mode = MS_QUERY_MULTIPLE;
+          if(mapserv->Mode == ITEMQUERY || mapserv->Mode == ITEMQUERYMAP) mapserv->map->query.mode = MS_QUERY_SINGLE;
           break;
         case NQUERY:
         case NQUERYMAP:
+          mapserv->map->query.mode = MS_QUERY_MULTIPLE; /* all of these cases return multiple results */
+          mapserv->map->query.layer = QueryLayerIndex;
+
           switch(QueryCoordSource) {
           case FROMIMGPNT:      
             setCoordinate();
-      
+            
             if(SearchMap) { /* compute new extent, pan etc then search that extent */
               setExtent(mapserv);
               mapserv->map->cellsize = msAdjustExtent(&(mapserv->map->extent), mapserv->map->width, mapserv->map->height);
               if((status = msCalculateScale(mapserv->map->extent, mapserv->map->units, mapserv->map->width, mapserv->map->height, mapserv->map->resolution, &mapserv->map->scaledenom)) != MS_SUCCESS) writeError();
-              if((status = msQueryByRect(mapserv->map, QueryLayerIndex, mapserv->map->extent)) != MS_SUCCESS) writeError();
+              mapserv->map->query.rect = mapserv->map->extent;
+	      mapserv->map->query.type = MS_QUERY_BY_RECT; 
             } else {
               mapserv->map->extent = mapserv->ImgExt; /* use the existing image parameters */
               mapserv->map->width = mapserv->ImgCols;
               mapserv->map->height = mapserv->ImgRows;
               if((status = msCalculateScale(mapserv->map->extent, mapserv->map->units, mapserv->map->width, mapserv->map->height, mapserv->map->resolution, &mapserv->map->scaledenom)) != MS_SUCCESS) writeError();     
-              if((status = msQueryByPoint(mapserv->map, QueryLayerIndex, MS_MULTIPLE, mapserv->mappnt, 0, 0)) != MS_SUCCESS) writeError();
+              mapserv->map->query.point = mapserv->mappnt;
+              mapserv->map->query.type = MS_QUERY_BY_POINT;
             }
+
             break;      
           case FROMIMGBOX:      
             if(SearchMap) { /* compute new extent, pan etc then search that extent */
               setExtent(mapserv);
               if((status = msCalculateScale(mapserv->map->extent, mapserv->map->units, mapserv->map->width, mapserv->map->height, mapserv->map->resolution, &mapserv->map->scaledenom)) != MS_SUCCESS) writeError();
               mapserv->map->cellsize = msAdjustExtent(&(mapserv->map->extent), mapserv->map->width, mapserv->map->height);
-              if((status = msQueryByRect(mapserv->map, QueryLayerIndex, mapserv->map->extent)) != MS_SUCCESS) writeError();
+              mapserv->map->query.rect = mapserv->map->extent;
+              mapserv->map->query.type = MS_QUERY_BY_RECT;
             } else {
               double cellx, celly;
         
               mapserv->map->extent = mapserv->ImgExt; /* use the existing image parameters */
               mapserv->map->width = mapserv->ImgCols;
               mapserv->map->height = mapserv->ImgRows;
-              if((status = msCalculateScale(mapserv->map->extent, mapserv->map->units, mapserv->map->width, mapserv->map->height, mapserv->map->resolution, &mapserv->map->scaledenom)) != MS_SUCCESS) writeError();      
-        
+              if((status = msCalculateScale(mapserv->map->extent, mapserv->map->units, mapserv->map->width, mapserv->map->height, mapserv->map->resolution, &mapserv->map->scaledenom)) != MS_SUCCESS) writeError();    
               cellx = MS_CELLSIZE(mapserv->ImgExt.minx, mapserv->ImgExt.maxx, mapserv->ImgCols); /* calculate the new search extent */
               celly = MS_CELLSIZE(mapserv->ImgExt.miny, mapserv->ImgExt.maxy, mapserv->ImgRows);
               mapserv->RawExt.minx = MS_IMAGE2MAP_X(mapserv->ImgBox.minx, mapserv->ImgExt.minx, cellx);          
@@ -1695,7 +1703,8 @@ int main(int argc, char *argv[]) {
               mapserv->RawExt.maxy = MS_IMAGE2MAP_Y(mapserv->ImgBox.miny, mapserv->ImgExt.maxy, celly); /* y's are flip flopped because img/map coordinate systems are */
               mapserv->RawExt.miny = MS_IMAGE2MAP_Y(mapserv->ImgBox.maxy, mapserv->ImgExt.maxy, celly);
 
-              if((status = msQueryByRect(mapserv->map, QueryLayerIndex, mapserv->RawExt)) != MS_SUCCESS) writeError();
+              mapserv->map->query.rect = mapserv->RawExt;
+              mapserv->map->query.type = MS_QUERY_BY_RECT;
             }
             break;
           case FROMIMGSHAPE:
@@ -1706,18 +1715,19 @@ int main(int argc, char *argv[]) {
             if((status = msCalculateScale(mapserv->map->extent, mapserv->map->units, mapserv->map->width, mapserv->map->height, mapserv->map->resolution, &mapserv->map->scaledenom)) != MS_SUCCESS) writeError();
       
             /* convert from image to map coordinates here (see setCoordinate) */
-            for(i=0; i<mapserv->SelectShape.numlines; i++) {
-              for(j=0; j<mapserv->SelectShape.line[i].numpoints; j++) {
-                mapserv->SelectShape.line[i].point[j].x = MS_IMAGE2MAP_X(mapserv->SelectShape.line[i].point[j].x, mapserv->map->extent.minx, mapserv->map->cellsize);
-                mapserv->SelectShape.line[i].point[j].y = MS_IMAGE2MAP_Y(mapserv->SelectShape.line[i].point[j].y, mapserv->map->extent.maxy, mapserv->map->cellsize);
+            for(i=0; i<mapserv->map->query.shape->numlines; i++) {
+              for(j=0; j<mapserv->map->query.shape->line[i].numpoints; j++) {
+                mapserv->map->query.shape->line[i].point[j].x = MS_IMAGE2MAP_X(mapserv->map->query.shape->line[i].point[j].x, mapserv->map->extent.minx, mapserv->map->cellsize);
+                mapserv->map->query.shape->line[i].point[j].y = MS_IMAGE2MAP_Y(mapserv->map->query.shape->line[i].point[j].y, mapserv->map->extent.maxy, mapserv->map->cellsize);             
               }
             }
-      
-            if((status = msQueryByShape(mapserv->map, QueryLayerIndex, &mapserv->SelectShape)) != MS_SUCCESS) writeError();
+
+            mapserv->map->query.type = MS_QUERY_BY_SHAPE;
             break;      
           case FROMUSERPNT:
-            if(mapserv->Buffer == 0) {
-              if((status = msQueryByPoint(mapserv->map, QueryLayerIndex, MS_MULTIPLE, mapserv->mappnt, mapserv->Buffer, 0)) != MS_SUCCESS) writeError();
+            if(mapserv->Buffer == 0) { /* do a *pure* point query */
+	      mapserv->map->query.point = mapserv->mappnt;
+              mapserv->map->query.type = MS_QUERY_BY_POINT;
               setExtent(mapserv);
             } else {
               setExtent(mapserv);
@@ -1725,22 +1735,25 @@ int main(int argc, char *argv[]) {
                 if((status = msCalculateScale(mapserv->map->extent, mapserv->map->units, mapserv->map->width, mapserv->map->height, mapserv->map->resolution, &mapserv->map->scaledenom)) != MS_SUCCESS) writeError();
                 mapserv->map->cellsize = msAdjustExtent(&(mapserv->map->extent), mapserv->map->width, mapserv->map->height); 
               }
-              if((status = msQueryByRect(mapserv->map, QueryLayerIndex, mapserv->map->extent)) != MS_SUCCESS) writeError();
+              mapserv->map->query.rect = mapserv->map->extent;
+              mapserv->map->query.type = MS_QUERY_BY_RECT;
             }
             break;
           case FROMUSERSHAPE:
-            setExtent(mapserv);        
-            if((status = msQueryByShape(mapserv->map, QueryLayerIndex, &mapserv->SelectShape)) != MS_SUCCESS) writeError();
-            break;      
+            setExtent(mapserv);
+            mapserv->map->query.type = MS_QUERY_BY_SHAPE;
+            break;
           default: /* from an extent of some sort */
             setExtent(mapserv);
             if(SearchMap) { /* the extent should be tied to a map, so we need to "adjust" it */
               if((status = msCalculateScale(mapserv->map->extent, mapserv->map->units, mapserv->map->width, mapserv->map->height, mapserv->map->resolution, &mapserv->map->scaledenom)) != MS_SUCCESS) writeError();
-              mapserv->map->cellsize = msAdjustExtent(&(mapserv->map->extent), mapserv->map->width, mapserv->map->height); 
-            }        
-            if((status = msQueryByRect(mapserv->map, QueryLayerIndex, mapserv->map->extent)) != MS_SUCCESS) writeError();
+              mapserv->map->cellsize = msAdjustExtent(&(mapserv->map->extent), mapserv->map->width, mapserv->map->height);
+            }
+
+	    mapserv->map->query.rect = mapserv->map->extent;
+	    mapserv->map->query.type = MS_QUERY_BY_RECT;
             break;
-			    }      
+	  }
           break;
         case QUERY:
         case QUERYMAP:
@@ -1750,24 +1763,35 @@ int main(int argc, char *argv[]) {
             mapserv->map->extent = mapserv->ImgExt; /* use the existing image parameters */
             mapserv->map->width = mapserv->ImgCols;
             mapserv->map->height = mapserv->ImgRows;
-            if((status = msCalculateScale(mapserv->map->extent, mapserv->map->units, mapserv->map->width, mapserv->map->height, mapserv->map->resolution, &mapserv->map->scaledenom)) != MS_SUCCESS) writeError();           
-            if((status = msQueryByPoint(mapserv->map, QueryLayerIndex, MS_SINGLE, mapserv->mappnt, 0, 0)) != MS_SUCCESS) writeError();
+            if((status = msCalculateScale(mapserv->map->extent, mapserv->map->units, mapserv->map->width, mapserv->map->height, mapserv->map->resolution, &mapserv->map->scaledenom)) != MS_SUCCESS) writeError();
             break;
           case FROMUSERPNT: /* only a buffer makes sense, DOES IT? */    
             setExtent(mapserv);    
-            if((status = msQueryByPoint(mapserv->map, QueryLayerIndex, MS_SINGLE, mapserv->mappnt, mapserv->Buffer, 0)) != MS_SUCCESS) writeError();
             break;
           default:
             msSetError(MS_WEBERR, "Query mode needs a point, imgxy and mapxy are not set.", "mapserv()");
             writeError();
             break;
           }
+
+          mapserv->map->query.type = MS_QUERY_BY_POINT;
+          mapserv->map->query.mode = MS_QUERY_SINGLE;
+          mapserv->map->query.layer = QueryLayerIndex;
+          mapserv->map->query.point = mapserv->mappnt;
+          mapserv->map->query.buffer = mapserv->Buffer;          
           break;
         case INDEXQUERY:
         case INDEXQUERYMAP:
-          if((status = msQueryByIndex(mapserv->map, QueryLayerIndex, TileIndex, ShapeIndex)) != MS_SUCCESS) writeError();
+          mapserv->map->query.type = MS_QUERY_BY_POINT;
+          mapserv->map->query.mode = MS_QUERY_SINGLE;
+          mapserv->map->query.layer = QueryLayerIndex;
+          mapserv->map->query.shapeindex = ShapeIndex;
+          mapserv->map->query.tileindex = TileIndex;
           break;
         } /* end mode switch */
+
+        /* finally execute the query */
+        if((status = msExecuteQuery(mapserv->map)) != MS_SUCCESS) writeError();
       }
       
       if(mapserv->map->querymap.width != -1) mapserv->map->width = mapserv->map->querymap.width; /* make sure we use the right size */
