@@ -1,9 +1,13 @@
 /******************************************************************************
+ * $Id: mapstring.c 8268 2008-12-22 14:48:10Z aboudreault $
  *
  * Project:  MapServer
  * Purpose:  Various string handling functions.
  * Author:   Steve Lime and the MapServer team.
  *
+ * Notes: A couple of string handling functions (strrstr, strlcat) were taken from
+ * other sources. Copyright notices accompany those functions below. 
+ *
  ******************************************************************************
  * Copyright (c) 1996-2005 Regents of the University of Minnesota.
  * Copyright (c) 1998 Todd C. Miller <Todd.Miller@courtesan.com>
@@ -29,15 +33,86 @@
 
 #include "mapserver.h"
 
-MS_CVSID("$Id: mapstring.c 6662 2007-08-20 16:50:43Z hobu $")
+MS_CVSID("$Id: mapstring.c 8268 2008-12-22 14:48:10Z aboudreault $")
 
 #include <ctype.h>
+#include <string.h>
+
+/*
+ * Find the first occurrence of find in s, ignore case.
+ */
+
+#ifdef USE_FRIBIDI
+#if (defined(_WIN32) && !defined(__CYGWIN__)) || defined(USE_FRIBIDI2)
+#include "fribidi.h"
+#else
+#include <fribidi/fribidi.h>
+#endif
+#define MAX_STR_LEN 65000
+#endif
 
 #ifdef USE_ICONV
 #include <iconv.h>
 #include <wchar.h>
 #endif
 
+#include "mapentities.h"
+
+#ifdef NEED_STRRSTR
+/* 
+** Copyright (c) 2000-2004  University of Illinois Board of Trustees
+** Copyright (c) 2000-2005  Mark D. Roth
+** All rights reserved.
+**
+** Developed by: Campus Information Technologies and Educational Services,
+** University of Illinois at Urbana-Champaign
+**
+** Permission is hereby granted, free of charge, to any person obtaining
+** a copy of this software and associated documentation files (the
+** ``Software''), to deal with the Software without restriction, including
+** without limitation the rights to use, copy, modify, merge, publish,
+** distribute, sublicense, and/or sell copies of the Software, and to
+** permit persons to whom the Software is furnished to do so, subject to
+** the following conditions:
+** 
+** * Redistributions of source code must retain the above copyright
+**   notice, this list of conditions and the following disclaimers.
+**
+** * Redistributions in binary form must reproduce the above copyright
+**   notice, this list of conditions and the following disclaimers in the
+**   documentation and/or other materials provided with the distribution.
+** 
+** * Neither the names of Campus Information Technologies and Educational
+**   Services, University of Illinois at Urbana-Champaign, nor the names
+**   of its contributors may be used to endorse or promote products derived
+**   from this Software without specific prior written permission.
+**
+** THE SOFTWARE IS PROVIDED ``AS IS'', WITHOUT WARRANTY OF ANY KIND,
+** EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+** MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
+** IN NO EVENT SHALL THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR
+** ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
+** TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
+** OR THE USE OR OTHER DEALINGS WITH THE SOFTWARE.
+*/
+char *strrstr(char *string, char *find)
+{
+  size_t stringlen, findlen;
+  char *cp;
+
+  findlen = strlen(find);
+  stringlen = strlen(string);
+  if (findlen > stringlen)
+    return NULL;
+
+  for (cp = string + stringlen - findlen; cp >= string; cp--)
+    if (strncmp(cp, find, findlen) == 0)
+      return cp;
+
+  return NULL;
+}
+#endif
+
 #ifdef NEED_STRLCAT
 /*
  * Copyright (c) 1998 Todd C. Miller <Todd.Miller@courtesan.com>
@@ -90,6 +165,58 @@ size_t strlcat(char *dst, const char *src, size_t siz)
 }
 #endif
 
+#ifdef NEED_STRCASESTR
+/*-
+ * Copyright (c) 1990, 1993
+ *	The Regents of the University of California.  All rights reserved.
+ *
+ * This code is derived from software contributed to Berkeley by
+ * Chris Torek.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ * 3. Neither the name of the University nor the names of its contributors
+ *    may be used to endorse or promote products derived from this software
+ *    without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
+ * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
+ * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+ * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
+ * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+ * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+ * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
+ * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+ * SUCH DAMAGE.
+ */
+char *strcasestr(const char *s, const char *find)
+{
+	char c, sc;
+	size_t len;
+
+	if ((c = *find++) != 0) {
+		c = tolower((unsigned char)c);
+		len = strlen(find);
+		do {
+			do {
+				if ((sc = *s++) == 0)
+					return (NULL);
+			} while ((char)tolower((unsigned char)sc) != c);
+		} while (strncasecmp(s, find, len) != 0);
+		s--;
+	}
+	return ((char *)s);
+}
+#endif
+
 #ifdef NEED_STRDUP
 char	*strdup(char *s)
 {
@@ -177,13 +304,44 @@ char *msLongToString(long value) {
   return(strdup(buffer));
 }
 
-char *msDoubleToString(double value) {
+char *msDoubleToString(double value, int force_f) {
   char buffer[256]; /* plenty of space */
 
-  sprintf(buffer, "%g", value);
+  if (force_f == MS_TRUE)
+    sprintf(buffer, "%f", value);
+  else
+    sprintf(buffer, "%g", value);
   return(strdup(buffer));
 }
 
+char *msIntToString(int value) {
+  char buffer[256]; /* plenty of space */
+  sprintf(buffer, "%i", value);
+  return(strdup(buffer));
+}
+
+void msStringToUpper(char *string) {
+  int i;
+
+  if (string != NULL) {
+    for (i = 0; i < strlen(string); i++) {
+      string[i] = toupper(string[i]);
+    }
+    return;
+  }
+}
+
+void msStringToLower(char *string) {
+  int i;
+
+  if (string != NULL) {
+    for (i = 0; i < strlen(string); i++) {
+      string[i] = tolower(string[i]);
+    }
+    return;
+  }
+}
+
 char *msStringChop(char *string) {  
   int n;
 
@@ -194,10 +352,38 @@ char *msStringChop(char *string) {
   return(string);
 }
 
-/**
- * remove leading white spaces and shif evey thing to the left
- */
+/*
+** Trim leading and trailing white space.
+*/
+void msStringTrim(char *str)
+{
+    int i;
 
+    /* Send nulls home without supper. */
+    if( ! str ) return;
+
+    /* Move non-white string to the front. */
+    i = strspn(str, " ");
+    if(i) {
+        memmove(str, str + i, strlen(str) - i + 1);
+    }
+    /* Nothing left? Exit. */
+    if(strlen(str) == 0) {
+        return;
+    }
+    /* Null-terminate end of non-white string. */
+    for(i=strlen(str)-1; i>=0; i--) { /* step backwards from end */
+        if(str[i] != ' ') { 
+	        str[i+1] = '\0'; 
+	        return; 
+        }
+    }
+    return;
+}
+
+/*
+** Remove leading white spaces and shift everything to the left.
+*/
 char *msStringTrimLeft(char *string)
 {
     char *read, *write;
@@ -232,7 +418,7 @@ char *msStringTrimLeft(char *string)
 }
     
 /* ------------------------------------------------------------------------------- */
-/*       Trims leading blanks from a string                                        */
+/*       Trims trailing blanks from a string                                        */
 /* ------------------------------------------------------------------------------- */
 void msStringTrimBlanks(char *string)
 {
@@ -325,6 +511,17 @@ char *msReplaceSubstring(char *str, const char *old, const char *new)
       return(str);
 }
 
+/*
+ * same goal as msReplaceSubstring, but for the known case
+ * when we won't have to do reallocs etc
+ * used to replace the wrap characetr by a newline for labels
+ */
+void msReplaceChar(char *str, char old, char new) {
+    while(*(str++))
+        if(*str==old)
+            *str=new;
+}
+
 /*
 ** how many times does ch occur in str
 */
@@ -372,13 +569,15 @@ char *msGetPath(char *fn)
     }
   }
 
+  if(strcmp(str, fn) == 0)
+  {
+    msFree(str);
 #if defined(_WIN32) && !defined(__CYGWIN__)  
-  if(strcmp(str, fn) == 0)
-    strcpy(str, ".\\");
+    str = strdup(".\\");
 #else
-  if(strcmp(str, fn) == 0)
-    strcpy(str, "./");
+    str= strdup("./");
 #endif  
+  }
 
   return(str);
 }
@@ -556,7 +755,122 @@ char **msStringSplit(const char *string, char ch, int *num_tokens)
   return(token);
 }
 
+/* This method is similar to msStringSplit but support quoted strings. 
+   It also support multi-characters delimiter and allows to preserve quotes */
+char **msStringTokenize( const char *pszLine, const char *pszDelim, 
+                         int *num_tokens, int preserve_quote )
+{
+    char **papszResult = NULL;
+    int n = 1, iChar, nLength = strlen(pszLine), iTokenChar = 0, bInQuotes = MS_FALSE;
+    char *pszToken = (char *) malloc(sizeof(char*)*(nLength+1));
+    int nDelimLen = strlen(pszDelim);
+
+    /* Compute the number of tokens */
+    for( iChar = 0; pszLine[iChar] != '\0'; iChar++ )
+    {
+        if( bInQuotes && pszLine[iChar] == '"' && pszLine[iChar+1] == '"' )
+        {
+            iChar++;
+        }
+        else if( pszLine[iChar] == '"' )
+        {
+            bInQuotes = !bInQuotes;
+        }
+        else if ( !bInQuotes && strncmp(pszLine+iChar,pszDelim,nDelimLen) == 0 )
+        {
+            iChar += nDelimLen - 1;
+            n++;
+        }
+    }
+
+    papszResult = (char **) malloc(sizeof(char *)*n);
+    n = iTokenChar = bInQuotes = 0;
+    for( iChar = 0; pszLine[iChar] != '\0'; iChar++ )
+    {
+        if( bInQuotes && pszLine[iChar] == '"' && pszLine[iChar+1] == '"' )
+        {
+           if (preserve_quote == MS_TRUE)
+              pszToken[iTokenChar++] = '"';
+           pszToken[iTokenChar++] = '"';
+           iChar++;
+        }
+        else if( pszLine[iChar] == '"' )
+        {
+           if (preserve_quote == MS_TRUE)
+              pszToken[iTokenChar++] = '"';
+            bInQuotes = !bInQuotes;
+        }
+        else if( !bInQuotes && strncmp(pszLine+iChar,pszDelim,nDelimLen) == 0 )
+        {
+            pszToken[iTokenChar++] = '\0';
+            papszResult[n] = pszToken;
+            pszToken = (char *) malloc(sizeof(char*)*(nLength+1));
+            iChar += nDelimLen - 1;
+            iTokenChar = 0;
+            n++;
+        }
+        else
+        {
+            pszToken[iTokenChar++] = pszLine[iChar];
+        }
+    }
+
+    pszToken[iTokenChar++] = '\0';
+    papszResult[n] = pszToken;
+    
+    *num_tokens = n+1;
+
+    return papszResult;
+}
+
+/**********************************************************************
+ *                       msEncodeChar()
+ *
+ * Return 1 if the character argument should be encoded for safety
+ * in URL use and 0 otherwise. Specific character map taken from 
+ * http://www.ietf.org/rfc/rfc2396.txt
+ *
+ **********************************************************************/
+
+int msEncodeChar(const char c)
+{
+  if ( 
+       (c >= 0x61 && c <= 0x7A ) ||   /* Letters a-z */
+       (c >= 0x41 && c <= 0x5A ) ||   /* Letters A-Z */
+       (c >= 0x30 && c <= 0x39 ) ||   /* Numbers 0-9 */
+       (c >= 0x27 && c <= 0x2A ) ||   /* * ' ( )     */
+       (c >= 0x2D && c <= 0x2E ) ||   /* - .         */
+       (c == 0x5F ) ||                /* _           */
+       (c == 0x21 ) ||                /* !           */
+       (c == 0x7E ) )                 /* ~           */
+  {
+    return(0);
+  }
+  else 
+  {
+    return(1);
+  }
+}
+
 char *msEncodeUrl(const char *data)
+{
+       /*
+        * Delegate to msEncodeUrlExcept, with a null second argument
+        * to render the except handling moot.
+        */ 
+	return(msEncodeUrlExcept(data, '\0'));
+}
+
+/**********************************************************************
+ *                       msEncodeCharExcept()
+ *
+ * URL encoding, applies RFP2396 encoding to all characters 
+ * except the one exception character. An exception character
+ * of '\0' implies no exception handling.
+ *
+ **********************************************************************/
+ 
+char *msEncodeUrlExcept(const char *data, const char except)
 {
   char *hex = "0123456789ABCDEF";
   const char *i;
@@ -565,7 +879,7 @@ char *msEncodeUrl(const char *data)
   unsigned char ch;
 
   for (inc=0, i=data; *i!='\0'; i++)
-    if (!isalnum(*i))
+    if (msEncodeChar(*i))
       inc += 2;
   
   if (!(code = (char*)malloc(strlen(data)+inc+1)))
@@ -576,11 +890,16 @@ char *msEncodeUrl(const char *data)
       if (*i == ' ')
 	*j = '+';
       else
-      if (!isalnum(*i))
+      if ( except != '\0' && *i == except )
+        {
+	  *j = except;
+        }
+      else 
+      if (msEncodeChar(*i))
 	{
 	  ch = *i;
-	  *j++ = '%';
-	  *j++ = hex[ch/16];
+	  *j++ = '%'; 
+	  *j++ = hex[ch/16]; 
 	  *j   = hex[ch%16];
 	}
       else
@@ -1020,6 +1339,103 @@ int msHexToInt(char *hex) {
 }
 
 
+/*
+** Use FRIBIDI to encode the string.
+** The return value must be freed by the caller.
+*/
+#ifdef USE_FRIBIDI
+char *msGetFriBidiEncodedString(const char *string, const char *encoding)
+{
+  FriBidiChar logical[MAX_STR_LEN];
+  FriBidiCharType base = FRIBIDI_TYPE_ON;
+  size_t len;
+  
+#ifdef FRIBIDI_NO_CHARSETS
+  iconv_t to_ucs4, from_ucs4;
+#else
+  int to_char_set_num;
+  int from_char_set_num;
+#endif
+
+  len = strlen(string);
+
+#ifdef FRIBIDI_NO_CHARSETS
+  to_ucs4 = iconv_open ("WCHAR_T", encoding);
+  from_ucs4 = iconv_open ("UTF-8", "WCHAR_T");
+#else
+  to_char_set_num = fribidi_parse_charset ((char*)encoding);
+  from_char_set_num = fribidi_parse_charset ("UTF-8");
+#endif
+
+#ifdef FRIBIDI_NO_CHARSETS
+  if (to_ucs4 == (iconv_t) (-1) || from_ucs4 == (iconv_t) (-1))
+#else
+  if (!to_char_set_num || !from_char_set_num)
+#endif
+  {
+    msSetError(MS_IDENTERR, "Encoding not supported (%s).", 
+               "msGetFriBidiEncodedString()", encoding);
+    return NULL;
+  }
+
+#ifdef FRIBIDI_NO_CHARSETS
+  {
+    char *st = string, *ust = (char *) logical;
+    int in_len = (int) len;
+    len = sizeof logical;
+    iconv (to_ucs4, &st, &in_len, &ust, (int *) &len);
+    len = (FriBidiChar *) ust - logical;
+  }
+#else
+  len = fribidi_charset_to_unicode (to_char_set_num, (char*)string, len, logical);
+#endif
+
+  {
+    FriBidiChar *visual;
+    char outstring[MAX_STR_LEN];
+    FriBidiStrIndex *ltov, *vtol;
+    FriBidiLevel *levels;
+    FriBidiStrIndex new_len;
+    fribidi_boolean log2vis;
+
+    visual = (FriBidiChar *) malloc (sizeof (FriBidiChar) * (len + 1));
+    ltov = NULL;
+    vtol = NULL;
+    levels = NULL;
+
+    /* Create a bidi string. */
+    log2vis = fribidi_log2vis (logical, len, &base,
+       /* output */
+       visual, ltov, vtol, levels);
+
+    if (!log2vis) {
+      msSetError(MS_IDENTERR, "Failed to create bidi string.", 
+             "msGetFriBidiEncodedString()");
+      return NULL;
+    }
+
+    new_len = len;
+
+    /* Convert it to utf-8 for display. */
+#ifdef FRIBIDI_NO_CHARSETS
+    {
+      char *str = outstring, *ust = (char *) visual;
+      int in_len = len * sizeof visual[0];
+      new_len = sizeof outstring;
+      iconv (from_ucs4, &ust, &in_len, &str, (int *) &new_len);
+      *str = '\0';
+      new_len = str - outstring;
+     }
+#else
+     new_len =
+       fribidi_unicode_to_charset (from_char_set_num,
+           visual, len, outstring);
+#endif
+     return strdup(outstring);
+  }
+}
+#endif
+
 /*
 ** Simple charset converter. Converts string from specified encoding to UTF-8.
 ** The return value must be freed by the caller.
@@ -1032,6 +1448,10 @@ char *msGetEncodedString(const char *string, const char *encoding)
   char *outp, *out = NULL;
   size_t len, bufsize, bufleft, status;
 
+#ifdef USE_FRIBIDI
+  if(fribidi_parse_charset ((char*)encoding))
+    return msGetFriBidiEncodedString(string, encoding);
+#endif 
   len = strlen(string);
 
   if (len == 0 || (encoding && strcasecmp(encoding, "UTF-8")==0))
@@ -1072,6 +1492,9 @@ char *msGetEncodedString(const char *string, const char *encoding)
 
   return out;
 #else
+  if (*string == '\0' || (encoding && strcasecmp(encoding, "UTF-8")==0))
+      return strdup(string);    /* Nothing to do: string already in UTF-8 */
+
   msSetError(MS_MISCERR, "Not implemeted since Iconv is not enabled.", "msGetEncodedString()");
   return NULL;
 #endif
@@ -1149,14 +1572,23 @@ char* msConvertWideStringToUTF8 (const wchar_t* string, const char* encoding) {
 }
 
 /*
-** Returns the next UTF-8 char in string and advances *in_ptr to the next
+** Returns the next glyph in string and advances *in_ptr to the next
 ** character.
 **
 ** If out_string is not NULL then the character (bytes) is copied to this 
 ** buffer and null-terminated. out_string must be a pre-allocated buffer of
-** at least 7 bytes.
+** at least 11 bytes.
 **
-** The function returns the number of bytes in this UTF-8 character.
+** The function returns the number of bytes in this glyph.
+**
+** This function treats 3 types of glyph encodings:
+*   - as an html entity, for example &#123; , &#x1af; , or &eacute;
+*   - as an utf8 encoded character
+*   - if utf8 decoding fails, as a raw character
+* 
+** This function mimics the character decoding function used in gdft.c of 
+* libGD. It is necessary to have the same behaviour, as input strings must be
+* split into the same glyphs as what gd does.
 **
 ** In UTF-8, the number of leading 1 bits in the first byte specifies the 
 ** number of bytes in the entire sequence.
@@ -1169,72 +1601,278 @@ char* msConvertWideStringToUTF8 (const wchar_t* string, const char* encoding) {
 ** U-00200000 U-03FFFFFF: 111110xx 10xxxxxx 10xxxxxx 10xxxxxx 10xxxxxx
 ** U-04000000 U-7FFFFFFF: 1111110x 10xxxxxx 10xxxxxx 10xxxxxx 10xxxxxx 10xxxxxx
 */
-int msGetNextUTF8Char(const char **in_ptr, char *out_string)
+int msGetNextGlyph(const char **in_ptr, char *out_string)
 {
     unsigned char in;
-    int numbytes=0;
+    int numbytes=0,unicode;
     int i;
 
     in = (unsigned char)**in_ptr;
 
     if (in == 0)
         return -1;  /* Empty string */
-
-    if (in < 0x80)
-    {
-        numbytes = 1;   /* 0xxxxxxx */
+    if((numbytes=msGetUnicodeEntity(*in_ptr,&unicode))>0) {
+        if(out_string) {
+            for(i=0;i<numbytes;i++) {
+                out_string[i]=(*in_ptr)[i];
+            }
+            out_string[numbytes]='\0';
+        }
+        *in_ptr+=numbytes;
+        return numbytes;
     }
-    else if ((in & 0xC0) == 0x80)
-    {
-        /* 10xxxxxx - this is not the first byte of a UTF-8 char, we need to 
-         * resynchronize . For now just return -1.
-         */
-        numbytes = 1;
-        return -1;
+    if (in < 0xC0)
+    {/*
+     * Handles properly formed UTF-8 characters between
+     * 0x01 and 0x7F.  Also treats \0 and naked trail
+     * bytes 0x80 to 0xBF as valid characters representing
+     * themselves.
+     */
+        /*goto end of loop to return just the char*/
     }
-    else if ((in & 0xE0) == 0xC0)
+    else if (in < 0xE0)
     {
-        numbytes = 2;   /* 110xxxxx 10xxxxxx */
+        if (((*in_ptr)[1]& 0xC0) == 0x80) {
+            if(out_string) {
+                out_string[0]=in;
+                out_string[1]=(*in_ptr)[1];
+                out_string[2]='\0';
+            }
+            *in_ptr+=2;
+            return 2; /*110xxxxx 10xxxxxx*/
+        }
     }
-    else if ((in & 0xF0) == 0xE0)
+    else if (in < 0xF0)
     {
-        numbytes = 3;   /* 1110xxxx 10xxxxxx 10xxxxxx */
+        if (((*in_ptr)[1]& 0xC0) == 0x80 && ((*in_ptr)[2]& 0xC0) == 0x80) {
+            if(out_string) {
+                out_string[0]=in;
+                *in_ptr+=numbytes;  out_string[1]=(*in_ptr)[1];
+                out_string[2]=(*in_ptr)[2];
+                out_string[3]='\0';
+            }
+            *in_ptr+=3;
+            return 3;   /* 1110xxxx 10xxxxxx 10xxxxxx */
+        }
     }
-    else if ((in & 0xF8) == 0xF0)
+    else if (in < 0xF8)
     {
-        numbytes = 4;   /* 11110xxx 10xxxxxx 10xxxxxx 10xxxxxx */
+        if (((*in_ptr)[1]& 0xC0) == 0x80 && ((*in_ptr)[2]& 0xC0) == 0x80 
+                && ((*in_ptr)[3]& 0xC0) == 0x80) {
+            if(out_string) {
+                out_string[0]=in;
+                out_string[1]=(*in_ptr)[1];
+                out_string[2]=(*in_ptr)[2];
+                out_string[3]=(*in_ptr)[3];
+                out_string[4]='\0';
+            }
+            *in_ptr+=4;
+            return 4;   /* 11110xxx 10xxxxxx 10xxxxxx 10xxxxxx */
+        }
     }
-    else if ((in & 0xFC) == 0xF8)
+    else if (in < 0xFC)
     {
-        numbytes = 5;   /* 111110xx 10xxxxxx 10xxxxxx 10xxxxxx 10xxxxxx */
+        if (((*in_ptr)[1]& 0xC0) == 0x80 && ((*in_ptr)[2]& 0xC0) == 0x80 
+                && ((*in_ptr)[3]& 0xC0) == 0x80 && ((*in_ptr)[4]& 0xC0) == 0x80) {
+            if(out_string) {
+                out_string[0]=in;
+                out_string[1]=(*in_ptr)[1];
+                out_string[2]=(*in_ptr)[2];
+                out_string[3]=(*in_ptr)[3];
+                out_string[4]=(*in_ptr)[4];
+                out_string[5]='\0';
+            }
+            *in_ptr+=5;
+            return 5;   /* 111110xx 10xxxxxx 10xxxxxx 10xxxxxx 10xxxxxx */
+        }
     }
-    else if ((in & 0xFE) == 0xFC)
+    else if (in < 0xFE)
     {
-        numbytes = 6;   /* 1111110x 10xxxxxx 10xxxxxx 10xxxxxx 10xxxxxx 10xxxxxx */
+        if (((*in_ptr)[1]& 0xC0) == 0x80 && ((*in_ptr)[2]& 0xC0) == 0x80 
+                && ((*in_ptr)[3]& 0xC0) == 0x80 && ((*in_ptr)[4]& 0xC0) == 0x80
+                && ((*in_ptr)[5]& 0xC0) == 0x80) {
+            if(out_string) {
+                out_string[0]=in;
+                out_string[1]=(*in_ptr)[1];
+                out_string[2]=(*in_ptr)[2];
+                out_string[3]=(*in_ptr)[3];
+                out_string[4]=(*in_ptr)[4];
+                out_string[5]=(*in_ptr)[5];
+                out_string[6]='\0';
+            }
+            *in_ptr+=6;
+            return 6;   /* 1111110x 10xxxxxx 10xxxxxx 10xxxxxx 10xxxxxx 10xxxxxx */
+        }
     }
-
-    /* Advance input buffer and copy the bytes to out_string if one is set */
-    for(i=0; i<numbytes && **in_ptr != '\0'; i++)
-    {
-        if (out_string)
-            out_string[i] = **in_ptr;
-        (*in_ptr)++;
+    
+    if (out_string) {
+        out_string[0]=in;
+        out_string[1] = '\0';   /* 0xxxxxxx */
     }
-    if (out_string)
-        out_string[i] = '\0';
-
-    return numbytes;
+    (*in_ptr)++;
+    return 1;
 }
 
 /*
-** Returns the number of UTF-8 chars in string
+** Returns the number of glyphs in string
 */
-int msGetNumUTF8Chars(const char *in_ptr)
+int msGetNumGlyphs(const char *in_ptr)
 {
     int numchars=0;
 
-    while( msGetNextUTF8Char(&in_ptr, NULL) != -1 )
+    while( msGetNextGlyph(&in_ptr, NULL) != -1 )
         numchars++;
 
     return numchars;
 }
+
+static int cmp_entities(const void *e1, const void *e2) {
+  struct mapentities_s *en1 = (struct mapentities_s *) e1;
+  struct mapentities_s *en2 = (struct mapentities_s *) e2;
+  return strcmp(en1->name, en2->name);
+}
+/*
+ * this function tests if the string pointed by inptr represents
+ * an HTML entity, in decimal form ( e.g. &#197;), in hexadecimal 
+ * form ( e.g. &#x6C34; ), or from html 4.0 spec ( e.g. &eacute; )
+ * - returns returns 0 if the string doesn't represent such an entity. 
+ * - if the string does start with such entity,it returns the number of 
+ * bytes occupied by said entity, and stores the unicode value in *unicode
+ */
+int msGetUnicodeEntity(const char *inptr, int *unicode) {
+    unsigned char *in = (unsigned char*)inptr;
+    int l,val=0;   
+    if(*in=='&') {
+        in++;
+        if(*in=='#') {
+            in++;
+            if(*in=='x'||*in=='X') {
+                in++;
+                for(l=3;l<8;l++) {
+                    char byte;
+                    if(*in>='0'&&*in<='9')
+                        byte = *in - '0';
+                    else if(*in>='a'&&*in<='f')
+                        byte = *in - 'a' + 10;
+                    else if(*in>='A'&&*in<='F')
+                        byte = *in - 'A' + 10;
+                    else
+                        break;
+                    in++;
+                    val = (val * 16) + byte;
+                }
+                if(*in==';' && l>3 ) {
+                    *unicode=val;
+                    return ++l;
+                }
+            } 
+            else
+            {
+                for(l=2;l<8;l++) {
+                    if(*in>='0'&&*in<='9') {
+                        val = val*10+*in-'0';
+                        in++;
+                    }
+                    else
+                        break;
+                }
+                if(*in==';' && l>2 ) {
+                    *unicode=val;
+                    return ++l;
+                }
+            }
+        }
+        else
+        {
+            char entity_name_buf[MAP_ENTITY_NAME_LENGTH_MAX+1];
+            char *p;
+            struct mapentities_s key, *res;
+            key.name = p = entity_name_buf;
+            for (l = 1; l <=  MAP_ENTITY_NAME_LENGTH_MAX+1; l++)
+            {
+                if (*in == '\0') /*end of string before possible entity: return*/
+                    break;
+                if (*in == ';') /*possible end of entity: do a lookup*/
+                {
+                    *p++ = '\0';
+                    res = bsearch(&key, mapentities, MAP_NR_OF_ENTITIES,
+                            sizeof(mapentities[0]), *cmp_entities);
+                    if (res)
+                    {
+                        *unicode = res->value;
+                        return ++l;
+                    }
+                    break; /*the string was of the form of an entity but didn't correspond to an existing one: return*/
+                }
+                *p++ = *in;
+                in++;
+            }
+        }
+    }
+    return 0;
+}
+
+/**
+ * msStringIsInteger()
+ *
+ * determines whether a given string is an integer
+ *
+ * @param string the string to be tested
+ *
+ * @return MS_SUCCESS or MS_FAILURE
+ */
+
+int msStringIsInteger(const char *string) {
+  int length, i;
+
+  length = strlen(string);
+
+  if (length == 0)
+    return MS_FAILURE;
+
+  for(i=0;i<length;i++) {
+    if (!isdigit(string[i]))
+      return MS_FAILURE;
+  }
+
+  return MS_SUCCESS;
+}
+
+/**
+ * msGetFirstLine()
+ *
+ * returns the first line of a given string.
+ * called by getLabelSize functions for calculating 
+ * the baseline offsets of labels
+ *
+ * this function was implemented to avoid using the 
+ * msSplitString function and its multiple mallocs, as
+ * only one malloc is used here.
+ *
+ * this function can be called if the input isn't a
+ * multiline string, but this wastes a malloc
+ */
+char* msGetFirstLine(char* text) {
+    int firstLineLength=0; /*number of bytes up to first \n character */
+    int glyphLength;
+    char glyph[11];
+    const char *textptr=text;
+    char *firstLine,*firstLineCur;
+    /*loop through glyphs in text*/
+    while((glyphLength=msGetNextGlyph(&textptr,glyph))) {
+        if(glyphLength==1 && *glyph=='\n') { /*we've hit the first \n char*/
+            firstLineCur = firstLine = malloc(firstLineLength+1);
+            
+            /*copy the first line into the return array*/
+            while(firstLineLength--) {
+                *firstLineCur++ = *text++;
+            }
+            *firstLineCur='\0';
+            return firstLine;
+        }
+        /*increment byte count if we haven't hit a \n yet*/
+        firstLineLength+=glyphLength;
+    }
+    /*no newline found in text*/
+    return strdup(text);
+}
