@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: mapsvg.c 7954 2008-10-04 10:59:35Z tbonfort $
+ * $Id: mapsvg.c 9472 2009-10-16 18:17:05Z aboudreault $
  *
  * Project:  MapServer
  * Purpose:  SVG output
@@ -37,7 +37,7 @@
 #endif
 #include "mapserver.h"
 
-MS_CVSID("$Id: mapsvg.c 7954 2008-10-04 10:59:35Z tbonfort $")
+MS_CVSID("$Id: mapsvg.c 9472 2009-10-16 18:17:05Z aboudreault $")
 
 
 static int msIO_fprintfgz(FILE *fp, int bCompressed, const char *format, ... )
@@ -102,7 +102,9 @@ MS_DLL_EXPORT imageObj *msImageCreateSVG(int width, int height,
     image->height = height;
     image->imagepath = NULL;
     image->imageurl = NULL;
-    
+    image->resolution = map->resolution;
+    image->resolutionfactor = map->resolution/map->defresolution;
+
     if (imagepath)
       image->imagepath = strdup(imagepath);
 
@@ -314,7 +316,7 @@ MS_DLL_EXPORT imageObj *msImageCreateSVG(int width, int height,
 /*      Draws an svg line element.                                      */
 /************************************************************************/
 static void imagePolyline(FILE *fp, int bCompressed, shapeObj *p, 
-                          colorObj *color, int size,
+                          colorObj *color, double size,
                           int symbolstylelength, int *symbolstyle, int bFullRes)
 {
     int i, j, k;
@@ -344,11 +346,11 @@ static void imagePolyline(FILE *fp, int bCompressed, shapeObj *p,
                 pszDashArray = msStringConcatenate(pszDashArray, szTmp);
             }
             
-            msIO_fprintfgz(fp, bCompressed, "<polyline fill=\"none\" stroke=\"#%02x%02x%02x\" stroke-width=\"%d\" stroke-dasharray=\"%s\" points=\"",color->red, color->green, 
+            msIO_fprintfgz(fp, bCompressed, "<polyline fill=\"none\" stroke=\"#%02x%02x%02x\" stroke-width=\"%0.2f\" stroke-dasharray=\"%s\" points=\"",color->red, color->green, 
                            color->blue,size, pszDashArray);
         }
         else
-          msIO_fprintfgz(fp, bCompressed,"<polyline fill=\"none\" stroke=\"#%02x%02x%02x\" stroke-width=\"%d\" points=\"",color->red, color->green, color->blue,size);
+          msIO_fprintfgz(fp, bCompressed,"<polyline fill=\"none\" stroke=\"#%02x%02x%02x\" stroke-width=\"%0.2f\" points=\"",color->red, color->green, color->blue,size);
         
         if (bFullRes) 
           msIO_fprintfgz(fp, bCompressed, "%.2f,%.2f", p->line[i].point[0].x, p->line[i].point[0].y); 
@@ -515,10 +517,11 @@ MS_DLL_EXPORT void msDrawLineSymbolSVG(symbolSetObj *symbolset,
     /* int styleDashed[100]; */
     int ox, oy;
     double size;
-    int width;
+    double width;
     /* gdPoint points[MS_MAXVECTORPOINTS]; */
     symbolObj *symbol;
     int bFullRes;
+    int symbol_pattern[MS_MAXPATTERNLENGTH], i;
 
 /* -------------------------------------------------------------------- */
 /*      if not SVG, return.                                             */
@@ -535,28 +538,28 @@ MS_DLL_EXPORT void msDrawLineSymbolSVG(symbolSetObj *symbolset,
                    "TRUE") == 0)
       bFullRes = 1;
     if(style->size == -1) {
-      size = (int)msSymbolGetDefaultSize( symbolset->symbol[style->symbol] );
+      size = (double)msSymbolGetDefaultSize( symbolset->symbol[style->symbol] );
     }
     else
       size = style->size;
 
     /* TODO: Don't get this modification, is it needed elsewhere? */
-    if(size*scalefactor > style->maxsize) scalefactor = (float)style->maxsize/(float)size;
-    if(size*scalefactor < style->minsize) scalefactor = (float)style->minsize/(float)size;
-    size = MS_NINT(size*scalefactor);
-    size = MS_MAX(size, style->minsize);
-    size = MS_MIN(size, style->maxsize);
+    if(size*scalefactor > style->maxsize*image->resolutionfactor) scalefactor = (float)style->maxsize*image->resolutionfactor/(float)size;
+    if(size*scalefactor < style->minsize*image->resolutionfactor) scalefactor = (float)style->minsize*image->resolutionfactor/(float)size;
+    size = size*scalefactor;
+    size = MS_MAX(size, style->minsize*image->resolutionfactor);
+    size = MS_MIN(size, style->maxsize*image->resolutionfactor);
 
-    width = MS_NINT(style->width*scalefactor);
-    width = MS_MAX(width, style->minwidth);
-    width = MS_MIN(width, style->maxwidth);
+    width = style->width*scalefactor;
+    width = MS_MAX(width, style->minwidth*image->resolutionfactor);
+    width = MS_MIN(width, style->maxwidth*image->resolutionfactor);
 
     if(style->symbol > symbolset->numsymbols || style->symbol < 0) return; /* no such symbol, 0 is OK */
 
      if (!MS_VALID_COLOR( style->color))
       return;
     
-    if(size < 1) return; /* size too small */
+    if(size < 0) return; /* size too small */
 
     /* TODO : do we need offset ?? */
     ox = MS_NINT(style->offsetx*scalefactor);
@@ -564,6 +567,14 @@ MS_DLL_EXPORT void msDrawLineSymbolSVG(symbolSetObj *symbolset,
 
     symbol = symbolset->symbol[style->symbol];
 
+    /* scale the symbol pattern */
+    if (symbol->patternlength > 0)
+    {
+        for (i=0; i<symbol->patternlength; i++)
+        {
+	  symbol_pattern[i] = MS_NINT(symbol->pattern[i]*scalefactor);
+        }
+    }
 
     /* TODO : all lines are draw as filled lines.  */
     
@@ -573,11 +584,11 @@ MS_DLL_EXPORT void msDrawLineSymbolSVG(symbolSetObj *symbolset,
     if(style->symbol == 0) 
       imagePolyline(image->img.svg->stream, image->img.svg->compressed, 
                     p, &style->color, width,
-                    symbol->patternlength,  symbol->pattern, bFullRes);
+                    symbol->patternlength,  symbol_pattern, bFullRes);
     else
       imagePolyline(image->img.svg->stream, image->img.svg->compressed, 
-                    p, &style->color, (int)size,
-                    symbol->patternlength,  symbol->pattern, bFullRes);
+                    p, &style->color, size,
+                    symbol->patternlength,  symbol_pattern, bFullRes);
         return;
     
 
@@ -600,14 +611,13 @@ void msImageStartLayerSVG(mapObj *map, layerObj *layer, imageObj *image)
 
 static void imageFillPolygon(FILE *fp, int bCompressed, 
                              shapeObj *p, colorObj *psFillColor, 
-                             colorObj *psOutlineColor, int size,
+                             colorObj *psOutlineColor, double size,
                              int symbolstylelength, int *symbolstyle, int bFullRes)
 {
     int i, j, k,max;
     char *pszDashArray = NULL;
     char szTmp[100];
 
-
     if (!fp || !p || (psFillColor==NULL && psOutlineColor==NULL) || size <0 )
       return;
 
@@ -650,7 +660,7 @@ static void imageFillPolygon(FILE *fp, int bCompressed,
                            psOutlineColor->red, psOutlineColor->green, 
                            psOutlineColor->blue,
                            size, pszDashArray);*/
-                msIO_fprintfgz(fp, bCompressed, "<path fill=\"#%02x%02x%02x\" stroke=\"#%02x%02x%02x\" stroke-width=\"%d\" %s d=\"",
+                msIO_fprintfgz(fp, bCompressed, "<path fill=\"#%02x%02x%02x\" stroke=\"#%02x%02x%02x\" stroke-width=\"%0.2f\" %s d=\"",
                            psFillColor->red, psFillColor->green, 
                            psFillColor->blue,
                            psOutlineColor->red, psOutlineColor->green, 
@@ -663,7 +673,7 @@ static void imageFillPolygon(FILE *fp, int bCompressed,
                            psOutlineColor->red, psOutlineColor->green, 
                            psOutlineColor->blue,
                            size, pszDashArray);*/
-                msIO_fprintfgz(fp,  bCompressed, "<path stroke=\"#%02x%02x%02x\" stroke-width=\"%d\" %s style=\"fill:none\" d=\"",
+                msIO_fprintfgz(fp,  bCompressed, "<path stroke=\"#%02x%02x%02x\" stroke-width=\"%0.2f\" %s style=\"fill:none\" d=\"",
                            psOutlineColor->red, psOutlineColor->green, 
                            psOutlineColor->blue,
                            size, pszDashArray);
@@ -719,8 +729,9 @@ void msDrawShadeSymbolSVG(symbolSetObj *symbolset, imageObj *image,
     colorObj    sFc;
     colorObj    sOc;
     symbolObj   *symbol;
-    int         size;
+    double         size;
     int bFullRes = 0;
+    int symbol_pattern[MS_MAXPATTERNLENGTH], i;
 
 /* -------------------------------------------------------------------- */
 /*      if not svg, return.                                             */
@@ -741,18 +752,27 @@ void msDrawShadeSymbolSVG(symbolSetObj *symbolset, imageObj *image,
     if(style->size == -1) 
     {
         size = (int)msSymbolGetDefaultSize( symbolset->symbol[style->symbol] );
-        size = MS_NINT(size*scalefactor);
+        size = size*scalefactor;
     }
     else
-      size = MS_NINT(style->size*scalefactor);
+      size = style->size*scalefactor;
 
-    size = MS_MAX(size, style->minsize);
-    size = MS_MIN(size, style->maxsize);
+    size = MS_MAX(size, style->minsize*image->resolutionfactor);
+    size = MS_MIN(size, style->maxsize*image->resolutionfactor);
+
+    /* scale the symbol pattern */
+    if (symbol->patternlength > 0)
+    {
+        for (i=0; i<symbol->patternlength; i++)
+        {
+	  symbol_pattern[i] = MS_NINT(symbol->pattern[i]*scalefactor);
+        }
+    }
 
     if(style->symbol > symbolset->numsymbols || style->symbol < 0) /* no such symbol, 0 is OK */
         return;
 
-    if(size < 1) /* size too small */
+    if(size < 0) /* size too small */
         return;
 
     sFc.red = style->color.red;
@@ -769,7 +789,7 @@ void msDrawShadeSymbolSVG(symbolSetObj *symbolset, imageObj *image,
       psOutlineColor = &sOc;
 
     imageFillPolygon(image->img.svg->stream, image->img.svg->compressed ,p, psFillColor, psOutlineColor, size,
-                     symbol->patternlength,  symbol->pattern, bFullRes);
+                     symbol->patternlength,  symbol_pattern, bFullRes);
 
 }
 
@@ -1055,8 +1075,8 @@ int msDrawTextSVG(imageObj *image, pointObj labelPnt, char *string,
         /* position, offset, angle, buffer, antialias, wrap, encoding */
 
         size = label->size*scalefactor;
-        size = MS_MAX(size, label->minsize);
-        size = MS_MIN(size, label->maxsize);
+        size = MS_MAX(size, label->minsize*image->resolutionfactor);
+        size = MS_MIN(size, label->maxsize*image->resolutionfactor);
 
         if(!fontset) {
             msSetError(MS_TTFERR, "No fontset defined.", "msDrawTextSVG()");
@@ -1154,9 +1174,13 @@ int msDrawLabelCacheSVG(imageObj *image, mapObj *map)
   layerObj *layerPtr=NULL;
   labelObj *labelPtr=NULL;
 
-  int marker_width, marker_height;
+  int marker_width, marker_height, label_offset_x, label_offset_y;
   int marker_offset_x, marker_offset_y;
   rectObj marker_rect;
+  int label_mindistance, label_buffer;
+
+  label_mindistance=-1;
+  label_buffer=0;
 
 /* -------------------------------------------------------------------- */
 /*      if not svg or invaid inputs, return.                            */
@@ -1182,7 +1206,12 @@ int msDrawLabelCacheSVG(imageObj *image, mapObj *map)
     if(msGetLabelSize(image, cachePtr->text, labelPtr, &r, &(map->fontset), layerPtr->scalefactor, MS_TRUE,NULL) == -1)
       return(-1);
 
-    if(labelPtr->autominfeaturesize && ((r.maxx-r.minx) > cachePtr->featuresize))
+    label_offset_x = MS_NINT(labelPtr->offsetx*layerPtr->scalefactor);
+    label_offset_y = MS_NINT(labelPtr->offsety*layerPtr->scalefactor);
+    label_buffer = MS_NINT(labelPtr->buffer*image->resolutionfactor);
+    label_mindistance = MS_NINT(labelPtr->mindistance*image->resolutionfactor);
+    
+    if(labelPtr->autominfeaturesize && (cachePtr->featuresize != -1) && ((r.maxx-r.minx) > cachePtr->featuresize))
       continue; /* label too large relative to the feature */
 
     marker_offset_x = marker_offset_y = 0; /* assume no marker */
@@ -1211,7 +1240,7 @@ int msDrawLabelCacheSVG(imageObj *image, mapObj *map)
 	  msFreeShape(cachePtr->poly);
 	  cachePtr->status = MS_TRUE; /* assume label *can* be drawn */
 
-	  p = get_metrics(&(cachePtr->point), position, r, (marker_offset_x + labelPtr->offsetx), (marker_offset_y + labelPtr->offsety), labelPtr->angle, labelPtr->buffer, cachePtr->poly);
+	  p = get_metrics(&(cachePtr->point), position, r, (marker_offset_x + label_offset_x), (marker_offset_y + label_offset_y), labelPtr->angle, label_buffer, cachePtr->poly);
 
 	  if(layerPtr->type == MS_LAYER_ANNOTATION && cachePtr->numstyles > 0)
 	    msRectToPolygon(marker_rect, cachePtr->poly); /* save marker bounding polygon */
@@ -1219,7 +1248,7 @@ int msDrawLabelCacheSVG(imageObj *image, mapObj *map)
           /* Compare against image bounds, rendered labels and markers (sets cachePtr->status) */
           msTestLabelCacheCollisions(&(map->labelcache), labelPtr, 
                                      map->width, map->height, 
-                                     labelPtr->buffer, cachePtr, priority, l);
+                                     label_buffer, cachePtr, priority, l, label_mindistance, (r.maxx-r.minx));
 
 
 	  if(cachePtr->status) /* found a suitable place for this label */
@@ -1233,7 +1262,7 @@ int msDrawLabelCacheSVG(imageObj *image, mapObj *map)
 	  msFreeShape(cachePtr->poly);
 	  cachePtr->status = MS_TRUE; /* assume label *can* be drawn */
 
-	  p = get_metrics(&(cachePtr->point), j, r, (marker_offset_x + labelPtr->offsetx), (marker_offset_y + labelPtr->offsety), labelPtr->angle, labelPtr->buffer, cachePtr->poly);
+	  p = get_metrics(&(cachePtr->point), j, r, (marker_offset_x + label_offset_x), (marker_offset_y + label_offset_y), labelPtr->angle, label_buffer, cachePtr->poly);
 
 	  if(layerPtr->type == MS_LAYER_ANNOTATION && cachePtr->numstyles > 0)
 	    msRectToPolygon(marker_rect, cachePtr->poly); /* save marker bounding polygon */
@@ -1241,7 +1270,7 @@ int msDrawLabelCacheSVG(imageObj *image, mapObj *map)
           /* Compare against image bounds, rendered labels and markers (sets cachePtr->status) */
           msTestLabelCacheCollisions(&(map->labelcache), labelPtr, 
                                      map->width, map->height, 
-                                     labelPtr->buffer, cachePtr, priority, l);
+                                     label_buffer, cachePtr, priority, l, label_mindistance, (r.maxx-r.minx));
 
 	  if(cachePtr->status) /* found a suitable place for this label */
 	    break;
@@ -1254,9 +1283,9 @@ int msDrawLabelCacheSVG(imageObj *image, mapObj *map)
       cachePtr->status = MS_TRUE; /* assume label *can* be drawn */
 
       if(labelPtr->position == MS_CC) /* don't need the marker_offset */
-        p = get_metrics(&(cachePtr->point), labelPtr->position, r, labelPtr->offsetx, labelPtr->offsety, labelPtr->angle, labelPtr->buffer, cachePtr->poly);
+        p = get_metrics(&(cachePtr->point), labelPtr->position, r, label_offset_x, label_offset_y, labelPtr->angle, label_buffer, cachePtr->poly);
       else
-        p = get_metrics(&(cachePtr->point), labelPtr->position, r, (marker_offset_x + labelPtr->offsetx), (marker_offset_y + labelPtr->offsety), labelPtr->angle, labelPtr->buffer, cachePtr->poly);
+        p = get_metrics(&(cachePtr->point), labelPtr->position, r, (marker_offset_x + label_offset_x), (marker_offset_y + label_offset_y), labelPtr->angle, label_buffer, cachePtr->poly);
 
       if(layerPtr->type == MS_LAYER_ANNOTATION && cachePtr->numstyles > 0)
 	msRectToPolygon(marker_rect, cachePtr->poly); /* save marker bounding polygon, part of overlap tests */
@@ -1266,7 +1295,7 @@ int msDrawLabelCacheSVG(imageObj *image, mapObj *map)
         /* Compare against image bounds, rendered labels and markers (sets cachePtr->status) */
         msTestLabelCacheCollisions(&(map->labelcache), labelPtr, 
                                    map->width, map->height, 
-                                   labelPtr->buffer, cachePtr, priority, l);
+                                   label_buffer, cachePtr, priority, l, label_mindistance, (r.maxx-r.minx));
       }
     } /* end position if-then-else */
 
@@ -1302,7 +1331,7 @@ void msDrawMarkerSymbolSVG(symbolSetObj *symbolset, imageObj *image,
     char szTmp[100];
     symbolObj *symbol=NULL;
     double size,d;
-    int width;
+    double width;
     int x,y,rx,ry;
     char *pszFill = NULL, *pszStroke=NULL;
     int bFillSetToNone = 0;
@@ -1332,12 +1361,12 @@ void msDrawMarkerSymbolSVG(symbolSetObj *symbolset, imageObj *image,
     size = MS_NINT(size*scalefactor);
     } else
       size = MS_NINT(style->size*scalefactor);
-    size = MS_MAX(size, style->minsize);
-    size = MS_MIN(size, style->maxsize);
+    size = MS_MAX(size, style->minsize*image->resolutionfactor);
+    size = MS_MIN(size, style->maxsize*image->resolutionfactor);
 
-    width = MS_NINT(style->width*scalefactor);
-    width = MS_MAX(width, style->minwidth);
-    width = MS_MIN(width, style->maxwidth);
+    width = style->width*scalefactor;
+    width = MS_MAX(width, style->minwidth*image->resolutionfactor);
+    width = MS_MIN(width, style->maxwidth*image->resolutionfactor);
 
     if(style->symbol > symbolset->numsymbols || style->symbol < 0) return; /* no such symbol, 0 is OK */
     
@@ -1505,7 +1534,7 @@ void msDrawMarkerSymbolSVG(symbolSetObj *symbolset, imageObj *image,
                             
                             msIO_fprintfgz(image->img.svg->stream, 
                                            image->img.svg->compressed,  
-                                         "<polygon %s %s stroke-width=\"%d\" points=\"", 
+                                         "<polygon %s %s stroke-width=\"%0.2f\" points=\"", 
                                          pszFill, pszStroke, width);
                             
                             for (i=0; i<k;i++)
@@ -1532,7 +1561,7 @@ void msDrawMarkerSymbolSVG(symbolSetObj *symbolset, imageObj *image,
 
                 msIO_fprintfgz(image->img.svg->stream, 
                                image->img.svg->compressed,  
-                             "<polygon %s %s stroke-width=\"%d\" points=\"", 
+                             "<polygon %s %s stroke-width=\"%0.2f\" points=\"", 
                              pszFill, pszStroke, width);
                             
                 for (i=0; i<k;i++)
@@ -1566,7 +1595,7 @@ void msDrawMarkerSymbolSVG(symbolSetObj *symbolset, imageObj *image,
                             newpnt.y = MS_NINT(d*symbol->points[j].y + offset_y);
                             msIO_fprintfgz(image->img.svg->stream, 
                                            image->img.svg->compressed,  
-                              "<line x1=\"%d\" y1=\"%d\" x2=\"%d\" y2=\"%d\" %s %s stroke-width=\"%d\" stroke-linecap=\"round\"/>\n",
+                              "<line x1=\"%d\" y1=\"%d\" x2=\"%d\" y2=\"%d\" %s %s stroke-width=\"%0.2f\" stroke-linecap=\"round\"/>\n",
                                          (int)oldpnt.x, (int)oldpnt.y, (int)newpnt.x, (int)newpnt.y,
                                          pszFill, pszStroke, width);
                             oldpnt = newpnt;
@@ -1769,7 +1798,7 @@ int msDrawRasterLayerSVG(mapObj *map, layerObj *layer, imageObj *image)
                     "msImageCreateSVG()" );
             return MS_FAILURE;
         }
-        msSaveImageGD(imagetmp->img.gd, pszTmpfile, format);
+        msSaveImageGD(imagetmp, pszTmpfile, format);
         pszURL = (char *)malloc(sizeof(char)*(strlen(map->web.imageurl)+
                                               strlen(pszTmpfile)+
                                               strlen(format->extension)+2));
