@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: maptemplate.h 8855 2009-04-01 03:18:19Z sdlime $
+ * $Id: maptemplate.h 9503 2009-10-23 04:39:51Z sdlime $
  *
  * Project:  MapServer
  * Purpose:  Template processing related declarations.
@@ -79,7 +79,7 @@ typedef struct
    
   int UseShapes; /* are results of a query to be used in calculating an extent of some sort */
 
-  shapeObj SelectShape, resultshape;
+  shapeObj resultshape;
 
   rectObj RawExt;
 
