@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: mapthread.c 6428 2007-07-31 19:58:45Z dmorissette $
+ * $Id: mapthread.c 9374 2009-10-01 22:23:58Z warmerdam $
  *
  * Project:  UMN MapServer
  * Purpose:  Support code for abstracting thread issues.
@@ -137,7 +137,7 @@ files instead.
 #include "mapserver.h"
 #include "mapthread.h"
 
-MS_CVSID("$Id: mapthread.c 6428 2007-07-31 19:58:45Z dmorissette $")
+MS_CVSID("$Id: mapthread.c 9374 2009-10-01 22:23:58Z warmerdam $")
 
 #if defined(USE_THREAD)
 static int thread_debug = 0;
@@ -170,7 +170,7 @@ void msThreadInit()
     static pthread_mutex_t core_lock = PTHREAD_MUTEX_INITIALIZER;
 
     if( thread_debug )
-        msDebug( "msThreadInit() (posix)\n" );
+        fprintf( stderr, "msThreadInit() (posix)\n" );
 
     pthread_mutex_lock( &core_lock );
 
@@ -203,7 +203,7 @@ void msAcquireLock( int nLockId )
     assert( nLockId >= 0 && nLockId < mutexes_initialized );
 
     if( thread_debug )
-        msDebug( "msAcquireLock(%d/%s) (posix)\n", 
+        fprintf( stderr, "msAcquireLock(%d/%s) (posix)\n", 
                  nLockId, lock_names[nLockId] );
 
     pthread_mutex_lock( mutex_locks + nLockId );
@@ -220,7 +220,7 @@ void msReleaseLock( int nLockId )
     assert( nLockId >= 0 && nLockId < mutexes_initialized );
 
     if( thread_debug )
-        msDebug( "msReleaseLock(%d/%s) (posix)\n", 
+        fprintf( stderr, "msReleaseLock(%d/%s) (posix)\n", 
                  nLockId, lock_names[nLockId] );
 
     pthread_mutex_unlock( mutex_locks + nLockId );
@@ -255,7 +255,7 @@ void msThreadInit()
         return;
 
     if( thread_debug )
-        msDebug( "msThreadInit() (win32)\n" );
+        fprintf( stderr, "msThreadInit() (win32)\n" );
 
     if( core_lock == NULL )
         core_lock = CreateMutex( NULL, TRUE, NULL );
@@ -291,7 +291,7 @@ void msAcquireLock( int nLockId )
     assert( nLockId >= 0 && nLockId < mutexes_initialized );
 
     if( thread_debug )
-        msDebug( "msAcquireLock(%d/%s) (win32)\n", 
+        fprintf( stderr, "msAcquireLock(%d/%s) (win32)\n", 
                  nLockId, lock_names[nLockId] );
 
     WaitForSingleObject( mutex_locks[nLockId], INFINITE );
@@ -308,7 +308,7 @@ void msReleaseLock( int nLockId )
     assert( nLockId >= 0 && nLockId < mutexes_initialized );
 
     if( thread_debug )
-        msDebug( "msReleaseLock(%d/%s) (win32)\n", 
+        fprintf( stderr, "msReleaseLock(%d/%s) (win32)\n", 
                  nLockId, lock_names[nLockId] );
 
     ReleaseMutex( mutex_locks[nLockId] );
