@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: maptime.h 6428 2007-07-31 19:58:45Z dmorissette $
+ * $Id: maptime.h 7779 2008-07-03 14:20:54Z Assefa $
  *
  * Project:  MapServer
  * Purpose:  Time processing related declarations.
@@ -39,7 +39,7 @@ struct mstimeval {
     long    tv_sec;         /* seconds */
     long    tv_usec;        /* and microseconds */
 };
-void msGettimeofday(struct mstimeval *t, void *__not_used_here__);
+MS_DLL_EXPORT void msGettimeofday(struct mstimeval *t, void *__not_used_here__);
 #else
 #  include <sys/time.h>     /* for gettimeofday() */
 #  define  mstimeval timeval
