@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: maputil.c 8979 2009-04-30 04:09:00Z sdlime $
+ * $Id: maputil.c 9472 2009-10-16 18:17:05Z aboudreault $
  *
  * Project:  MapServer
  * Purpose:  Various utility functions ... a real hodgepodge.
@@ -39,6 +39,7 @@
 #include "mapparser.h"
 #include "mapthread.h"
 #include "mapfile.h"
+#include "mapcopy.h"
 
 #ifdef _WIN32
 #include <fcntl.h>
@@ -49,7 +50,7 @@
 #include <process.h>
 #endif
 
-MS_CVSID("$Id: maputil.c 8979 2009-04-30 04:09:00Z sdlime $")
+MS_CVSID("$Id: maputil.c 9472 2009-10-16 18:17:05Z aboudreault $")
 
 extern int msyyparse(void);
 extern int msyylex(void);
@@ -164,6 +165,16 @@ int msBindLayerToShape(layerObj *layer, shapeObj *shape, int querymapMode)
           MS_INIT_COLOR(style->outlinecolor, -1,-1,-1);
           bindColorAttribute(&style->outlinecolor, shape->values[style->bindings[MS_STYLE_BINDING_OUTLINECOLOR].index]);
         }
+
+        if(style->bindings[MS_STYLE_BINDING_OUTLINEWIDTH].index != -1) {
+          style->outlinewidth = 1;
+          bindDoubleAttribute(&style->outlinewidth, shape->values[style->bindings[MS_STYLE_BINDING_OUTLINEWIDTH].index]);
+        }
+
+        if(style->bindings[MS_STYLE_BINDING_OPACITY].index != -1) {
+          style->opacity = 100;
+          bindIntegerAttribute(&style->opacity, shape->values[style->bindings[MS_STYLE_BINDING_OPACITY].index]);
+        }
       }
     } /* next styleObj */
 
@@ -375,6 +386,9 @@ int msEvalExpression(expressionObj *expression, int itemindex, char **items, int
       tmpstr = msReplaceSubstring(tmpstr, expression->items[i], tmpstr2);
       free(tmpstr2);
     }
+
+    // fprintf(stderr, "exp; %s\n", tmpstr);
+
     msAcquireLock( TLOCK_PARSER );
     msyystate = MS_TOKENIZE_EXPRESSION;
     msyystring = tmpstr; /* set lexer state to EXPRESSION_STRING */
@@ -389,7 +403,8 @@ int msEvalExpression(expressionObj *expression, int itemindex, char **items, int
     }
 
     free(tmpstr);
-    return expresult;    
+    return expresult;
+    
     break;
   case(MS_REGEX):
     if(itemindex == -1) {
@@ -649,15 +664,38 @@ int msSaveImage(mapObj *map, imageObj *img, char *filename)
 
     if (img)
     {
-        if( MS_DRIVER_GD(img->format) )
+        if (MS_RENDERER_PLUGIN(img->format)) {
+            rendererVTableObj *renderer = img->format->vtable;
+            FILE *stream;
+            if(filename) 
+                stream = fopen(msBuildPath(szPath, map->mappath, filename),"wb");
+            else {
+                if ( msIO_needBinaryStdout() == MS_FAILURE )
+                    return MS_FAILURE;
+                stream = stdout;
+            }
+            if(!stream)
+                return MS_FAILURE;
+            if(renderer->supports_pixel_buffer) {
+                rasterBufferObj data;
+                renderer->getRasterBuffer(img,&data);
+
+                msSaveRasterBuffer(&data,stream,img->format );
+            } else {
+                renderer->saveImage(img, stream, img->format);
+            }
+            fclose(stream);
+            return MS_SUCCESS;
+        }
+    	else if( MS_DRIVER_GD(img->format) )
         {
             if(map != NULL && filename != NULL )
-                nReturnVal = msSaveImageGD(img->img.gd, 
+                nReturnVal = msSaveImageGD(img, 
                                            msBuildPath(szPath, map->mappath, 
                                                        filename), 
                                            img->format );
             else
-                nReturnVal = msSaveImageGD(img->img.gd, filename, img->format);
+                nReturnVal = msSaveImageGD(img, filename, img->format);
         }
 #ifdef USE_AGG
         else if( MS_DRIVER_AGG(img->format) )
@@ -745,10 +783,24 @@ int msSaveImage(mapObj *map, imageObj *img, char *filename)
 unsigned char *msSaveImageBuffer(imageObj* image, int *size_ptr, outputFormatObj *format)
 {
     *size_ptr = 0;
-	
-	if( MS_DRIVER_GD(image->format) )
+    if( MS_RENDERER_PLUGIN(image->format)){
+        rasterBufferObj data;
+        rendererVTableObj *renderer = image->format->vtable;
+        if(renderer->supports_pixel_buffer) {
+            bufferObj buffer;
+            msBufferInit(&buffer);
+            renderer->getRasterBuffer(image,&data);
+            msSaveRasterBufferToBuffer(&data,&buffer,format);
+            return buffer.data;
+            //don't free the bufferObj as we don't own the bytes anymore
+        } else {
+	        msSetError(MS_MISCERR, "Unsupported image type", "msSaveImageBuffer()");
+            return NULL;
+        }
+    }
+    else if( MS_DRIVER_GD(image->format) )
     {
-        return msSaveImageBufferGD(image->img.gd, size_ptr, format);
+        return msSaveImageBufferGD(image, size_ptr, format);
     }
 #ifdef USE_AGG
     else if( MS_DRIVER_AGG(image->format) )
@@ -768,9 +820,23 @@ void msFreeImage(imageObj *image)
 {
     if (image)
     {
-        if( MS_RENDERER_GD(image->format) ) {
+        if(MS_RENDERER_PLUGIN(image->format)) {
+            rendererVTableObj *renderer = image->format->vtable;
+            if(renderer->supports_imagecache) {
+                tileCacheObj *next,*cur = image->tilecache;
+                while(cur) {
+                    renderer->freeTile(cur->data);
+                    next = cur->next;
+                    free(cur);
+                    cur = next;
+                }
+                image->ntiles = 0;
+            }
+        	renderer->freeImage(image);
+        }
+        else if( MS_RENDERER_GD(image->format) ) {
             if( image->img.gd != NULL )
-                msFreeImageGD(image->img.gd);
+                msFreeImageGD(image);
 #ifdef USE_AGG
         } else if( MS_RENDERER_AGG(image->format) ) {
             msFreeImageAGG(image);
@@ -1315,18 +1381,38 @@ imageObj *msImageCreate(int width, int height, outputFormatObj *format,
                         char *imagepath, char *imageurl, mapObj *map)
 {
     imageObj *image = NULL;
-
     if( MS_RENDERER_GD(format) )
     {
         image = msImageCreateGD(width, height, format,
-                                imagepath, imageurl);
+                                imagepath, imageurl, map->resolution, map->defresolution);
         if( image != NULL && map) msImageInitGD( image, &map->imagecolor );
     }
+    else if(MS_RENDERER_PLUGIN(format)) {
+    	image = format->vtable->createImage(width,height,format,&map->imagecolor);
+    	image->format = format;
+		format->refcount++;
+
+		image->width = width;
+		image->height = height;
+		image->imagepath = NULL;
+		image->imageurl = NULL;
+                image->tilecache = NULL;
+                image->ntiles = 0;
+                image->resolution = map->resolution;
+                image->resolutionfactor = map->resolution/map->defresolution;
+
+		if (imagepath)
+			image->imagepath = strdup(imagepath);
+		if (imageurl)
+			image->imageurl = strdup(imageurl);
+
+		return image;
+    }
 #ifdef USE_AGG
     else if( MS_RENDERER_AGG(format) )
     {
         image = msImageCreateAGG(width, height, format,
-                                imagepath, imageurl);
+                                 imagepath, imageurl, map->resolution, map->defresolution);
         if( image != NULL && map) msImageInitAGG( image, &map->imagecolor );
     }
 #endif
@@ -1370,7 +1456,9 @@ imageObj *msImageCreate(int width, int height, outputFormatObj *format,
         image->height = height;
         image->imagepath = NULL;
         image->imageurl = NULL;
-            
+        image->resolution = map->resolution;
+        image->resolutionfactor = map->resolution/map->defresolution;
+
         if (imagepath)
             image->imagepath = strdup(imagepath);
         if (imageurl)
@@ -1381,7 +1469,7 @@ imageObj *msImageCreate(int width, int height, outputFormatObj *format,
     else if( MS_RENDERER_IMAGEMAP(format) )
     {
         image = msImageCreateIM(width, height, format,
-                                imagepath, imageurl);
+                                imagepath, imageurl, map->resolution, map->defresolution);
         if( image != NULL ) msImageInitIM( image );
     }
 #ifdef USE_MING_FLASH
@@ -1420,7 +1508,11 @@ imageObj *msImageCreate(int width, int height, outputFormatObj *format,
 void  msTransformShape(shapeObj *shape, rectObj extent, double cellsize, 
                        imageObj *image)   
 {
+	if (image != NULL && MS_RENDERER_PLUGIN(image->format)) {
+		image->format->vtable->transformShape(shape, extent, cellsize);
 
+		return;
+	}
 #ifdef USE_MING_FLASH
     if (image != NULL && MS_RENDERER_SWF(image->format) )
     {
@@ -1622,9 +1714,9 @@ shapeObj *msOffsetPolyline(shapeObj *p, double offsetx, double offsety) {
     }
   } else { /* normal offset (eg. drop shadow) */
     for (i = 0; i < p->numlines; i++) {
-      for(j=1; j<p->line[i].numpoints; j++) {
-        ret->line[i].point[j-1].x=p->line[i].point[j-1].x+offsetx;
-        ret->line[i].point[j-1].y=p->line[i].point[j-1].y+offsety;
+      for(j=0; j<p->line[i].numpoints; j++) {
+        ret->line[i].point[j].x=p->line[i].point[j].x+offsetx;
+        ret->line[i].point[j].y=p->line[i].point[j].y+offsety;
       }
     }
   }
@@ -1782,3 +1874,94 @@ int msCheckParentPointer(void* p, char *objname) {
     return MS_SUCCESS;
 }
 
+inline void msBufferInit(bufferObj *buffer) {
+    buffer->data=NULL;
+    buffer->size=0;
+    buffer->available=0;
+    buffer->_next_allocation_size = MS_DEFAULT_BUFFER_ALLOC;
+}
+
+inline void msBufferResize(bufferObj *buffer, size_t target_size){
+    while(buffer->available <= target_size) {
+        buffer->data = realloc(buffer->data,buffer->available+buffer->_next_allocation_size);
+        buffer->available += buffer->_next_allocation_size;
+        buffer->_next_allocation_size *= 2;
+    }
+}
+
+inline void msBufferAppend(bufferObj *buffer, void *data, size_t length) {
+    if(buffer->available < buffer->size+length) {
+        msBufferResize(buffer,buffer->size+length);
+    }
+    memcpy(&(buffer->data[buffer->size]),data,length);
+    buffer->size += length;
+}
+
+inline void msBufferFree(bufferObj *buffer) {
+    if(buffer->available>0)
+        free(buffer->data);
+}
+
+
+void msFreeRasterBuffer(rasterBufferObj *b) {
+    msFree(b->pixelbuffer);
+}
+
+/*
+** Issue #3043: Layer extent comparison short circuit.
+**
+** msExtentsOverlap()
+**
+** Returns MS_TRUE if map extent and layer extent overlap, 
+** MS_FALSE if they are disjoint, and MS_UNKNOWN if there is 
+** not enough info to calculate a deterministic answer.
+**
+*/
+int msExtentsOverlap(mapObj *map, layerObj *layer)
+{
+    rectObj map_extent;
+    rectObj layer_extent;
+    
+    /* No extent info? Nothing we can do, return MS_UNKNOWN. */
+    if( (map->extent.minx == -1) && (map->extent.miny == -1) && (map->extent.maxx == -1 ) && (map->extent.maxy == -1) ) return MS_UNKNOWN;
+    if( (layer->extent.minx == -1) && (layer->extent.miny == -1) && (layer->extent.maxx == -1 ) && (layer->extent.maxy == -1) ) return MS_UNKNOWN;
+        
+#ifdef USE_PROJ
+
+    /* No map projection? Let someone else sort this out. */
+    if( ! (map->projection.numargs > 0) ) 
+        return MS_UNKNOWN;
+
+    /* No layer projection? Perform naive comparison, because they are 
+    ** in the same projection. */
+    if( ! (layer->projection.numargs > 0) ) 
+        return msRectOverlap( &(map->extent), &(layer->extent) );
+    
+    /* We need to transform our rectangles for comparison, 
+    ** so we will work with copies and leave the originals intact. */
+    MS_COPYRECT(&map_extent, &(map->extent) );
+    MS_COPYRECT(&layer_extent, &(layer->extent) );
+
+    /* Transform map extents into geographics for comparison. */
+    if( msProjectRect(&(map->projection), &(map->latlon), &map_extent) )
+        return MS_UNKNOWN;
+        
+    /* Transform layer extents into geographics for comparison. */
+    if( msProjectRect(&(layer->projection), &(map->latlon), &layer_extent) )
+        return MS_UNKNOWN;
+
+    /* Simple case? Return simple answer. */
+    if ( map_extent.minx < map_extent.maxx && layer_extent.minx < layer_extent.maxx )
+        return msRectOverlap( &(map_extent), &(layer_extent) );
+        
+    /* Uh oh, one of the rects crosses the dateline!
+    ** Let someone else handle it. */
+    return MS_UNKNOWN;
+   
+#else
+    /* No proj? Naive comparison. */
+    if( msRectOverlap( &(map->extent), &(layer->extent) ) ) return MS_TRUE;
+    return MS_FALSE;
+#endif
+
+}
