@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: mapwcs.c 8628 2009-03-03 05:24:17Z sdlime $
+ * $Id: mapwcs.c 9492 2009-10-19 14:55:32Z aboudreault $
  *
  * Project:  MapServer
  * Purpose:  OpenGIS Web Coverage Server (WCS) Implementation.
@@ -32,7 +32,7 @@
 #include "mapthread.h"
 #include <assert.h>
 
-MS_CVSID("$Id: mapwcs.c 8628 2009-03-03 05:24:17Z sdlime $")
+MS_CVSID("$Id: mapwcs.c 9492 2009-10-19 14:55:32Z aboudreault $")
 
 #ifdef USE_WCS_SVR
 
@@ -127,6 +127,7 @@ int msWCSException(mapObj *map, const char *code, const char *locator,
                    const char *version )
 {
   char *pszEncodedVal = NULL;
+  const char *encoding;
 
   if( version == NULL )
       version = "1.0.0";
@@ -134,7 +135,12 @@ int msWCSException(mapObj *map, const char *code, const char *locator,
   if( msOWSParseVersionString(version) >= OWS_1_1_0 )
       return msWCSException11( map, code, locator, version );
 
-  msIO_printf("Content-type: application/vnd.ogc.se_xml%c%c",10,10);
+  encoding = msOWSLookupMetadata(&(map->web.metadata), "CO", "encoding");
+  if (encoding)
+      msIO_printf("Content-type: application/vnd.ogc.se_xml; charset=%s%c%c", encoding,10,10);
+  else
+      msIO_printf("Content-type: application/vnd.ogc.se_xml%c%c",10,10);
+
   /* msIO_printf("Content-type: text/xml%c%c",10,10); */
 
   msOWSPrintEncodeMetadata(stdout, &(map->web.metadata), NULL, "wcs_encoding", OWS_NOERR, "<?xml version='1.0' encoding=\"%s\" ?>\n", "ISO-8859-1");
@@ -374,8 +380,10 @@ static int msWCSParseRequest(cgiRequestObj *request, wcsParamsObj *params, mapOb
 	     params->interpolation = strdup(request->ParamValues[i]);
        else if(strcasecmp(request->ParamNames[i], "SERVICE") == 0)
 	     params->service = strdup(request->ParamValues[i]);
-	   else if(strcasecmp(request->ParamNames[i], "SECTION") == 0)
-	     params->section = strdup(request->ParamValues[i]); /* TODO: validate value here */
+       else if(strcasecmp(request->ParamNames[i], "SECTION") == 0) /* 1.0 */
+           params->section = strdup(request->ParamValues[i]); /* TODO: validate value here */
+       else if(strcasecmp(request->ParamNames[i], "SECTIONS") == 0) /* 1.1 */
+           params->section = strdup(request->ParamValues[i]); /* TODO: validate value here */
 
        /* GetCoverage parameters. */
        else if(strcasecmp(request->ParamNames[i], "BBOX") == 0) {
@@ -735,7 +743,10 @@ static int msWCSGetCapabilities_ContentMetadata(mapObj *map, wcsParamsObj *param
            "   xsi:schemaLocation=\"http://www.opengis.net/wcs %s/wcs/%s/wcsCapabilities.xsd\">\n", params->version, params->updatesequence, msOWSGetSchemasLocation(map), params->version); 
 
   for(i=0; i<map->numlayers; i++)
-    msWCSGetCapabilities_CoverageOfferingBrief((GET_LAYER(map, i)), params);
+  {
+      if( msWCSGetCapabilities_CoverageOfferingBrief((GET_LAYER(map, i)), params) != MS_SUCCESS )
+          return MS_FAILURE;
+  }
 
   /* done */
   msIO_printf("</ContentMetadata>\n");
@@ -754,6 +765,9 @@ static int msWCSGetCapabilities(mapObj *map, wcsParamsObj *params, cgiRequestObj
   int wcsSupportedVersions[] = {OWS_1_1_1, OWS_1_1_0, OWS_1_0_0};
   int wcsNumSupportedVersions = 3;
   const char *updatesequence=NULL;
+  const char *encoding;
+
+  encoding = msOWSLookupMetadata(&(map->web.metadata), "CO", "encoding");
 
 /* if version is not passed/set, set it to 1.1.1, this will trigger
     msOWSNegotiateVersion to handle accordingly
@@ -803,7 +817,10 @@ static int msWCSGetCapabilities(mapObj *map, wcsParamsObj *params, cgiRequestObj
       strcasecmp(params->section, "/WCS_Capabilities/Capability") != 0 &&
       strcasecmp(params->section, "/WCS_Capabilities/ContentMetadata") != 0 &&
       strcasecmp(params->section, "/") != 0) {
-      msIO_printf("Content-type: application/vnd.ogc.se_xml%c%c",10,10);
+      if (encoding)
+          msIO_printf("Content-type: application/vnd.ogc.se_xml; charset=%s%c%c", encoding,10,10);
+      else
+          msIO_printf("Content-type: application/vnd.ogc.se_xml%c%c",10,10);
       msSetError( MS_WCSERR,
         "Invalid SECTION parameter \"%s\"",
         "msWCSGetCapabilities()", params->section);
@@ -812,7 +829,10 @@ static int msWCSGetCapabilities(mapObj *map, wcsParamsObj *params, cgiRequestObj
   }
 
   else {
-    msIO_printf("Content-type: text/xml%c%c",10,10);
+      if (encoding)
+          msIO_printf("Content-type: text/xml; charset=%s%c%c", encoding,10,10);
+      else
+          msIO_printf("Content-type: text/xml%c%c",10,10);
   
     /* print common capability elements  */
     /* TODO: DocType? */
@@ -1133,9 +1153,11 @@ static int msWCSDescribeCoverage(mapObj *map, wcsParamsObj *params)
   const char *updatesequence=NULL;
   char **coverages=NULL;
   int numcoverages=0;
+  const char *encoding;
 
   char *coverageName=NULL; 
 
+  encoding = msOWSLookupMetadata(&(map->web.metadata), "CO", "encoding");
 /* -------------------------------------------------------------------- */
 /*      1.1.x is sufficiently different we have a whole case for        */
 /*      it.  The remainder of this function is for 1.0.0.               */
@@ -1171,7 +1193,10 @@ static int msWCSDescribeCoverage(mapObj *map, wcsParamsObj *params)
     updatesequence = strdup("0");
 
   /* printf("Content-type: application/vnd.ogc.se_xml%c%c",10,10); */
-  msIO_printf("Content-type: text/xml%c%c",10,10);
+  if (encoding)
+      msIO_printf("Content-type: text/xml; charset=%s%c%c", encoding,10,10);
+  else
+      msIO_printf("Content-type: text/xml%c%c",10,10);
 
   /* print common capability elements  */
   msOWSPrintEncodeMetadata(stdout, &(map->web.metadata), NULL, "wcs_encoding", OWS_NOERR, "<?xml version='1.0' encoding=\"%s\" ?>\n", "ISO-8859-1");
@@ -1434,7 +1459,7 @@ static int msWCSGetCoverage(mapObj *map, cgiRequestObj *request,
   /* fill in bands rangeset info, if required.  */
   msWCSSetDefaultBandsRangeSetInfo(params, &cm, lp);
 
-  /* handle the response CRS, that is set the map object projection */
+  /* handle the response CRS, that is, set the map object projection */
   if(params->response_crs || params->crs ) {
     int iUnits;
     const char *crs_to_use = params->response_crs;
@@ -1442,18 +1467,16 @@ static int msWCSGetCoverage(mapObj *map, cgiRequestObj *request,
     if( crs_to_use == NULL )
       crs_to_use = params->crs;
 
-    if (strncasecmp(crs_to_use, "EPSG:", 5) == 0
-        || strncasecmp(crs_to_use,"urn:ogc:def:crs:",16) == 0 ) {
+    if (strncasecmp(crs_to_use, "EPSG:", 5) == 0 || strncasecmp(crs_to_use,"urn:ogc:def:crs:",16) == 0 ) {
       if (msLoadProjectionString(&(map->projection), (char *) crs_to_use) != 0)
-          return msWCSException( map, NULL, NULL,params->version);
+        return msWCSException( map, NULL, NULL,params->version);
     } else if( strcasecmp(crs_to_use,"imageCRS") == 0 ) {
-        /* use layer native CRS, and rework bounding box accordingly */
-        if( msWCSGetCoverage_ImageCRSSetup( map, request, params, &cm, lp )
-            != MS_SUCCESS )
-            return MS_FAILURE;
+      /* use layer native CRS, and rework bounding box accordingly */
+      if( msWCSGetCoverage_ImageCRSSetup( map, request, params, &cm, lp ) != MS_SUCCESS )
+        return MS_FAILURE;
     } else {  /* should we support WMS style AUTO: projections? (not for now) */
       msSetError(MS_WCSERR, "Unsupported SRS namespace (only EPSG currently supported).", "msWCSGetCoverage()");
-      return msWCSException(map, "InvalidParameterValue", "srs",params->version);
+      return msWCSException(map, "InvalidParameterValue", "srs", params->version);
     }
 
     iUnits = GetMapserverUnitUsingProj(&(map->projection));
@@ -1471,28 +1494,24 @@ static int msWCSGetCoverage(mapObj *map, cgiRequestObj *request,
     /* check format of TIME parameter */
     if(strchr(params->time, ',')) {
       msSetError( MS_WCSERR, "Temporal lists are not supported, only individual values.", "msWCSGetCoverage()" );
-      return msWCSException(map, "InvalidParameterValue", "time",
-                            params->version);
+      return msWCSException(map, "InvalidParameterValue", "time", params->version);
     }
     if(strchr(params->time, '/')) {
       msSetError( MS_WCSERR, "Temporal ranges are not supported, only individual values.", "msWCSGetCoverage()" );
-      return msWCSException(map, "InvalidParameterValue", "time",
-                            params->version);
+      return msWCSException(map, "InvalidParameterValue", "time", params->version);
     }
       
     /* TODO: will need to expand this check if a time period is supported */
     value = msOWSLookupMetadata(&(lp->metadata), "COM", "timeposition");
     if(!value) {
       msSetError( MS_WCSERR, "The coverage does not support temporal subsetting.", "msWCSGetCoverage()" );
-      return msWCSException(map, "InvalidParameterValue", "time", 
-                            params->version );
+      return msWCSException(map, "InvalidParameterValue", "time", params->version );
     }
     
     /* check if timestamp is covered by the wcs_timeposition definition */
     if (msValidateTimeValue(params->time, value) == MS_FALSE) {
       msSetError( MS_WCSERR, "The coverage does not have a time position of %s.", "msWCSGetCoverage()", params->time );
-      return msWCSException(map, "InvalidParameterValue", "time",
-                            params->version);
+      return msWCSException(map, "InvalidParameterValue", "time", params->version);
     }
       
     /* make sure layer is tiled appropriately */
@@ -1527,84 +1546,72 @@ static int msWCSGetCoverage(mapObj *map, cgiRequestObj *request,
   }
            
   if( strncasecmp(params->version,"1.0",3) == 0 )
-      status = msWCSGetCoverageBands10( map, request, params, lp, &bandlist );
+    status = msWCSGetCoverageBands10( map, request, params, lp, &bandlist );
   else
-      status = msWCSGetCoverageBands11( map, request, params, lp, &bandlist );
+    status = msWCSGetCoverageBands11( map, request, params, lp, &bandlist );
   if( status != MS_SUCCESS )
-      return status;
+    return status;
 
   /* did we get BBOX values? if not use the exent stored in the coverageMetadataObj */
-  if( fabs((params->bbox.maxx - params->bbox.minx)) < 0.000000000001  
-      || fabs(params->bbox.maxy - params->bbox.miny) < 0.000000000001 ) {
+  if( fabs((params->bbox.maxx - params->bbox.minx)) < 0.000000000001  || fabs(params->bbox.maxy - params->bbox.miny) < 0.000000000001 ) {
+    params->bbox = cm.extent;
 
-      params->bbox = cm.extent;
-
-      /* WCS 1.1 boundbox is center of pixel oriented. */
-      if( strncasecmp(params->version,"1.1",3) == 0 )
-      {
-          params->bbox.minx += cm.geotransform[1]/2 + cm.geotransform[2]/2;
-          params->bbox.maxx -= cm.geotransform[1]/2 + cm.geotransform[2]/2;
-          params->bbox.maxy += cm.geotransform[4]/2 + cm.geotransform[5]/2;
-          params->bbox.miny -= cm.geotransform[4]/2 + cm.geotransform[5]/2;
-      }
+    /* WCS 1.1 boundbox is center of pixel oriented. */
+    if( strncasecmp(params->version,"1.1",3) == 0 ) {
+      params->bbox.minx += cm.geotransform[1]/2 + cm.geotransform[2]/2;
+      params->bbox.maxx -= cm.geotransform[1]/2 + cm.geotransform[2]/2;
+      params->bbox.maxy += cm.geotransform[4]/2 + cm.geotransform[5]/2;
+      params->bbox.miny -= cm.geotransform[4]/2 + cm.geotransform[5]/2;
+    }
   }
 
   /* WCS 1.1+ GridOrigin is effectively resetting the minx/maxy 
      BOUNDINGBOX values, so apply that here */
-  if( params->originx != 0.0 || params->originy != 0.0 )
-  {
-      /* should never be 1.0 in this logic. */
-      assert( strncasecmp(params->version,"1.0",3) != 0 );
-      params->bbox.minx = params->originx;
-      params->bbox.maxy = params->originy;
+  if( params->originx != 0.0 || params->originy != 0.0 ) {
+    assert( strncasecmp(params->version,"1.0",3) != 0 ); /* should always be 1.0 in this logic. */
+    params->bbox.minx = params->originx;
+    params->bbox.maxy = params->originy;
   }
     
-  /* if necessary, project the BBOX */
+  /* if necessary, project the BBOX to the map->projection */
+  if(params->response_crs && params->crs) {
+    projectionObj tmp_proj;
+
+    msInitProjection(&tmp_proj);
+    if (msLoadProjectionString(&tmp_proj, (char *) params->crs) != 0)
+      return msWCSException( map, NULL, NULL, params->version);
+    msProjectRect(&tmp_proj, &map->projection, &(params->bbox));
+    msFreeProjection(&tmp_proj);
+  }
 
   /* in WCS 1.1 the default is full resolution */
-  if( strncasecmp(params->version,"1.1",3) == 0 
-      && params->resx == 0.0 && params->resy == 0.0 ) {
-
+  if( strncasecmp(params->version,"1.1",3) == 0 && params->resx == 0.0 && params->resy == 0.0 ) {
     params->resx = cm.geotransform[1];
     params->resy = fabs(cm.geotransform[5]);
   }
 
   /* compute width/height from BBOX and cellsize.  */
-  if( (params->resx == 0.0 || params->resy == 0.0) 
-      && params->width != 0 && params->height != 0 ) {
-
-    /* should always be 1.0 in this logic. */
-    assert( strncasecmp(params->version,"1.0",3) == 0 );
-
+  if( (params->resx == 0.0 || params->resy == 0.0) && params->width != 0 && params->height != 0 ) {
+    assert( strncasecmp(params->version,"1.0",3) == 0 ); /* should always be 1.0 in this logic. */
     params->resx = (params->bbox.maxx -params->bbox.minx) / params->width;
     params->resy = (params->bbox.maxy -params->bbox.miny) / params->height;
   }
     
   /* compute cellsize/res from bbox and raster size. */
-  if( (params->width == 0 || params->height == 0) 
-      && params->resx != 0 && params->resy != 0 ) {
+  if( (params->width == 0 || params->height == 0) && params->resx != 0 && params->resy != 0 ) {
 
     /* WCS 1.0 boundbox is edge of pixel oriented. */
-    if( strncasecmp(params->version,"1.0",3) == 0 )
-    {
-        params->width = (int) ((params->bbox.maxx - params->bbox.minx) 
-                               / params->resx + 0.5);
-        params->height = (int) ((params->bbox.maxy - params->bbox.miny) 
-                                / params->resy + 0.5);
-    }
-    else
-    {
-        params->width = (int) ((params->bbox.maxx - params->bbox.minx) 
-                               / params->resx + 1.000001);
-        params->height = (int) ((params->bbox.maxy - params->bbox.miny) 
-                                / params->resy + 1.000001);
+    if( strncasecmp(params->version,"1.0",3) == 0 ) {
+      params->width = (int) ((params->bbox.maxx - params->bbox.minx) / params->resx + 0.5);
+      params->height = (int) ((params->bbox.maxy - params->bbox.miny) / params->resy + 0.5);
+    } else {
+      params->width = (int) ((params->bbox.maxx - params->bbox.minx) / params->resx + 1.000001);
+      params->height = (int) ((params->bbox.maxy - params->bbox.miny) / params->resy + 1.000001);
 
-        /* recompute bounding box so we get exactly the origin and
-           resolution requested. */
-        params->bbox.maxx = params->bbox.minx 
-            + (params->width-1) * params->resx;
-        params->bbox.miny = params->bbox.maxy 
-            - (params->height-1) * params->resy;
+      /* recompute bounding box so we get exactly the origin and
+         resolution requested. */
+      params->bbox.maxx = params->bbox.minx + (params->width-1) * params->resx;
+      params->bbox.miny = params->bbox.maxy - (params->height-1) * params->resy;
     }
   }
 
@@ -1618,28 +1625,22 @@ static int msWCSGetCoverage(mapObj *map, cgiRequestObj *request,
 
   /* Do we need to force special handling?  */
   if( fabs(params->resx/params->resy - 1.0) > 0.001 ) {
-      map->gt.need_geotransform = MS_TRUE;
-      if( map->debug )
-          msDebug( "RESX and RESY don't match.  Using geotransform/resample.\n");
+    map->gt.need_geotransform = MS_TRUE;
+    if( map->debug ) msDebug( "RESX and RESY don't match.  Using geotransform/resample.\n");
   }
 
   /* Do we have a specified interpolation method */
-  if( params->interpolation != NULL )
-  {
-      if( strncasecmp(params->interpolation,"NEAREST",7) == 0 )
-          msLayerSetProcessingKey(lp, "RESAMPLE", "NEAREST");
-      else if( strcasecmp(params->interpolation,"BILINEAR") == 0 )
-          msLayerSetProcessingKey(lp, "RESAMPLE", "BILINEAR");
-      else if( strcasecmp(params->interpolation,"AVERAGE") == 0 )
-          msLayerSetProcessingKey(lp, "RESAMPLE", "AVERAGE");
-      else
-      {
-          msSetError( MS_WCSERR, 
-                      "INTERPOLATION=%s specifies an unsupported interpolation method.",
-                      "msWCSGetCoverage()",
-                      params->interpolation );
-          return msWCSException(map, "InvalidParameterValue", "interpolation", params->version);
-      }
+  if( params->interpolation != NULL ) {
+    if( strncasecmp(params->interpolation,"NEAREST",7) == 0 )
+      msLayerSetProcessingKey(lp, "RESAMPLE", "NEAREST");
+    else if( strcasecmp(params->interpolation,"BILINEAR") == 0 )
+      msLayerSetProcessingKey(lp, "RESAMPLE", "BILINEAR");
+    else if( strcasecmp(params->interpolation,"AVERAGE") == 0 )
+      msLayerSetProcessingKey(lp, "RESAMPLE", "AVERAGE");
+    else {
+      msSetError( MS_WCSERR, "INTERPOLATION=%s specifies an unsupported interpolation method.", "msWCSGetCoverage()", params->interpolation );
+      return msWCSException(map, "InvalidParameterValue", "interpolation", params->version);
+    }
   }
    
   /* apply region and size to map object.  */
@@ -1647,12 +1648,11 @@ static int msWCSGetCoverage(mapObj *map, cgiRequestObj *request,
   map->height = params->height;
 
   /* adjust OWS BBOX to MapServer's pixel model */
-  if( strncasecmp(params->version,"1.0",3) == 0 )
-  {
-      params->bbox.minx += params->resx*0.5;
-      params->bbox.miny += params->resy*0.5;
-      params->bbox.maxx -= params->resx*0.5;
-      params->bbox.maxy -= params->resy*0.5;
+  if( strncasecmp(params->version,"1.0",3) == 0 ) {
+    params->bbox.minx += params->resx*0.5;
+    params->bbox.miny += params->resy*0.5;
+    params->bbox.maxx -= params->resx*0.5;
+    params->bbox.maxy -= params->resy*0.5;
   }
 
   map->extent = params->bbox;
@@ -1723,11 +1723,11 @@ static int msWCSGetCoverage(mapObj *map, cgiRequestObj *request,
     msSetError(MS_WCSERR, "The map outputformat is missing!", "msWCSGetCoverage()");
     return msWCSException(map, NULL, NULL, params->version );
   } else if( MS_RENDERER_GD(map->outputformat) ) {
-    image = msImageCreateGD(map->width, map->height, map->outputformat, map->web.imagepath, map->web.imageurl);        
+    image = msImageCreateGD(map->width, map->height, map->outputformat, map->web.imagepath, map->web.imageurl, map->resolution, map->defresolution);        
     if( image != NULL ) msImageInitGD( image, &map->imagecolor );
 #ifdef USE_AGG
   } else if( MS_RENDERER_AGG(map->outputformat) ) {
-    image = msImageCreateAGG(map->width, map->height, map->outputformat, map->web.imagepath, map->web.imageurl);        
+     image = msImageCreateAGG(map->width, map->height, map->outputformat, map->web.imagepath, map->web.imageurl, map->resolution, map->defresolution);        
     if( image != NULL ) msImageInitAGG( image, &map->imagecolor );
 #endif
   } else if( MS_RENDERER_RAWDATA(map->outputformat) ) {
@@ -1937,8 +1937,11 @@ int msWCSGetCoverageMetadata( layerObj *layer, coverageMetadataObj *cm )
   /* -------------------------------------------------------------------- */
   /*      If we have "virtual dataset" metadata on the layer, then use    */
   /*      that in preference to inspecting the file(s).                   */
+  /*      We require extent and either size or resolution.                */
   /* -------------------------------------------------------------------- */
-  if( msOWSLookupMetadata(&(layer->metadata), "COM", "extent") != NULL ) {
+  if( msOWSLookupMetadata(&(layer->metadata), "COM", "extent") != NULL 
+      && (msOWSLookupMetadata(&(layer->metadata), "COM", "resolution") != NULL
+          || msOWSLookupMetadata(&(layer->metadata), "COM", "size") != NULL) ){
     const char *value;
 
     /* get extent */
@@ -1999,7 +2002,7 @@ int msWCSGetCoverageMetadata( layerObj *layer, coverageMetadataObj *cm )
 
     /* do we have information to do anything */
     if( cm->xresolution == 0.0 || cm->yresolution == 0.0 || cm->xsize == 0 || cm->ysize == 0 ) {
-      msSetError( MS_WCSERR, "Failed to collect extent and resolution for WCS coverage from metadata.  Need value wcs|ows_resolution or wcs|ows_size values.", "msWCSGetCoverageMetadata()");
+      msSetError( MS_WCSERR, "Failed to collect extent and resolution for WCS coverage from metadata for layer '%s'.  Need value wcs|ows_resolution or wcs|ows_size values.", "msWCSGetCoverageMetadata()", layer->name );
       return MS_FAILURE;
     }
         
@@ -2041,8 +2044,9 @@ int msWCSGetCoverageMetadata( layerObj *layer, coverageMetadataObj *cm )
   
     msGDALInitialize();
 
+    msTryBuildPath3(szPath,  layer->map->mappath, layer->map->shapepath, layer->data);
     msAcquireLock( TLOCK_GDAL );
-    hDS = GDALOpen(msBuildPath3(szPath,  layer->map->mappath, layer->map->shapepath, layer->data) , GA_ReadOnly );
+    hDS = GDALOpen( szPath, GA_ReadOnly );
     if( hDS == NULL ) {
       msReleaseLock( TLOCK_GDAL );
       msSetError( MS_IOERR, "%s", "msWCSGetCoverageMetadata()", CPLGetLastErrorMsg() );
