@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: mapwcs11.c 7906 2008-09-11 16:11:48Z tomkralidis $
+ * $Id: mapwcs11.c 9498 2009-10-20 13:38:46Z aboudreault $
  *
  * Project:  MapServer
  * Purpose:  OpenGIS Web Coverage Server (WCS) 1.1.0 Implementation.  This
@@ -35,7 +35,7 @@
 #include "mapthread.h"
 #include "mapwcs.h"
 
-MS_CVSID("$Id: mapwcs11.c 7906 2008-09-11 16:11:48Z tomkralidis $")
+MS_CVSID("$Id: mapwcs11.c 9498 2009-10-20 13:38:46Z aboudreault $")
 
 #if defined(USE_WCS_SVR) && defined(USE_LIBXML2)
 
@@ -59,6 +59,7 @@ int msWCSException11(mapObj *map, const char *locator,
   char *errorString     = NULL;
   char *errorMessage    = NULL;
   char *schemasLocation = NULL;
+  const char * encoding;
 
   xmlDocPtr  psDoc      = NULL;   
   xmlNodePtr psRootNode = NULL;
@@ -67,6 +68,7 @@ int msWCSException11(mapObj *map, const char *locator,
 
   psNsOws = xmlNewNs(NULL, BAD_CAST "http://www.opengis.net/ows/1.1", BAD_CAST "ows");
 
+  encoding = msOWSLookupMetadata(&(map->web.metadata), "CO", "encoding");
   errorString = msGetErrorString("\n");
   errorMessage = msEncodeHTMLEntities(errorString);
   schemasLocation = msEncodeHTMLEntities(msOWSGetSchemasLocation(map));
@@ -79,8 +81,12 @@ int msWCSException11(mapObj *map, const char *locator,
 
   psNsOws = xmlNewNs(psRootNode, BAD_CAST "http://www.opengis.net/ows/1.1", BAD_CAST "ows");
 
-  msIO_printf("Content-type: text/xml%c%c",10,10);
-  xmlDocDumpFormatMemoryEnc(psDoc, &buffer, &size, "ISO-8859-1", 1);
+  if (encoding)
+      msIO_printf("Content-type: text/xml; charset=%s%c%c", encoding,10,10);
+  else
+      msIO_printf("Content-type: text/xml%c%c",10,10);
+
+  xmlDocDumpFormatMemoryEnc(psDoc, &buffer, &size, (encoding ? encoding : "ISO-8859-1"), 1);
     
   msIO_printf("%s", buffer);
 
@@ -263,11 +269,6 @@ static int msWCSGetCapabilities11_CoverageSummary(
     value = msOWSLookupMetadata( &(layer->metadata), "COM", "abstract");
     xmlNewChild( psCSummary, psOwsNs, BAD_CAST "Abstract", BAD_CAST value );
 
-/* -------------------------------------------------------------------- */
-/*      Identifier (layer name)                                         */
-/* -------------------------------------------------------------------- */
-    xmlNewChild( psCSummary, NULL, BAD_CAST "Identifier", BAD_CAST layer->name );
-
 /* -------------------------------------------------------------------- */
 /*      Keywords                                                        */
 /* -------------------------------------------------------------------- */
@@ -287,23 +288,6 @@ static int msWCSGetCapabilities11_CoverageSummary(
         }
     }
 
-/* -------------------------------------------------------------------- */
-/*      imageCRS bounding box.                                          */
-/* -------------------------------------------------------------------- */
-    xmlAddChild( 
-        psCSummary,
-        msOWSCommonBoundingBox( psOwsNs, "urn:ogc:def:crs:OGC::imageCRS",
-                                2, 0, 0, cm.xsize-1, cm.ysize-1 ));
-
-/* -------------------------------------------------------------------- */
-/*      native CRS bounding box.                                        */
-/* -------------------------------------------------------------------- */
-    xmlAddChild( 
-        psCSummary,
-        msOWSCommonBoundingBox( psOwsNs, cm.srs_urn, 2, 
-                                cm.extent.minx, cm.extent.miny,
-                                cm.extent.maxx, cm.extent.maxy ));
-
 /* -------------------------------------------------------------------- */
 /*      WGS84 bounding box.                                             */
 /* -------------------------------------------------------------------- */
@@ -313,17 +297,6 @@ static int msWCSGetCapabilities11_CoverageSummary(
                                      cm.llextent.minx, cm.llextent.miny,
                                      cm.llextent.maxx, cm.llextent.maxy ));
 
-/* -------------------------------------------------------------------- */
-/*      SupportedFormats                                                */
-/* -------------------------------------------------------------------- */
-    format_list = msWCSGetFormatsList11( map, layer );
-
-    if (strlen(format_list) > 0 )
-        msLibXml2GenerateList( psCSummary, NULL, "SupportedFormat", 
-                                format_list, ',' );
-
-    msFree( format_list );
-    
 /* -------------------------------------------------------------------- */
 /*      Supported CRSes.                                                */
 /* -------------------------------------------------------------------- */
@@ -344,6 +317,22 @@ static int msWCSGetCapabilities11_CoverageSummary(
                                 owned_value, ' ' );
 
     msFree( owned_value );
+
+/* -------------------------------------------------------------------- */
+/*      SupportedFormats                                                */
+/* -------------------------------------------------------------------- */
+    format_list = msWCSGetFormatsList11( map, layer );
+
+    if (strlen(format_list) > 0 )
+        msLibXml2GenerateList( psCSummary, NULL, "SupportedFormat",
+                                format_list, ',' );
+
+    msFree( format_list );
+
+/* -------------------------------------------------------------------- */
+/*      Identifier (layer name)                                         */
+/* -------------------------------------------------------------------- */
+    xmlNewChild( psCSummary, NULL, BAD_CAST "Identifier", BAD_CAST layer->name );
   
     return MS_SUCCESS;
 }
@@ -359,21 +348,24 @@ int msWCSGetCapabilities11(mapObj *map, wcsParamsObj *params,
     xmlNodePtr psTmpNode;
     char *identifier_list = NULL, *format_list = NULL;
     const char *updatesequence=NULL;
+    const char *encoding;
     xmlNsPtr psOwsNs, psXLinkNs;
-
+    char *schemaLocation = NULL;
+    char *xsi_schemaLocation = NULL;
     char *script_url=NULL, *script_url_encoded=NULL;
 
     xmlChar *buffer = NULL;
     int size = 0, i;
     msIOContext *context = NULL;
 
-    int ows_version = OWS_1_0_0;
+    int ows_version = OWS_1_1_0;
 
 /* -------------------------------------------------------------------- */
 /*      Handle updatesequence                                           */
 /* -------------------------------------------------------------------- */
 
     updatesequence = msOWSLookupMetadata(&(map->web.metadata), "CO", "updatesequence");
+    encoding = msOWSLookupMetadata(&(map->web.metadata), "CO", "encoding");
 
     if (params->updatesequence != NULL) {
       i = msOWSNegotiateUpdateSequence(params->updatesequence, updatesequence);
@@ -387,8 +379,6 @@ int msWCSGetCapabilities11(mapObj *map, wcsParamsObj *params,
       }
     }
 
-
-
 /* -------------------------------------------------------------------- */
 /*      Build list of layer identifiers available.                      */
 /* -------------------------------------------------------------------- */
@@ -422,11 +412,12 @@ int msWCSGetCapabilities11(mapObj *map, wcsParamsObj *params,
 /*      Name spaces                                                     */
 /* -------------------------------------------------------------------- */
     xmlSetNs(psRootNode, xmlNewNs(psRootNode, BAD_CAST "http://www.opengis.net/wcs/1.1", NULL));
-    psOwsNs = xmlNewNs(psRootNode, BAD_CAST MS_OWSCOMMON_OWS_NAMESPACE_URI, BAD_CAST MS_OWSCOMMON_OWS_NAMESPACE_PREFIX);
+    psOwsNs = xmlNewNs(psRootNode, BAD_CAST MS_OWSCOMMON_OWS_110_NAMESPACE_URI, BAD_CAST MS_OWSCOMMON_OWS_NAMESPACE_PREFIX);
     psXLinkNs = xmlNewNs(psRootNode, BAD_CAST MS_OWSCOMMON_W3C_XLINK_NAMESPACE_URI, BAD_CAST MS_OWSCOMMON_W3C_XLINK_NAMESPACE_PREFIX);
     xmlNewNs(psRootNode, BAD_CAST MS_OWSCOMMON_W3C_XSI_NAMESPACE_URI, BAD_CAST MS_OWSCOMMON_W3C_XSI_NAMESPACE_PREFIX);
     xmlNewNs(psRootNode, BAD_CAST MS_OWSCOMMON_OGC_NAMESPACE_URI, BAD_CAST MS_OWSCOMMON_OGC_NAMESPACE_PREFIX );
 
+    /*xmlNewProp(psRootNode, BAD_CAST " */
     xmlNewProp(psRootNode, BAD_CAST "version", BAD_CAST params->version );
 
     updatesequence = msOWSLookupMetadata(&(map->web.metadata), "CO", "updatesequence");
@@ -434,17 +425,40 @@ int msWCSGetCapabilities11(mapObj *map, wcsParamsObj *params,
     if (updatesequence)
       xmlNewProp(psRootNode, BAD_CAST "updateSequence", BAD_CAST updatesequence);
 
+    schemaLocation = msEncodeHTMLEntities( msOWSGetSchemasLocation(map) );
+    xsi_schemaLocation = strdup("http://www.opengis.net/wcs/1.1");
+    xsi_schemaLocation = msStringConcatenate(xsi_schemaLocation, " ");
+    xsi_schemaLocation = msStringConcatenate(xsi_schemaLocation, schemaLocation);
+    xsi_schemaLocation = msStringConcatenate(xsi_schemaLocation, "/wcs/1.1/wcsGetCapabilities.xsd ");
+    xsi_schemaLocation = msStringConcatenate(xsi_schemaLocation, MS_OWSCOMMON_OWS_110_NAMESPACE_URI);
+    xsi_schemaLocation = msStringConcatenate(xsi_schemaLocation, " ");
+    xsi_schemaLocation = msStringConcatenate(xsi_schemaLocation, schemaLocation);
+    xsi_schemaLocation = msStringConcatenate(xsi_schemaLocation, "/ows/1.1.0/owsAll.xsd");
+    xmlNewNsProp(psRootNode, NULL, BAD_CAST "xsi:schemaLocation", BAD_CAST xsi_schemaLocation);
+
 /* -------------------------------------------------------------------- */
 /*      Service metadata.                                               */
 /* -------------------------------------------------------------------- */
-
-    psTmpNode = xmlAddChild(psRootNode, msOWSCommonServiceIdentification(
-                                psOwsNs, map, "OGC WCS", params->version, "CO"));
+    if( params->section == NULL 
+        || strstr(params->section,"All") != NULL
+        || strstr(params->section,"ServiceIdentification") != NULL )
+    {
+        psTmpNode = xmlAddChild(psRootNode, msOWSCommonServiceIdentification(
+                                    psOwsNs, map, "OGC WCS", params->version, "CO"));
+    }
 
     /*service provider*/
-    psTmpNode = xmlAddChild(psRootNode, msOWSCommonServiceProvider(
-                                psOwsNs, psXLinkNs, map, "CO"));
+    if( params->section == NULL 
+        || strstr(params->section,"All") != NULL
+        || strstr(params->section,"ServiceProvider") != NULL )
+    {
+        psTmpNode = xmlAddChild(psRootNode, msOWSCommonServiceProvider(
+                                    psOwsNs, psXLinkNs, map, "CO"));
+    }
 
+/* -------------------------------------------------------------------- */
+/*      Operations metadata.                                            */
+/* -------------------------------------------------------------------- */
     /*operation metadata */
     if ((script_url=msOWSGetOnlineResource(map, "COM", "onlineresource", req)) == NULL 
         || (script_url_encoded = msEncodeHTMLEntities(script_url)) == NULL)
@@ -454,84 +468,91 @@ int msWCSGetCapabilities11(mapObj *map, wcsParamsObj *params,
     }
     free( script_url );
 
-/* -------------------------------------------------------------------- */
-/*      Operations metadata.                                            */
-/* -------------------------------------------------------------------- */
-    psMainNode= xmlAddChild(psRootNode,msOWSCommonOperationsMetadata(psOwsNs));
+    if( params->section == NULL 
+        || strstr(params->section,"All") != NULL
+        || strstr(params->section,"OperationsMetadata") != NULL )
+    {
+        psMainNode= xmlAddChild(psRootNode,msOWSCommonOperationsMetadata(psOwsNs));
 
 /* -------------------------------------------------------------------- */
 /*      GetCapabilities - add Sections and AcceptVersions?              */
 /* -------------------------------------------------------------------- */
-    psNode = msOWSCommonOperationsMetadataOperation( 
-        psOwsNs, psXLinkNs,
-        "GetCapabilities", OWS_METHOD_GET, script_url_encoded);
-
-    xmlAddChild(psMainNode, psNode);
-    xmlAddChild(psNode, msOWSCommonOperationsMetadataDomainType(
-                    ows_version, psOwsNs, "Parameter", "service", "WCS"));
-    xmlAddChild(psNode, msOWSCommonOperationsMetadataDomainType(
-                    ows_version, psOwsNs, "Parameter", "version", (char *)params->version));
+        psNode = msOWSCommonOperationsMetadataOperation( 
+            psOwsNs, psXLinkNs,
+            "GetCapabilities", OWS_METHOD_GET, script_url_encoded);
+        
+        xmlAddChild(psMainNode, psNode);
+        xmlAddChild(psNode, msOWSCommonOperationsMetadataDomainType(
+                        ows_version, psOwsNs, "Parameter", "service", "WCS"));
+        xmlAddChild(psNode, msOWSCommonOperationsMetadataDomainType(
+                        ows_version, psOwsNs, "Parameter", "version", (char *)params->version));
 
 /* -------------------------------------------------------------------- */
 /*      DescribeCoverage                                                */
 /* -------------------------------------------------------------------- */
-    psNode = msOWSCommonOperationsMetadataOperation(
-        psOwsNs, psXLinkNs,
-        "DescribeCoverage", OWS_METHOD_GET, script_url_encoded);
-
-    xmlAddChild(psMainNode, psNode);
-    xmlAddChild(psNode, msOWSCommonOperationsMetadataDomainType(
-                    ows_version, psOwsNs, "Parameter", "service", "WCS"));
-    xmlAddChild(psNode, msOWSCommonOperationsMetadataDomainType(
-                    ows_version, psOwsNs, "Parameter", "version", (char *)params->version));
-    xmlAddChild(psNode, msOWSCommonOperationsMetadataDomainType(
-                    ows_version, psOwsNs, "Parameter", "identifiers", identifier_list ));
-
+        psNode = msOWSCommonOperationsMetadataOperation(
+            psOwsNs, psXLinkNs,
+            "DescribeCoverage", OWS_METHOD_GET, script_url_encoded);
+        
+        xmlAddChild(psMainNode, psNode);
+        xmlAddChild(psNode, msOWSCommonOperationsMetadataDomainType(
+                        ows_version, psOwsNs, "Parameter", "service", "WCS"));
+        xmlAddChild(psNode, msOWSCommonOperationsMetadataDomainType(
+                        ows_version, psOwsNs, "Parameter", "version", (char *)params->version));
+        xmlAddChild(psNode, msOWSCommonOperationsMetadataDomainType(
+                        ows_version, psOwsNs, "Parameter", "identifiers", identifier_list ));
+        
 /* -------------------------------------------------------------------- */
 /*      GetCoverage                                                     */
 /* -------------------------------------------------------------------- */
-    psNode = msOWSCommonOperationsMetadataOperation(
-        psOwsNs, psXLinkNs,
-        "GetCoverage", OWS_METHOD_GET, script_url_encoded);
+        psNode = msOWSCommonOperationsMetadataOperation(
+            psOwsNs, psXLinkNs,
+            "GetCoverage", OWS_METHOD_GET, script_url_encoded);
+        
+        format_list = msWCSGetFormatsList11( map, NULL );
+        
+        xmlAddChild(psMainNode, psNode);
+        xmlAddChild(psNode, msOWSCommonOperationsMetadataDomainType(
+                        ows_version, psOwsNs, "Parameter", "service", "WCS"));
+        xmlAddChild(psNode, msOWSCommonOperationsMetadataDomainType(
+                        ows_version, psOwsNs, "Parameter", "version", (char *)params->version));
+        xmlAddChild(psNode, msOWSCommonOperationsMetadataDomainType(
+                        ows_version, psOwsNs, "Parameter", "Identifier", identifier_list ));
+        xmlAddChild(psNode, msOWSCommonOperationsMetadataDomainType(
+                        ows_version, psOwsNs, "Parameter", "InterpolationType", 
+                        "NEAREST_NEIGHBOUR,BILINEAR" ));
+        xmlAddChild(psNode, msOWSCommonOperationsMetadataDomainType(
+                        ows_version, psOwsNs, "Parameter", "format", format_list ));
+        xmlAddChild(psNode, msOWSCommonOperationsMetadataDomainType(
+                        ows_version, psOwsNs, "Parameter", "store", "false" ));
+        xmlAddChild(psNode, msOWSCommonOperationsMetadataDomainType(
+                        ows_version, psOwsNs, "Parameter", "GridBaseCRS", 
+                        "urn:ogc:def:crs:epsg::4326" ));
+        
+        msFree( format_list );
+    }    
 
-    format_list = msWCSGetFormatsList11( map, NULL );
-    
-    xmlAddChild(psMainNode, psNode);
-    xmlAddChild(psNode, msOWSCommonOperationsMetadataDomainType(
-                    ows_version, psOwsNs, "Parameter", "service", "WCS"));
-    xmlAddChild(psNode, msOWSCommonOperationsMetadataDomainType(
-                    ows_version, psOwsNs, "Parameter", "version", (char *)params->version));
-    xmlAddChild(psNode, msOWSCommonOperationsMetadataDomainType(
-                    ows_version, psOwsNs, "Parameter", "Identifier", identifier_list ));
-    xmlAddChild(psNode, msOWSCommonOperationsMetadataDomainType(
-                    ows_version, psOwsNs, "Parameter", "InterpolationType", 
-                    "NEAREST_NEIGHBOUR,BILINEAR" ));
-    xmlAddChild(psNode, msOWSCommonOperationsMetadataDomainType(
-                    ows_version, psOwsNs, "Parameter", "format", format_list ));
-    xmlAddChild(psNode, msOWSCommonOperationsMetadataDomainType(
-                    ows_version, psOwsNs, "Parameter", "store", "false" ));
-    xmlAddChild(psNode, msOWSCommonOperationsMetadataDomainType(
-                    ows_version, psOwsNs, "Parameter", "GridBaseCRS", 
-                    "urn:ogc:def:crs:epsg::4326" ));
-
-    msFree( format_list );
-    
 /* -------------------------------------------------------------------- */
 /*      Contents section.                                               */
 /* -------------------------------------------------------------------- */
-    psMainNode = xmlNewChild( psRootNode, NULL, BAD_CAST "Contents", NULL );
-
-    for(i=0; i<map->numlayers; i++)
+    if( params->section == NULL 
+        || strstr(params->section,"All") != NULL
+        || strstr(params->section,"Contents") != NULL )
     {
-        layerObj *layer = map->layers[i];
-        int       status;
+        psMainNode = xmlNewChild( psRootNode, NULL, BAD_CAST "Contents", NULL );
 
-        if(!msWCSIsLayerSupported(layer)) 
-            continue;
-
-        status = msWCSGetCapabilities11_CoverageSummary( 
-            map, params, req, psDoc, psMainNode, layer );
-        if(status != MS_SUCCESS) return MS_FAILURE;
+        for(i=0; i<map->numlayers; i++)
+        {
+            layerObj *layer = map->layers[i];
+            int       status;
+            
+            if(!msWCSIsLayerSupported(layer)) 
+                continue;
+            
+            status = msWCSGetCapabilities11_CoverageSummary( 
+                map, params, req, psDoc, psMainNode, layer );
+            if(status != MS_SUCCESS) return MS_FAILURE;
+        }
     }
 
 /* -------------------------------------------------------------------- */
@@ -541,11 +562,14 @@ int msWCSGetCapabilities11(mapObj *map, wcsParamsObj *params,
     if( msIO_needBinaryStdout() == MS_FAILURE )
         return MS_FAILURE;
      
-    msIO_printf("Content-type: text/xml%c%c",10,10);
+    if (encoding)
+        msIO_printf("Content-type: text/xml; charset=%s%c%c", encoding,10,10);
+    else
+        msIO_printf("Content-type: text/xml%c%c",10,10);
     
     context = msIO_getHandler(stdout);
 
-    xmlDocDumpFormatMemoryEnc(psDoc, &buffer, &size, "ISO-8859-1", 1);
+    xmlDocDumpFormatMemoryEnc(psDoc, &buffer, &size, (encoding ? encoding : "ISO-8859-1"), 1);
     msIO_contextWrite(context, buffer, size);
     xmlFree(buffer);
 
@@ -616,11 +640,6 @@ msWCSDescribeCoverage_CoverageDescription11(
     value = msOWSLookupMetadata( &(layer->metadata), "COM", "abstract");
     xmlNewChild( psCD, psOwsNs, BAD_CAST "Abstract", BAD_CAST value );
 
-/* -------------------------------------------------------------------- */
-/*      Identifier (layer name)                                         */
-/* -------------------------------------------------------------------- */
-    xmlNewChild( psCD, NULL, BAD_CAST "Identifier", BAD_CAST layer->name );
-
 /* -------------------------------------------------------------------- */
 /*      Keywords                                                        */
 /* -------------------------------------------------------------------- */
@@ -631,6 +650,11 @@ msWCSDescribeCoverage_CoverageDescription11(
             xmlNewChild(psCD, psOwsNs, BAD_CAST "Keywords", NULL),
             NULL, "Keyword", value, ',' );
 
+/* -------------------------------------------------------------------- */
+/*      Identifier (layer name)                                         */
+/* -------------------------------------------------------------------- */
+    xmlNewChild( psCD, NULL, BAD_CAST "Identifier", BAD_CAST layer->name);
+
 /* -------------------------------------------------------------------- */
 /*      Domain                                                          */
 /* -------------------------------------------------------------------- */
@@ -717,7 +741,7 @@ msWCSDescribeCoverage_CoverageDescription11(
 /*      Range                                                           */
 /* -------------------------------------------------------------------- */
     {
-        xmlNodePtr psField, psInterpMethods, psAxis;
+        xmlNodePtr psField, psInterpMethods, psAxis, psDefinition;
         const char *value;
 
         psField = 
@@ -735,15 +759,19 @@ msWCSDescribeCoverage_CoverageDescription11(
         value = msOWSGetEncodeMetadata( &(layer->metadata), "COM", 
                                         "rangeset_name", "raster" );
         xmlNewChild( psField, NULL, BAD_CAST "Identifier", BAD_CAST value );
-        
-        /* <NullValue> TODO */
-        
+       
+        psDefinition =
+            xmlNewChild(
+                xmlNewChild( psField, NULL, BAD_CAST "Definition", NULL ),
+                psOwsNs, BAD_CAST "AnyValue", NULL );
+ 
         psInterpMethods = 
             xmlNewChild( psField, NULL, BAD_CAST "InterpolationMethods", NULL );
 
+        xmlNewChild( psInterpMethods, NULL, BAD_CAST "InterpolationMethod", BAD_CAST "bilinear" );
         xmlNewChild( psInterpMethods, NULL, 
-                     BAD_CAST "DefaultMethod", BAD_CAST "nearest neighbor" );
-        xmlNewChild( psInterpMethods, NULL, BAD_CAST "OtherMethod", BAD_CAST "bilinear" );
+                     BAD_CAST "Default", BAD_CAST "nearest neighbor" );
+
 
 /* -------------------------------------------------------------------- */
 /*      Bands axis.                                                     */
@@ -823,8 +851,14 @@ int msWCSDescribeCoverage11(mapObj *map, wcsParamsObj *params)
     xmlDocPtr psDoc = NULL;       /* document pointer */
     xmlNodePtr psRootNode;
     xmlNsPtr psOwsNs, psXLinkNs;
+    char *schemaLocation = NULL;
+    char *xsi_schemaLocation = NULL;
+    const char *encoding;
+
     int i,j;
 
+    encoding = msOWSLookupMetadata(&(map->web.metadata), "CO", "encoding");
+
 /* -------------------------------------------------------------------- */
 /*      We will actually get the coverages list as a single item in     */
 /*      a string list with that item having the comma delimited         */
@@ -867,12 +901,21 @@ int msWCSDescribeCoverage11(mapObj *map, wcsParamsObj *params)
 /*      Name spaces                                                     */
 /* -------------------------------------------------------------------- */
     xmlSetNs(psRootNode, xmlNewNs(psRootNode, BAD_CAST "http://www.opengis.net/wcs/1.1", NULL));
-    psOwsNs = xmlNewNs(psRootNode, BAD_CAST MS_OWSCOMMON_OWS_NAMESPACE_URI, BAD_CAST MS_OWSCOMMON_OWS_NAMESPACE_PREFIX);
+    psOwsNs = xmlNewNs(psRootNode, BAD_CAST MS_OWSCOMMON_OWS_110_NAMESPACE_URI, BAD_CAST MS_OWSCOMMON_OWS_NAMESPACE_PREFIX);
     psXLinkNs = xmlNewNs(psRootNode, BAD_CAST MS_OWSCOMMON_W3C_XLINK_NAMESPACE_URI, BAD_CAST MS_OWSCOMMON_W3C_XLINK_NAMESPACE_PREFIX);
     xmlNewNs(psRootNode, BAD_CAST MS_OWSCOMMON_W3C_XSI_NAMESPACE_URI, BAD_CAST MS_OWSCOMMON_W3C_XSI_NAMESPACE_PREFIX);
     xmlNewNs(psRootNode, BAD_CAST MS_OWSCOMMON_OGC_NAMESPACE_URI, BAD_CAST MS_OWSCOMMON_OGC_NAMESPACE_PREFIX );
 
-    xmlNewProp(psRootNode, BAD_CAST "version", BAD_CAST params->version );
+    schemaLocation = msEncodeHTMLEntities( msOWSGetSchemasLocation(map) );
+    xsi_schemaLocation = strdup("http://www.opengis.net/wcs/1.1");
+    xsi_schemaLocation = msStringConcatenate(xsi_schemaLocation, " ");
+    xsi_schemaLocation = msStringConcatenate(xsi_schemaLocation, schemaLocation);
+    xsi_schemaLocation = msStringConcatenate(xsi_schemaLocation, "/wcs/1.1/wcsDescribeCoverage.xsd ");
+    xsi_schemaLocation = msStringConcatenate(xsi_schemaLocation, MS_OWSCOMMON_OWS_110_NAMESPACE_URI);
+    xsi_schemaLocation = msStringConcatenate(xsi_schemaLocation, " ");
+    xsi_schemaLocation = msStringConcatenate(xsi_schemaLocation, schemaLocation);
+    xsi_schemaLocation = msStringConcatenate(xsi_schemaLocation, "/ows/1.1.0/owsAll.xsd");
+    xmlNewNsProp(psRootNode, NULL, BAD_CAST "xsi:schemaLocation", BAD_CAST xsi_schemaLocation);
 
 /* -------------------------------------------------------------------- */
 /*      Generate a CoverageDescription for each requested coverage.     */
@@ -903,11 +946,14 @@ int msWCSDescribeCoverage11(mapObj *map, wcsParamsObj *params)
         if( msIO_needBinaryStdout() == MS_FAILURE )
             return MS_FAILURE;
      
-        msIO_printf("Content-type: text/xml%c%c",10,10);
+        if (encoding)
+            msIO_printf("Content-type: text/xml; charset=%s%c%c", encoding,10,10);
+        else
+            msIO_printf("Content-type: text/xml%c%c",10,10);
     
         context = msIO_getHandler(stdout);
 
-        xmlDocDumpFormatMemoryEnc(psDoc, &buffer, &size, "ISO-8859-1", 1);
+        xmlDocDumpFormatMemoryEnc(psDoc, &buffer, &size, (encoding ? encoding : "ISO-8859-1"), 1);
         msIO_contextWrite(context, buffer, size);
         xmlFree(buffer);
     }
@@ -1070,6 +1116,9 @@ int  msWCSReturnCoverage11( wcsParamsObj *params, mapObj *map,
 {
     int status, i;
     char *filename = NULL;
+    const char *encoding;
+
+    encoding = msOWSLookupMetadata(&(map->web.metadata), "CO", "encoding");
 
 /* -------------------------------------------------------------------- */
 /*      Fetch the driver we will be using and check if it supports      */
@@ -1121,22 +1170,41 @@ int  msWCSReturnCoverage11( wcsParamsObj *params, mapObj *map,
 /* -------------------------------------------------------------------- */
 /*      Output stock header.                                            */
 /* -------------------------------------------------------------------- */
-    msIO_fprintf( 
-        stdout, 
-        "Content-Type: multipart/mixed; boundary=wcs%c%c"
-        "--wcs\n"
-        "Content-Type: text/xml\n"
-        "Content-ID: wcs.xml%c%c"
-        "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"
-        "<Coverages\n"
-        "     xmlns=\"http://www.opengis.net/wcs/1.1\"\n"
-        "     xmlns:ows=\"http://www.opengis.net/ows\"\n"
-        "     xmlns:xlink=\"http://www.w3.org/1999/xlink\"\n"
-        "     xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"\n"
-        "     xsi:schemaLocation=\"http://www.opengis.net/ows/1.1 ../owsCoverages.xsd\">\n"
-        "  <Coverage>\n",
-        10, 10,
-        10, 10 );
+    if (encoding)
+        msIO_fprintf( 
+            stdout, 
+            "Content-Type: multipart/mixed; boundary=wcs%c%c"
+            "--wcs\n"
+            "Content-Type: text/xml; charset=%s\n"
+            "Content-ID: wcs.xml%c%c"
+            "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"
+            "<Coverages\n"
+            "     xmlns=\"http://www.opengis.net/wcs/1.1\"\n"
+            "     xmlns:ows=\"http://www.opengis.net/ows\"\n"
+            "     xmlns:xlink=\"http://www.w3.org/1999/xlink\"\n"
+            "     xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"\n"
+            "     xsi:schemaLocation=\"http://www.opengis.net/ows/1.1 ../owsCoverages.xsd\">\n"
+            "  <Coverage>\n",
+            10, 10,
+            encoding,
+            10, 10 );
+    else
+        msIO_fprintf( 
+            stdout, 
+            "Content-Type: multipart/mixed; boundary=wcs%c%c"
+            "--wcs\n"
+            "Content-Type: text/xml\n"
+            "Content-ID: wcs.xml%c%c"
+            "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"
+            "<Coverages\n"
+            "     xmlns=\"http://www.opengis.net/wcs/1.1\"\n"
+            "     xmlns:ows=\"http://www.opengis.net/ows\"\n"
+            "     xmlns:xlink=\"http://www.w3.org/1999/xlink\"\n"
+            "     xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"\n"
+            "     xsi:schemaLocation=\"http://www.opengis.net/ows/1.1 ../owsCoverages.xsd\">\n"
+            "  <Coverage>\n",
+            10, 10,
+            10, 10 );
 
 /* -------------------------------------------------------------------- */
 /*      If we weren't able to write data under /vsimem, then we just    */
