@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: mapwfs.c 8575 2009-02-18 15:18:37Z aboudreault $
+ * $Id: mapwfs.c 9573 2009-12-04 17:11:29Z assefa $
  *
  * Project:  MapServer
  * Purpose:  WFS server implementation
@@ -28,7 +28,7 @@
 
 #include "mapserver.h"
 
-MS_CVSID("$Id: mapwfs.c 8575 2009-02-18 15:18:37Z aboudreault $")
+MS_CVSID("$Id: mapwfs.c 9573 2009-12-04 17:11:29Z assefa $")
 
 #if defined(USE_WFS_SVR)
 
@@ -46,6 +46,7 @@ MS_CVSID("$Id: mapwfs.c 8575 2009-02-18 15:18:37Z aboudreault $")
 int msWFSException(mapObj *map, const char *locator, const char *code,
                    const char *version )
 {
+    const char *encoding;
     char *schemalocation = NULL;
     /* In WFS, exceptions are always XML.
     */
@@ -56,7 +57,12 @@ int msWFSException(mapObj *map, const char *locator, const char *code,
     if( msOWSParseVersionString(version) >= OWS_1_1_0 )
       return msWFSException11( map, code, locator, version );
 
-    msIO_printf("Content-type: text/xml%c%c",10,10);
+    encoding = msOWSLookupMetadata(&(map->web.metadata), "FO", "encoding");
+    if (encoding)
+        msIO_printf("Content-type: text/xml; charset=%s%c%c", encoding,10,10);
+    else
+        msIO_printf("Content-type: text/xml%c%c",10,10);
+
     msOWSPrintEncodeMetadata(stdout, &(map->web.metadata), "FO", "encoding", OWS_NOERR,
                 "<?xml version='1.0' encoding=\"%s\" ?>\n", "ISO-8859-1");
 
@@ -277,7 +283,7 @@ int msWFSDumpLayer(mapObj *map, layerObj *lp)
    }
    else
    {
-       msIO_printf("<!-- WARNING: Optional LatLongBoundingBox could not be established for this layer.  Consider setting LAYER.EXTENT or wfs_extent metadata. Also check that your data exists in the DATA statement -->\n");
+       msIO_printf("<!-- WARNING: Optional LatLongBoundingBox could not be established for this layer.  Consider setting the EXTENT in the LAYER object, or wfs_extent metadata. Also check that your data exists in the DATA statement -->\n");
    }
 
    msOWSPrintURLType(stdout, &(lp->metadata), "FO", "metadataurl", 
@@ -304,13 +310,13 @@ int msWFSGetCapabilities(mapObj *map, wfsParamsObj *wfsparams, cgiRequestObj *re
   char *script_url=NULL, *script_url_encoded;
   const char *updatesequence=NULL;
   const char *wmtver=NULL;
+  const char *encoding;
   char tmpString[OWS_VERSION_MAXLEN];
   int wfsSupportedVersions[] = {OWS_1_1_0, OWS_1_0_0};
   int wfsNumSupportedVersions = 2;
 
   int i=0, tmpInt=0;
 
-  
   /* negotiate version */
   tmpInt = msOWSNegotiateVersion(msOWSParseVersionString(wfsparams->pszVersion), wfsSupportedVersions, 
                                  wfsNumSupportedVersions);
@@ -351,7 +357,12 @@ int msWFSGetCapabilities(mapObj *map, wfsParamsObj *wfsparams, cgiRequestObj *re
       }
   }
 
-  msIO_printf("Content-type: text/xml%c%c",10,10);
+  encoding = msOWSLookupMetadata(&(map->web.metadata), "FO", "encoding");
+    if (encoding)
+        msIO_printf("Content-type: text/xml; charset=%s%c%c", encoding,10,10);
+    else
+        msIO_printf("Content-type: text/xml%c%c",10,10);
+
   msOWSPrintEncodeMetadata(stdout, &(map->web.metadata), "FO", "encoding", OWS_NOERR,
                 "<?xml version='1.0' encoding=\"%s\" ?>\n",
                 "ISO-8859-1");
@@ -763,7 +774,11 @@ int msWFSDescribeFeatureType(mapObj *map, wfsParamsObj *paramsObj)
   /*
   ** DescribeFeatureType response
   */
-  msIO_printf("Content-type: text/xml%c%c",10,10);
+  value = msOWSLookupMetadata(&(map->web.metadata), "FO", "encoding");
+  if (value)
+      msIO_printf("Content-type: text/xml; charset=%s%c%c", value,10,10);
+  else
+      msIO_printf("Content-type: text/xml%c%c",10,10);
 
   msOWSPrintEncodeMetadata(stdout, &(map->web.metadata), "FO", "encoding", OWS_NOERR,
 			   "<?xml version='1.0' encoding=\"%s\" ?>\n",
@@ -974,7 +989,7 @@ int msWFSDescribeFeatureType(mapObj *map, wfsParamsObj *paramsObj)
 int msWFSGetFeature(mapObj *map, wfsParamsObj *paramsObj, cgiRequestObj *req)
   /* const char *wmtver, char **names, char **values, int numentries) */
 {
-  int         i, maxfeatures=-1;
+  int         i, j, maxfeatures=-1;
   const char *typename="";
   char       *script_url=NULL, *script_url_encoded;
   rectObj     bbox;
@@ -1003,8 +1018,12 @@ int msWFSGetFeature(mapObj *map, wfsParamsObj *paramsObj, cgiRequestObj *req)
   char **aFIDLayers = NULL;
   char **aFIDValues = NULL;
   int iFIDLayers = 0;
-
+  int iNumberOfFeatures = 0;
+  int iResultTypeHits = 0;
+  
   gmlNamespaceListObj *namespaceList=NULL; /* for external application schema support */
+  char **papszPropertyName = NULL;
+  int nPropertyNames = 0;
 
   /* Default filter is map extents */
   bbox = map->extent;
@@ -1013,7 +1032,12 @@ int msWFSGetFeature(mapObj *map, wfsParamsObj *paramsObj, cgiRequestObj *req)
   /*  */
   /* __TODO__ Need to support XML encoded requests */
   /*  */
-  
+
+  if (paramsObj->pszResultType != NULL)
+  {
+    if (strcasecmp(paramsObj->pszResultType, "hits") == 0)
+      iResultTypeHits = 1;
+  }  
 
   /* typename is mandatory unlsess featureid is specfied. We do not
      support featureid */
@@ -1025,12 +1049,13 @@ int msWFSGetFeature(mapObj *map, wfsParamsObj *paramsObj, cgiRequestObj *req)
       return msWFSException(map, "typename", "MissingParameterValue", paramsObj->pszVersion);
   }
 
-
   if(paramsObj->pszTypeName) {
-    int j, k;
+    int j, k, z;
     const char *pszMapSRS = NULL;
     char **tokens;
     int n=0, i=0;
+    char szTmp[256];
+    const char *pszFullName = NULL;
 
     /* keep a ref for layer use. */
     typename = paramsObj->pszTypeName;
@@ -1043,7 +1068,7 @@ int msWFSGetFeature(mapObj *map, wfsParamsObj *paramsObj, cgiRequestObj *req)
     
     /* ==================================================================== */
     /*      TODO: check if the typename contains namespaces (ex cdf:Other), */
-    /*      If that is the case extarct only the layer name.                */
+    /*      If that is the case extract only the layer name.                */
     /* ==================================================================== */
     
     if (layers==NULL || numlayers < 1) {
@@ -1068,6 +1093,7 @@ int msWFSGetFeature(mapObj *map, wfsParamsObj *paramsObj, cgiRequestObj *req)
     else
       msFreeCharArray(tokens, n);
 
+   
     pszMapSRS = msOWSGetEPSGProj(&(map->projection), &(map->web.metadata), "FO", MS_TRUE);
 
     /* Keep only selected layers, set to OFF by default. */
@@ -1085,12 +1111,12 @@ int msWFSGetFeature(mapObj *map, wfsParamsObj *paramsObj, cgiRequestObj *req)
       
       for (j=0; j<map->numlayers; j++) {
 	layerObj *lp;
-	
+        char   *pszPropertyName = NULL;
+
 	lp = GET_LAYER(map, j);
 	
 	if (msWFSIsLayerSupported(lp) && lp->name && strcasecmp(lp->name, layers[k]) == 0) {
 	  const char *pszThisLayerSRS;
-          char szBuf[32];
           rectObj ext;
 	  bLayerFound = MS_TRUE;
 	  
@@ -1100,6 +1126,91 @@ int msWFSGetFeature(mapObj *map, wfsParamsObj *paramsObj, cgiRequestObj *req)
 	    lp->template = strdup("ttt.html");
 	  }
 
+          /* set the gml_include_items METADATA */
+          if (paramsObj->pszPropertyName) 
+          {
+              pszPropertyName = paramsObj->pszPropertyName;
+
+              /*we parse the propertyname parameter only once*/
+              if (papszPropertyName == NULL)
+              {
+                  if (strlen(pszPropertyName) > 0 && pszPropertyName[0] == '(') 
+                  {
+                      tokens = msStringSplit(pszPropertyName+1, '(', &nPropertyNames);
+                      /*expecting to always have a list of property names equal to
+                        the number of layers(typename)*/
+                      if (nPropertyNames != numlayers)
+                      {
+                          if (tokens) 
+                            msFreeCharArray(tokens, nPropertyNames);
+                          msSetError(MS_WFSERR, 
+                                     "Optional PROPERTYNAME parameter. A list of properties may be specified for each type name. Example TYPENAME=name1&name2&PROPERTYNAME=(prop1,prop2)(prop1)", 
+                                     "msWFSGetFeature()");
+                          return msWFSException(map, "PROPERTYNAME", "InvalidParameterValue", paramsObj->pszVersion);
+                      }
+                      
+                      papszPropertyName = (char **)malloc(sizeof(char *)*nPropertyNames);
+                      for (i=0; i<nPropertyNames; i++) 
+                      {
+                          if (strlen(tokens[i]) > 0)
+                          {
+                              papszPropertyName[i] = strdup(tokens[i]);
+                              /* remove trailing ) */
+                              papszPropertyName[i][strlen(papszPropertyName[i])-1] = '\0';
+                          }
+                          else
+                            papszPropertyName[i] = NULL; /*should return an error*/
+                      }
+                      if (tokens) 
+                        msFreeCharArray(tokens, nPropertyNames);
+                  }
+                  else
+                  {
+                      msSetError(MS_WFSERR, 
+                                     "Optional PROPERTYNAME parameter. A list of properties may be specified for each type name. Example TYPENAME=name1&name2&PROPERTYNAME=(prop1,prop2)(prop1)", 
+                                     "msWFSGetFeature()");
+                          return msWFSException(map, "PROPERTYNAME", "InvalidParameterValue", paramsObj->pszVersion);
+                  }
+              }
+              /*do an alias replace for the current layer*/
+              if (papszPropertyName && msLayerOpen(lp) == MS_SUCCESS && msLayerGetItems(lp) == MS_SUCCESS) 
+              {
+                  for(z=0; z<lp->numitems; z++) 
+                  {
+                      if (!lp->items[z] || strlen(lp->items[z]) <= 0)
+                        continue;
+                      sprintf(szTmp, "%s_alias", lp->items[z]);
+                      pszFullName = msOWSLookupMetadata(&(lp->metadata), "G", szTmp);
+                      if (pszFullName)
+                        papszPropertyName[k] = msReplaceSubstring(papszPropertyName[i], pszFullName, lp->items[z]);
+                  
+                  }
+              } 
+                      
+
+              if (papszPropertyName) 
+              {
+                  if (strlen(papszPropertyName[k]) > 0)
+                  {
+                      msInsertHashTable(&(lp->metadata), "GML_INCLUDE_ITEMS", papszPropertyName[k]);
+
+                      /* exclude geometry if it was not asked for */
+                      if (msOWSLookupMetadata(&(lp->metadata), "OFG", "geometries") != NULL) 
+                        sprintf(szTmp, "%s", msOWSLookupMetadata(&(lp->metadata), "OFG", "geometries"));
+                      else
+                        sprintf(szTmp, OWS_GML_DEFAULT_GEOMETRY_NAME);
+                  
+                      if (strstr(papszPropertyName[k], szTmp) == NULL) 
+                        msInsertHashTable(&(lp->metadata), "GML_GEOMETRIES", "none");
+                  }
+                  else /*empty string*/
+                     msInsertHashTable(&(lp->metadata), "GML_GEOMETRIES", "none");
+
+              }
+          }
+              
+
+          
 	  /* See comments in msWFSGetCapabilities() about the rules for SRS. */
 	  if ((pszThisLayerSRS = pszMapSRS) == NULL) {
 	    pszThisLayerSRS = msOWSGetEPSGProj(&(lp->projection), &(lp->metadata), "FO", MS_TRUE);
@@ -1131,9 +1242,11 @@ int msWFSGetFeature(mapObj *map, wfsParamsObj *paramsObj, cgiRequestObj *req)
           /* if the client specifies BBOX or a spatial filter */
 
           if (msOWSGetLayerExtent(map, lp, "FO", &ext) == MS_SUCCESS) {
-            sprintf(szBuf, "init=epsg:%.10s", pszMapSRS+5);
+            char szBuf[32];
 
-            if (szBuf != NULL) {
+            if (pszMapSRS != NULL && strncmp(pszMapSRS, "EPSG:", 5) == 0) {
+              sprintf(szBuf, "init=epsg:%.10s", pszMapSRS+5);
+                
               if (msLoadProjectionString(&(map->projection), szBuf) != 0) {
                  msSetError(MS_WFSERR, "msLoadProjectionString() failed: %s", "msWFSGetFeature()", szBuf);
                  return msWFSException(map, "mapserv", "NoApplicableCode", paramsObj->pszVersion);
@@ -1160,6 +1273,14 @@ int msWFSGetFeature(mapObj *map, wfsParamsObj *paramsObj, cgiRequestObj *req)
       }
     }
   }
+  if (papszPropertyName && nPropertyNames > 0)
+  {
+    for (i=0; i<nPropertyNames; i++)
+      msFree(papszPropertyName[i]);
+
+    msFree(papszPropertyName);
+  }
+
 
   /* Validate outputformat */
   if (paramsObj->pszOutputFormat) {
@@ -1469,15 +1590,67 @@ int msWFSGetFeature(mapObj *map, wfsParamsObj *paramsObj, cgiRequestObj *req)
       msFreeCharArray(layers, numlayers);
 
 
+    /*
+      srs is defined for wfs 1.1. If it is available. If it used to set 
+      the map projection. For EPSG codes between 4000 and 5000 
+      that coordinates order follow what is defined  in the ESPG database
+      see #2899
+  */
+    if (strncmp(paramsObj->pszVersion,"1.1",3) == 0 && paramsObj->pszSrs && pszOutputSRS)
+    {     
+          /*check if the srs passed is valid. Assuming that it is an EPSG:xxx format,
+            Or urn:ogc:def:crs:EPSG:xxx format. */
+        if (strncasecmp(paramsObj->pszSrs, "EPSG:", 5) == 0)
+        {
+            if (strcasecmp(paramsObj->pszSrs, pszOutputSRS) != 0)
+            {
+                msSetError(MS_WFSERR, 
+                           "Invalid GetFeature Request: SRSNAME value should be valid for all the TYPENAMES. Please check the capabilities and reformulate your request.",
+                           "msWFSGetFeature()");
+                return msWFSException(map, "typename", "InvalidParameterValue", paramsObj->pszVersion);
+            }
+                /*we load the projection sting in the map and possibly 
+                  set the axis order*/
+            msFreeProjection(&map->projection);
+            msLoadProjectionStringEPSG(&(map->projection), paramsObj->pszSrs);
+        }
+        else if (strncasecmp(paramsObj->pszSrs, "urn:EPSG:geographicCRS:",23) == 0)
+        {
+            char epsg_string[100];
+            const char *code;
+
+            code = paramsObj->pszSrs + 23;
+            
+            sprintf( epsg_string, "EPSG:%s", code );
+            if (strcasecmp(epsg_string, pszOutputSRS) != 0)
+            {
+                msSetError(MS_WFSERR, 
+                           "Invalid GetFeature Request: SRSNAME value should be valid for all the TYPENAMES. Please check the capabilities and reformulate your request.",
+                           "msWFSGetFeature()");
+                return msWFSException(map, "typename", "InvalidParameterValue", paramsObj->pszVersion);
+            }
+                /*we load the projection sting in the map and possibly 
+                  set the axis order*/
+            msFreeProjection(&map->projection);
+            msLoadProjectionStringEPSG(&(map->projection), epsg_string);
+        }
+    }
     /* Set map output projection to which output features should be reprojected */
-    if (pszOutputSRS && strncasecmp(pszOutputSRS, "EPSG:", 5) == 0) {
-      char szBuf[32];
-      sprintf(szBuf, "init=epsg:%.10s", pszOutputSRS+5);
-      
-      if (msLoadProjectionString(&(map->projection), szBuf) != 0) {
-        msSetError(MS_WFSERR, "msLoadProjectionString() failed", "msWFSGetFeature()");
-	return msWFSException(map, "mapserv", "NoApplicableCode", paramsObj->pszVersion);
-      }
+    else if (pszOutputSRS && strncasecmp(pszOutputSRS, "EPSG:", 5) == 0) 
+    {
+        int nTmp =0;
+        msFreeProjection(&map->projection);
+        msInitProjection(&map->projection);
+
+        if (strncmp(paramsObj->pszVersion,"1.1",3) == 0)
+          nTmp = msLoadProjectionStringEPSG(&(map->projection), pszOutputSRS);
+        else
+          nTmp = msLoadProjectionString(&(map->projection), pszOutputSRS);
+
+        if (nTmp != 0) {
+            msSetError(MS_WFSERR, "msLoadProjectionString() failed", "msWFSGetFeature()");
+            return msWFSException(map, "mapserv", "NoApplicableCode", paramsObj->pszVersion);
+        }
     }
 
     /*
@@ -1486,7 +1659,12 @@ int msWFSGetFeature(mapObj *map, wfsParamsObj *paramsObj, cgiRequestObj *req)
     /* __TODO__ Using a rectangle query may not be the most efficient way */
     /* to do things here. */
     if (!bFilterSet) {
-      if(msQueryByRect(map, -1, bbox) != MS_SUCCESS) {
+
+      map->query.type = MS_QUERY_BY_RECT; /* setup the query */
+      map->query.mode = MS_QUERY_MULTIPLE;
+      map->query.rect = bbox;
+
+      if(msQueryByRect(map) != MS_SUCCESS) {
 	errorObj   *ms_error;
 	ms_error = msGetErrorObj();
 	
@@ -1497,6 +1675,14 @@ int msWFSGetFeature(mapObj *map, wfsParamsObj *paramsObj, cgiRequestObj *req)
       }
     }
 
+    /* if no results where written (TODO: this needs to be GML2/3 specific I imagine */
+    for(j=0; j<map->numlayers; j++) {
+      if (GET_LAYER(map, j)->resultcache && GET_LAYER(map, j)->resultcache->numresults > 0)
+      {
+        iNumberOfFeatures += GET_LAYER(map, j)->resultcache->numresults;
+      }
+    }
+
     /*
     ** GetFeature response
     */
@@ -1512,7 +1698,11 @@ int msWFSGetFeature(mapObj *map, wfsParamsObj *paramsObj, cgiRequestObj *req)
     */
     namespaceList = msGMLGetNamespaces(&(map->web), "G");
 
-    msIO_printf("Content-type: text/xml%c%c",10,10);
+    value = msOWSLookupMetadata(&(map->web.metadata), "FO", "encoding");
+    if (value)
+        msIO_printf("Content-type: text/xml; charset=%s%c%c", value,10,10);
+    else
+        msIO_printf("Content-type: text/xml%c%c",10,10);
 
     msOWSPrintEncodeMetadata(stdout, &(map->web.metadata), "FO", 
                              "encoding", OWS_NOERR,
@@ -1596,9 +1786,13 @@ int msWFSGetFeature(mapObj *map, wfsParamsObj *paramsObj, cgiRequestObj *req)
         }
       }
 
+     if(maxfeatures > 0 && maxfeatures < iNumberOfFeatures) {
+         iNumberOfFeatures = maxfeatures;
+     }
+
       if(paramsObj->pszVersion && strncmp(paramsObj->pszVersion,"1.1",3) == 0 )
-        msIO_printf("   xsi:schemaLocation=\"%s %sSERVICE=WFS&amp;VERSION=%s&amp;REQUEST=DescribeFeatureType&amp;TYPENAME=%s&amp;OUTPUTFORMAT=%s  http://www.opengis.net/wfs http://schemas.opengis.net/wfs/1.1.0/wfs.xsd\">\n",
-		  user_namespace_uri_encoded, script_url_encoded, encoded, encoded_typename, output_schema_format);
+        msIO_printf("   xsi:schemaLocation=\"%s %sSERVICE=WFS&amp;VERSION=%s&amp;REQUEST=DescribeFeatureType&amp;TYPENAME=%s&amp;OUTPUTFORMAT=%s  http://www.opengis.net/wfs http://schemas.opengis.net/wfs/1.1.0/wfs.xsd\" numberOfFeatures=\"%d\">\n",
+		  user_namespace_uri_encoded, script_url_encoded, encoded, encoded_typename, output_schema_format, iNumberOfFeatures);
       else
         msIO_printf("   xsi:schemaLocation=\"%s %sSERVICE=WFS&amp;VERSION=%s&amp;REQUEST=DescribeFeatureType&amp;TYPENAME=%s&amp;OUTPUTFORMAT=%s\">\n",
 		  user_namespace_uri_encoded, script_url_encoded, encoded, encoded_typename, output_schema_format);
@@ -1609,15 +1803,10 @@ int msWFSGetFeature(mapObj *map, wfsParamsObj *paramsObj, cgiRequestObj *req)
     msFree(encoded_typename);
 
     /* handle case of maxfeatures = 0 */
-    if(maxfeatures != 0)
+    if(maxfeatures != 0 && iResultTypeHits == 0)
       msGMLWriteWFSQuery(map, stdout, maxfeatures, pszNameSpace, outputformat);
-    
-    /* if no results where written (TODO: this needs to be GML2/3 specific I imagine */
-    for(i=0; i<map->numlayers; i++) {
-      if (GET_LAYER(map, i)->resultcache && GET_LAYER(map, i)->resultcache->numresults > 0)
-        break;
-    }
-    if ((i==map->numlayers) || (maxfeatures == 0)) {
+
+    if (((iNumberOfFeatures==0) || (maxfeatures == 0)) && iResultTypeHits == 0) {
       msIO_printf("   <gml:boundedBy>\n"); 
       msIO_printf("      <gml:null>missing</gml:null>\n");
       msIO_printf("   </gml:boundedBy>\n"); 
@@ -1860,6 +2049,10 @@ void msWFSFreeParamsObj(wfsParamsObj *wfsparams)
           free(wfsparams->pszFeatureId);
         if (wfsparams->pszOutputFormat)
           free(wfsparams->pszOutputFormat);
+        if (wfsparams->pszSrs)
+          free(wfsparams->pszSrs);
+        if (wfsparams->pszResultType)
+          free(wfsparams->pszResultType);
     }
 }
 
@@ -1900,6 +2093,12 @@ void msWFSParseRequest(cgiRequestObj *request, wfsParamsObj *wfsparams)
                 else if (strcasecmp(request->ParamNames[i], "BBOX") == 0)
                   wfsparams->pszBbox = strdup(request->ParamValues[i]);
                 
+                else if (strcasecmp(request->ParamNames[i], "SRSNAME") == 0)
+                  wfsparams->pszSrs = strdup(request->ParamValues[i]);
+
+                else if (strcasecmp(request->ParamNames[i], "RESULTTYPE") == 0)
+                  wfsparams->pszResultType = strdup(request->ParamValues[i]);
+
                 else if (strcasecmp(request->ParamNames[i], "TYPENAME") == 0)
                   wfsparams->pszTypeName = strdup(request->ParamValues[i]);
                 
@@ -1911,6 +2110,9 @@ void msWFSParseRequest(cgiRequestObj *request, wfsParamsObj *wfsparams)
 
                  else if (strcasecmp(request->ParamNames[i], "FEATUREID") == 0)
                   wfsparams->pszFeatureId = strdup(request->ParamValues[i]);
+                
+                else if (strcasecmp(request->ParamNames[i], "PROPERTYNAME") == 0)
+                  wfsparams->pszPropertyName = strdup(request->ParamValues[i]);
             }
         }
         /* version is optional is the GetCapabilities. If not */
@@ -1926,12 +2128,12 @@ void msWFSParseRequest(cgiRequestObj *request, wfsParamsObj *wfsparams)
 #ifdef USE_OGR
     if (request->postrequest)
     {
-        CPLXMLNode *psRoot, *psQuery, *psFilter, *psTypeName = NULL;
+        CPLXMLNode *psRoot, *psQuery, *psFilter, *psTypeName,  *psPropertyName  = NULL;
         CPLXMLNode *psGetFeature = NULL;
         CPLXMLNode  *psGetCapabilities = NULL;
         CPLXMLNode *psDescribeFeature = NULL;
         CPLXMLNode *psOperation = NULL;
-        char *pszValue, *pszSerializedFilter, *pszTmp = NULL;
+        char *pszValue, *pszSerializedFilter, *pszTmp, *pszTmp2 = NULL;
         int bMultiLayer = 0;
 
         psRoot = CPLParseXMLString(request->postrequest);
@@ -2007,6 +2209,10 @@ void msWFSParseRequest(cgiRequestObj *request, wfsParamsObj *wfsparams)
                 if (pszValue)
                   wfsparams->pszService = strdup(pszValue);
 
+                pszValue = (char*)CPLGetXMLValue(psGetFeature,  "resultType",
+                                                 NULL);
+                if (pszValue)
+                  wfsparams->pszResultType = strdup(pszValue);
 
                 pszValue = (char*)CPLGetXMLValue(psGetFeature,  "maxFeatures", 
                                                  NULL);
@@ -2036,10 +2242,17 @@ void msWFSParseRequest(cgiRequestObj *request, wfsParamsObj *wfsparams)
 /*      typenames will be build with comma between thme                 */
 /*      (typename1,typename2,...) and filters will be build with        */
 /*      bracets enclosinf the filters :(filter1)(filter2)...            */
+/*      propertynames are stored like (property1,property2)(property1)  */
 /* -------------------------------------------------------------------- */
                     while (psQuery &&  psQuery->pszValue && 
                            strcasecmp(psQuery->pszValue, "Query") == 0)
                     {
+                        /* get SRS */
+                        pszValue = (char*)CPLGetXMLValue(psGetFeature,  "srsName",
+                                                 NULL);
+                        if (pszValue)
+                          wfsparams->pszSrs = strdup(pszValue);
+
                         /* parse typenames */
                         pszValue = (char*)CPLGetXMLValue(psQuery,  
                                                          "typename", NULL);
@@ -2061,7 +2274,59 @@ void msWFSParseRequest(cgiRequestObj *request, wfsParamsObj *wfsparams)
                                 free(pszTmp);
                             }
                         }
+                        
+                        /*parse property name*/
+                        psPropertyName = CPLGetXMLNode(psQuery, "PropertyName");
+                        pszTmp2= NULL;
 
+                        /*when there is no PropertyName, we outpout () so that it is parsed properly in GetFeature*/
+                        if (psPropertyName == NULL)
+                          pszTmp2 = strdup("");
+
+                        while (psPropertyName)
+                        {
+                            if (!psPropertyName->pszValue || 
+                                strcasecmp(psPropertyName->pszValue, "PropertyName") != 0)
+                            {
+                                psPropertyName = psPropertyName->psNext;
+                                continue;
+                            }
+                            pszValue = (char *) CPLGetXMLValue(psPropertyName, NULL, NULL);
+                            if (pszTmp2 == NULL) {
+                                pszTmp2 = strdup(pszValue);
+                            } 
+                            else 
+                            {
+                                pszTmp = strdup(pszTmp2);
+                                pszTmp2 = (char *)realloc(pszTmp2, sizeof(char)* (strlen(pszTmp)+ strlen(pszValue)+2));
+                                sprintf(pszTmp2,"%s,%s",pszTmp, pszValue);
+                                msFree(pszTmp);
+                            }
+                            psPropertyName = psPropertyName->psNext;
+                        }
+                        if (pszTmp2)
+                        {
+                            pszTmp = strdup(pszTmp2);
+                            pszTmp2 = (char *)realloc(pszTmp2, sizeof(char)* (strlen(pszTmp)+ 3));
+                            sprintf(pszTmp2,"(%s)", pszTmp);
+                            msFree(pszTmp);
+
+                            if (wfsparams->pszPropertyName == NULL)
+                              wfsparams->pszPropertyName = strdup(pszTmp2);
+                            else
+                            {
+                                pszTmp = strdup(wfsparams->pszPropertyName);
+                                wfsparams->pszPropertyName =
+                                  (char *)realloc(wfsparams->pszPropertyName,
+                                                  sizeof(char)*(strlen(pszTmp)+ strlen(pszTmp2)+1));
+                                sprintf(wfsparams->pszPropertyName,"%s%s",wfsparams->pszPropertyName,
+                                        pszTmp2);
+                                msFree(pszTmp);
+                            }
+                            msFree(pszTmp2);
+                            pszTmp2 = NULL;
+                        }
+                            
                         /* parse filter */
                         psFilter = CPLGetXMLNode(psQuery, "Filter");
 
