@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: mapwfslayer.c 8754 2009-03-10 13:24:04Z dmorissette $
+ * $Id: mapwfslayer.c 9471 2009-10-16 17:04:04Z assefa $
  *
  * Project:  MapServer
  * Purpose:  Implementation of WFS CONNECTIONTYPE - client to WFS servers
@@ -37,7 +37,7 @@
 #include <process.h>
 #endif
 
-MS_CVSID("$Id: mapwfslayer.c 8754 2009-03-10 13:24:04Z dmorissette $")
+MS_CVSID("$Id: mapwfslayer.c 9471 2009-10-16 17:04:04Z assefa $")
 
 #define WFS_V_0_0_14  14
 #define WFS_V_1_0_0  100
@@ -331,7 +331,8 @@ static char *msBuildWFSLayerGetURL(mapObj *map, layerObj *lp, rectObj *bbox,
     
    
     if (strncmp(pszVersion, "0.0.14", 6) != 0 &&
-        strncmp(pszVersion, "1.0.0", 5) != 0 )
+        strncmp(pszVersion, "1.0.0", 5) != 0 &&
+         strncmp(pszVersion, "1.1", 3) != 0)
     {
         msSetError(MS_WFSCONNERR, "MapServer supports only WFS 1.0.0 or 0.0.14 (please verify the version metadata wfs_version).", "msBuildWFSLayerGetURL()");
         return NULL;
@@ -446,7 +447,7 @@ typedef struct ms_wfs_layer_info_t
     rectObj     rect;                     /* set by WhichShapes */
     char        *pszGetUrl;
     int         nStatus;           /* HTTP status */
-    int         bLayerOpened;      /* False until msWFSLayerOpen() is called*/
+    int         bLayerHasValidGML;  /* False until msWFSLayerWhichShapes() is called and determines the result GML is valid with features*/
 } msWFSLayerInfo;
 
 
@@ -466,7 +467,6 @@ static msWFSLayerInfo *msAllocWFSLayerInfo(void)
         psInfo->rect.miny = psInfo->rect.maxy = 0;
         psInfo->pszGetUrl = NULL;
         psInfo->nStatus = 0;
-        psInfo->bLayerOpened = MS_FALSE;
     }
     else
     {
@@ -779,6 +779,11 @@ int msWFSLayerOpen(layerObj *lp,
             (psInfo->pszGMLFilename && pszGMLFilename && 
              strcmp(psInfo->pszGMLFilename, pszGMLFilename) == 0) )
         {
+            if (lp->layerinfo == NULL)
+            {
+                if (msWFSLayerWhichShapes(lp, psInfo->rect) == MS_FAILURE)
+                  return MS_FAILURE;
+            }
             return MS_SUCCESS;  /* Nothing to do... layer is already opened */
         }
         else
@@ -840,7 +845,6 @@ int msWFSLayerOpen(layerObj *lp,
     if (msWFSLayerWhichShapes(lp, psInfo->rect) == MS_FAILURE)
         status = MS_FAILURE;
     
-    psInfo->bLayerOpened = MS_TRUE;
 
     return status;
 #else
@@ -904,6 +908,120 @@ int msWFSLayerInitItemInfo(layerObj *layer)
     return MS_SUCCESS;
 }
 
+/**********************************************************************
+ *                          msWFSLayerGetShape()
+ *
+ **********************************************************************/
+int msWFSLayerGetShape(layerObj *layer, shapeObj *shape, int tile, 
+                       long record)
+{
+#ifdef USE_WFS_LYR
+    msWFSLayerInfo* psInfo = NULL;
+
+    if(layer != NULL && layer->wfslayerinfo != NULL)
+    	psInfo = (msWFSLayerInfo*)layer->wfslayerinfo;
+    else
+    {
+	msSetError(MS_WFSERR, "Layer is not opened.", "msWFSLayerGetShape()");
+	return MS_FAILURE;
+    }
+
+    if(psInfo->bLayerHasValidGML)
+      return msOGRLayerGetShape(layer, shape, tile, record);
+    else
+    {
+          /* Layer is successful, but there is no data to process */
+        msFreeShape(shape);
+        shape->type = MS_SHAPE_NULL;
+        return MS_FAILURE;
+    }
+#else
+/* ------------------------------------------------------------------
+ * WFS CONNECTION Support not included...
+ * ------------------------------------------------------------------ */
+  msSetError(MS_WFSCONNERR, "WFS CLIENT CONNECTION support is not available.", 
+             "msWFSLayerGetShape()");
+  return(MS_FAILURE);
+#endif /* USE_WFS_LYR */
+
+}
+
+
+
+
+/**********************************************************************
+ *                          msWFSLayerGetNextShape()
+ *
+ **********************************************************************/
+int msWFSLayerNextShape(layerObj *layer, shapeObj *shape)
+{
+#ifdef USE_WFS_LYR
+    msWFSLayerInfo* psInfo = NULL;
+
+    if(layer != NULL && layer->wfslayerinfo != NULL)
+      psInfo = (msWFSLayerInfo*)layer->wfslayerinfo;
+    else
+    {
+	msSetError(MS_WFSERR, "Layer is not opened.", "msWFSLayerNextShape()");
+	return MS_FAILURE;
+    }
+
+    if(psInfo->bLayerHasValidGML)
+      return msOGRLayerNextShape(layer, shape);
+    else
+    {
+        /* Layer is successful, but there is no data to process */
+        msFreeShape(shape);
+        shape->type = MS_SHAPE_NULL;
+        return MS_FAILURE;
+    }
+#else
+/* ------------------------------------------------------------------
+ * WFS CONNECTION Support not included...
+ * ------------------------------------------------------------------ */
+  msSetError(MS_WFSCONNERR, "WFS CLIENT CONNECTION support is not available.", 
+             "msWFSLayerNextShape()");
+  return(MS_FAILURE);
+#endif /* USE_WFS_LYR */
+
+}
+
+/**********************************************************************
+ *                          msWFSLayerGetExtent()
+ *
+ **********************************************************************/
+int msWFSLayerGetExtent(layerObj *layer, rectObj *extent) 
+{
+#ifdef USE_WFS_LYR
+    msWFSLayerInfo* psInfo = NULL;
+
+    if(layer != NULL && layer->wfslayerinfo != NULL)
+      psInfo = (msWFSLayerInfo*)layer->wfslayerinfo;
+    else
+    {
+	msSetError(MS_WFSERR, "Layer is not opened.", "msWFSLayerGetExtent()");
+	return MS_FAILURE;
+    }
+
+    if(psInfo->bLayerHasValidGML)
+      return msOGRLayerGetExtent(layer, extent);
+    else
+    {
+        /* Layer is successful, but there is no data to process */
+        msSetError(MS_WFSERR, "Unable to get extents for this layer.", "msWFSLayerGetExtent()");
+        return MS_FAILURE;
+    }
+#else
+/* ------------------------------------------------------------------
+ * WFS CONNECTION Support not included...
+ * ------------------------------------------------------------------ */
+  msSetError(MS_WFSCONNERR, "WFS CLIENT CONNECTION support is not available.", 
+             "msWFSLayerGetExtent()");
+  return(MS_FAILURE);
+#endif /* USE_WFS_LYR */
+
+}
+
 /**********************************************************************
  *                          msWFSLayerGetItems()
  *
@@ -911,12 +1029,40 @@ int msWFSLayerInitItemInfo(layerObj *layer)
 
 int msWFSLayerGetItems(layerObj *layer)
 {
+#ifdef USE_WFS_LYR
     /* For now this method simply lets OGR parse the GML and figure the  */
     /* schema itself. */
     /* It could also be implemented to call DescribeFeatureType for */
     /* this layer, but we don't need to do it so why waste resources? */
 
-    return msOGRLayerGetItems(layer);
+    msWFSLayerInfo* psInfo = NULL;
+
+    if(layer != NULL && layer->wfslayerinfo != NULL)
+    	psInfo = (msWFSLayerInfo*)layer->wfslayerinfo;
+    else
+    {
+	msSetError(MS_WFSERR, "Layer is not opened.", "msWFSLayerGetItems()");
+	return MS_FAILURE;
+    }
+
+    if(psInfo->bLayerHasValidGML)
+        return msOGRLayerGetItems(layer);
+    else
+    {
+        /* Layer is successful, but there is no data to process */
+        layer->numitems = 0;
+	layer->items = NULL;
+        return MS_SUCCESS;
+    }
+#else
+/* ------------------------------------------------------------------
+ * WFS CONNECTION Support not included...
+ * ------------------------------------------------------------------ */
+  msSetError(MS_WFSCONNERR, "WFS CLIENT CONNECTION support is not available.", 
+             "msWFSLayerGetItems()");
+  return(MS_FAILURE);
+#endif /* USE_WFS_LYR */
+
 }
 
 /**********************************************************************
@@ -975,8 +1121,9 @@ int msWFSLayerWhichShapes(layerObj *lp, rectObj rect)
         msProjectRect(&(lp->map->latlon), &(lp->projection), &ext);
         if (!msRectOverlap(&rect, &ext))
         {
-            /* No overlap... nothing to do */
-            return MS_DONE;  /* No overlap. */
+            /* No overlap... nothing to do. If layer was never opened, go open it.*/
+            if (lp->layerinfo)
+              return MS_DONE;  /* No overlap. */
         }
     }
 
@@ -1084,7 +1231,10 @@ int msWFSLayerWhichShapes(layerObj *lp, rectObj rect)
         return status;
     
     status = msOGRLayerWhichShapes(lp, rect);
-
+   
+    /* Mark that the OGR Layer is valid */
+    psInfo->bLayerHasValidGML = MS_TRUE;
+    
     return status;
 #else
 /* ------------------------------------------------------------------
@@ -1182,18 +1332,15 @@ msWFSLayerInitializeVirtualTable(layerObj *layer)
     layer->vtable->LayerOpen = msWFSLayerOpenVT;
     layer->vtable->LayerIsOpen = msWFSLayerIsOpen;
     layer->vtable->LayerWhichShapes = msWFSLayerWhichShapes;
-    layer->vtable->LayerNextShape = msOGRLayerNextShape; /* yes, OGR */
-    layer->vtable->LayerGetShape = msOGRLayerGetShape; /* yes, OGR */
-
+    layer->vtable->LayerNextShape = msWFSLayerNextShape;
+    layer->vtable->LayerResultsGetShape = msWFSLayerGetShape; 
+    layer->vtable->LayerGetShape = msWFSLayerGetShape;
     layer->vtable->LayerClose = msWFSLayerClose;
     layer->vtable->LayerGetItems = msWFSLayerGetItems;
-    layer->vtable->LayerGetExtent = msOGRLayerGetExtent; /* yes, OGR */
+    layer->vtable->LayerGetExtent = msWFSLayerGetExtent;
     /* layer->vtable->LayerGetAutoStyle, use default */
-
     /* layer->vtable->LayerApplyFilterToLayer, use default */
-
     /* layer->vtable->LayerCloseConnection, use default */
-
     layer->vtable->LayerSetTimeFilter = msLayerMakePlainTimeFilter;
     /* layer->vtable->LayerCreateItems, use default */
     /* layer->vtable->LayerGetNumFeatures, use default */
