@@ -1,4 +1,5 @@
 /******************************************************************************
+ * $Id: shp2pdf.c 8489 2009-02-03 20:35:24Z tomkralidis $
  *
  * Project:  MapServer
  * Purpose:  Commandline shape to pdf converter.
@@ -28,7 +29,7 @@
 
 #include "mapserver.h"
 
-MS_CVSID("$Id: shp2pdf.c 6428 2007-07-31 19:58:45Z dmorissette $")
+MS_CVSID("$Id: shp2pdf.c 8489 2009-02-03 20:35:24Z tomkralidis $")
 
 #ifdef USE_PDF
 #include <pdflib.h>
@@ -72,6 +73,9 @@ int main(int argc, char *argv[])
   char **layers=NULL;
   int num_layers=0;
 
+  int layer_found=0;
+  char *invalid_layer=NULL;
+
   char *outfile=NULL; /* no -o sends image to STDOUT */
 
   /* A4 Portrait */
@@ -86,7 +90,14 @@ int main(int argc, char *argv[])
 
   /* ---- check the number of arguments, return syntax if not correct ---- */
   if( argc < 3 ) {
-    fprintf(stdout,"Syntax: shp2pdf -m [mapfile] -o [pdf] -l [layers]\n" );
+    fprintf(stdout, "\nPurpose: convert a mapfile to a PDF document\n\n");
+    fprintf(stdout,"Syntax: shp2pdf -m mapfile -o pdf [-e minx miny maxx maxy] [-l \"layer1 [layers2...]\"] [-t] [-d layername datavalue]\n" );
+    fprintf(stdout,"  -m mapfile: Map file to operate on - required\n" );
+    fprintf(stdout,"  -o pdf: output filename (stdout if not provided)\n");
+    fprintf(stdout,"  -e minx miny maxx maxy: extents to render\n");
+    fprintf(stdout,"  -l layers: layers to enable - make sure they are quoted and space seperated if more than one listed\n" );
+    fprintf(stdout,"  -t : turn on transparency\n");
+    fprintf(stdout,"  -d layername datavalue: change DATA value for a layer\n" );
     exit(0);
   }
 
@@ -144,18 +155,36 @@ int main(int argc, char *argv[])
     if(strncmp(argv[i],"-l",2) == 0) { /* load layer list */
       layers = msStringSplit(argv[i+1], ' ', &(num_layers));
 
-      for(j=0; j<map->numlayers; j++) {
-    if(GET_LAYER(map, j)->status == MS_DEFAULT)
-      continue;
-    else {
-      GET_LAYER(map, j)->status = MS_OFF;
-      for(k=0; k<num_layers; k++) {
-        if(strcmp(GET_LAYER(map, j)->name, layers[k]) == 0) {
-          GET_LAYER(map, j)->status = MS_ON;
-          break;
+      for(j=0; j<num_layers; j++) { /* loop over -l */
+        layer_found=0;
+        for(k=0; k<map->numlayers; k++) {
+          if(GET_LAYER(map, k)->name && strcmp(GET_LAYER(map, k)->name, layers[j]) == 0) {
+            layer_found=1;
+            break;
+          }
+          else {
+            invalid_layer = strdup(layers[j]);
+          }
+        }
+        if (layer_found==0) {
+          fprintf(stderr, "Layer (-l) %s not found\n", invalid_layer);
+          msCleanup();
+          exit(0);
         }
       }
-    }
+
+      for(j=0; j<map->numlayers; j++) {
+        if(GET_LAYER(map, j)->status == MS_DEFAULT)
+          continue;
+        else {
+          GET_LAYER(map, j)->status = MS_OFF;
+          for(k=0; k<num_layers; k++) {
+            if(strcmp(GET_LAYER(map, j)->name, layers[k]) == 0) {
+              GET_LAYER(map, j)->status = MS_ON;
+              break;
+            }
+          }
+        }
       }
       msFreeCharArray(layers, num_layers);
 
