@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: shptreetst.c 7418 2008-02-29 00:02:49Z nsavard $
+ * $Id: shptreetst.c 8774 2009-03-10 21:04:16Z tomkralidis $
  *
  * Project:  MapServer
  * Purpose:  Utility program to visualize a quadtree search
@@ -32,7 +32,7 @@
 #include <unistd.h>
 #include <stdlib.h>
 
-MS_CVSID("$Id: shptreetst.c 7418 2008-02-29 00:02:49Z nsavard $")
+MS_CVSID("$Id: shptreetst.c 8774 2009-03-10 21:04:16Z tomkralidis $")
 
 #ifdef SHPT_POLYGON
    #undef MAPSERVER
@@ -80,7 +80,7 @@ int main( int argc, char ** argv )
     rectObj	rect;
  
     int		pos;
-    char	*bitmap = NULL;
+    ms_bitarray bitmap = NULL;
      
     char	mBigEndian;
     treeNodeObj *node = NULL;
