@@ -1,38 +1,40 @@
 /******************************************************************************
-* $Id: tile4ms.c 6373 2007-07-24 02:37:18Z dmorissette $
-*
-* tile4ms.mc  
-*
-* Version 1.0
-* Author Herbie Freytag hfreytag@dlwc.nsw.gov.au
-*
-* Create shapefile of rectangles from extents of several shapefiles (=tiles)
-* Create DBF with file names for shape tiles, in column LOCATION as required
-* by mapserv.
-* For use with Mapserv tiling capability.
-* Issues: resulting shape files do not display in ArcView.
-*
-* Copyright (c) 1996-2005 Regents of the University of Minnesota.
-*
-* Permission is hereby granted, free of charge, to any person obtaining a
-* copy of this software and associated documentation files (the "Software"),
-* to deal in the Software without restriction, including without limitation
-* the rights to use, copy, modify, merge, publish, distribute, sublicense,
-* and/or sell copies of the Software, and to permit persons to whom the
-* Software is furnished to do so, subject to the following conditions:
-*
-* The above copyright notice and this permission notice shall be included in 
-* all copies of this Software or works derived from this Software.
-*
-* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-* THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
-* FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
-* DEALINGS IN THE SOFTWARE.
-******************************************************************************
-*
+ * $Id: tile4ms.c 7785 2008-07-03 23:47:28Z pramsey $
+ *
+ * Project:  MapServer
+ * Purpose:  Create shapefile of rectangles from extents of several shapefiles 
+ *           (=tiles)
+ *           Create DBF with file names for shape tiles, in column LOCATION as
+ *           required by mapserv.
+ *           For use with Mapserv tiling capability.
+ * Author:   Herbie Freytag hfreytag@dlwc.nsw.gov.au
+ *
+ * Note:
+ * Resulting shape files do not display in ArcView.
+ *
+ ******************************************************************************
+ * Copyright (c) 1996-2005 Regents of the University of Minnesota.
+ *
+ * Permission is hereby granted, free of charge, to any person obtaining a
+ * copy of this software and associated documentation files (the "Software"),
+ * to deal in the Software without restriction, including without limitation
+ * the rights to use, copy, modify, merge, publish, distribute, sublicense,
+ * and/or sell copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following conditions:
+ *
+ * The above copyright notice and this permission notice shall be included in 
+ * all copies of this Software or works derived from this Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
+ * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
+ * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+ * DEALINGS IN THE SOFTWARE.
+ *****************************************************************************/
+
+/*
 * This is a modified version of Herbie's program that works with MapServer's
 * shapelib and should be using the MapServer makefile.
 *
@@ -41,7 +43,7 @@
 #include "mapserver.h"
 #include <string.h>
 
-MS_CVSID("$Id: tile4ms.c 6373 2007-07-24 02:37:18Z dmorissette $")
+MS_CVSID("$Id: tile4ms.c 7785 2008-07-03 23:47:28Z pramsey $")
 
 /***********************************************************************/
 int process_shapefiles(char *metaFileNameP, char *tileFileNameP, 
@@ -93,9 +95,10 @@ int     tilesProcessed = 0;
   /* create new tileindex shapefiles and create a header */
   /* -------------------------------------------------- */
   sprintf(tileshapeName, "%s.shp", tileFileNameP);
+
   if(NULL==(tileSHP=msSHPCreate(tileFileNameP, SHP_POLYGON))) {
     fclose(metaFP);
-    printf("Unable to create %s.shp (.shx)\n", tileshapeName);
+    printf("Unable to create %s.shp (.shx)\n", tileFileNameP);
     return(1);
     }
 
